package com.singularity.aop;

import cn.dev33.satoken.stp.StpUtil;
import com.singularity.dao.LogsMapper;
import com.singularity.pojo.entity.Logs;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Date;

@Aspect
@Component
public class LogOperationAction {

    private final LogsMapper logsMapper;

    @Autowired
    public LogOperationAction(LogsMapper logsMapper) {
        this.logsMapper = logsMapper;
    }

    @AfterReturning("@annotation(LogOperation) && execution(* com.singularity.controller.*.*(..))")
    public void doAfter(JoinPoint jp) {
        // 方法签名
        MethodSignature sig = (MethodSignature) jp.getSignature();
        // 方法
        Method method = sig.getMethod();

        LogOperation logOperation = method.getAnnotation(LogOperation.class);

        // 获取请求上下文
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();

        // 获取token， 用户获取用户id
        String tokenHeader = request.getHeader("Authorization");
        // 日志内容
        String message = logOperation.message();

        //  通过token找到用户，通过用户信息和日志信息记录一条日志到数据库

        Logs logs = new Logs();
        String loginIdByToken = (String) StpUtil.getLoginIdByToken(tokenHeader);
        if(StringUtils.isNotEmpty(loginIdByToken)){
            int i = loginIdByToken.indexOf("-");
            Integer id = Integer.valueOf(loginIdByToken.substring(i + 1));
            logs.setUserId(id);
        }

        logs.setContent(message);
        logs.setCreatedAt(new Date());
        logs.setUpdatedAt(new Date());
        logsMapper.insert(logs);

    }


}
