package com.singularity.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.ThreadPoolExecutor;

@Configuration
public class ThreadPoolConfig {


    /**
     * cpu核心数
     */
    private static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();

    /**
     * 线程池核心线程
     */
    private static final int COUR_SIZE = CPU_COUNT * 2;

    /**
     * 线程池最大线程
     */
    private static final int MAX_COUR_SIZE = CPU_COUNT * 4;

    /**
     * 队列
     */
    private static final int QUEUE_SIZE = 100;

    @Bean("threadPool")
    public ThreadPoolTaskExecutor threadPoolTaskExecutor() {
        ThreadPoolTaskExecutor threadPoolTaskExecutor = new ThreadPoolTaskExecutor();
        // 设置核心线程数
        threadPoolTaskExecutor.setCorePoolSize(COUR_SIZE);
        // 配置最大线程数
        threadPoolTaskExecutor.setMaxPoolSize(MAX_COUR_SIZE);
        // 配置队列容量（这里设置成最大线程数的四倍）
        threadPoolTaskExecutor.setQueueCapacity(QUEUE_SIZE);
        // 给线程池设置名称
        threadPoolTaskExecutor.setThreadNamePrefix("thread-");
        // 设置任务的拒绝策略（谁提交任务谁来执行这个任务）
        threadPoolTaskExecutor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
        return threadPoolTaskExecutor;
    }
}
