package com.singularity.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.entity.TableChart;
import com.singularity.service.TableChartService;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@RestController
@Api(tags = "系统设置-图像管理")
@RequestMapping(value = "/api/chartManage")
public class ChartTableController {
    @Resource
    private TableChartService tableChartService;

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("增加图像表字段")
    @LogOperation(message = "增加图像表字段")
    @PostMapping("/insertTableChartData")
    public Result<?> insertTableChartData(@RequestBody @Validated List<TableChart> tableChartList){
        tableChartService.saveBatch(tableChartList);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("删除图像表字段")
    @LogOperation(message = "删除图像表字段")
    @PostMapping("/deleteTableChartData")
    public Result<?> deleteTableChartData(@RequestBody @Validated List<TableChart> tableChartList){
        tableChartService.removeBatchByIds(tableChartList);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("修改图像表字段")
    @LogOperation(message = "修改图像表字段")
    @PostMapping("/updateTableChartData")
    public Result<?> updateTableChartData(@RequestBody @Validated List<TableChart> tableChartList){
        tableChartService.updateBatchById(tableChartList);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("查询图像表字段")
    @LogOperation(message = "查询图像表字段")
    @PostMapping("/queryTableChartData")
    public Result<List<TableChart>> queryTableChartData(){
        LambdaQueryWrapper<TableChart> qw = new LambdaQueryWrapper<>();
        qw.eq(TableChart::getStat,1);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,tableChartService.list());
    }
}
