package com.singularity.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.dto.fieldManagement.QueryFormFieldDTO;
import com.singularity.pojo.dto.fieldManagement.QueryTableHeadDTO;
import com.singularity.pojo.entity.HeadField;
import com.singularity.pojo.entity.SysTableField;
import com.singularity.pojo.entity.TableHead;
import com.singularity.service.HeadFieldService;
import com.singularity.service.TableFieldService;
import com.singularity.service.TableHeadService;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Slf4j
@RestController
@Api(tags = "系统设置-管理表格相关接口")
@RequestMapping(value = "/api/fieldManagement")
public class FieldManagementController {
    @Resource
    private TableFieldService tableFieldService;

    @Resource
    private TableHeadService tableHeadService;

    @Resource
    private HeadFieldService headFieldService;


    @ApiOperationSupport(author = "windsky")
    @ApiOperation("增加表格字段")
    @LogOperation(message = "增加表格字段")
    @PostMapping("/insertFormField")
    public Result<?> insertFormField(@Validated @RequestBody SysTableField sysTableField){
        tableFieldService.save(sysTableField);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("修改表格字段")
    @LogOperation(message = "修改表格字段")
    @PostMapping("/updateFormField")
    public Result<?> updateFormField(@Validated @RequestBody SysTableField sysTableField){
        tableFieldService.updateById(sysTableField);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("删除表格字段")
    @LogOperation(message = "删除表格字段")
    @PostMapping("/deleteFormField")
    public Result<?> deleteFormField(@Validated @RequestBody List<SysTableField> sysTableFields){
        tableFieldService.removeBatchByIds(sysTableFields);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("查询表格字段")
    @PostMapping("/queryFormField")
    public Result<List<SysTableField>> queryFormField(@Validated @RequestBody QueryFormFieldDTO queryFormFieldDTO){
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,tableFieldService.queryTableFieldList(queryFormFieldDTO));
    }


    @ApiOperationSupport(author = "windsky")
    @ApiOperation("创建表格")
    @LogOperation(message = "创建表格")
    @PostMapping("/generateForm")
    public Result<?> generateForm(@Validated @RequestBody TableHead tableHead){
        Date date = new Date();
        tableHead.setCreateTime(date);
        tableHead.setUpdateTime(date);
        tableHead.setStat(true);
        tableHeadService.saveDatas(tableHead);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("删除表格")
    @LogOperation(message = "删除表格")
    @PostMapping("/deleteForm")
    public Result<?> deleteForm(@Validated @RequestBody List<TableHead> tableHeads){
        tableHeadService.deleteForm(tableHeads);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("修改表格属性")
    @LogOperation(message = "修改表格属性")
    @PostMapping("/updateForm")
    public Result<?> updateForm(@Validated @RequestBody TableHead tableHead){
        Date date = new Date();
        tableHead.setUpdateTime(date);
        tableHeadService.updateById(tableHead);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("查询表格")
    @PostMapping("/queryForm")
    public Result<List<TableHead>> queryForm(@Validated @RequestBody QueryTableHeadDTO queryTableHeadDTO){
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,tableHeadService.queryForm(queryTableHeadDTO));
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("为表格增加已有字段")
    @PostMapping("/insertExistingFormField")
    public Result<?> insertExistingFormField(@Validated @RequestBody HeadField headField){
        headFieldService.save(headField);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "windsky")
    @ApiOperation("为表格删除已有字段")
    @PostMapping("/deleteExistingFormField")
    public Result<?> deleteExistingFormField(@Validated @RequestBody List<HeadField> headFields){
        headFieldService.removeBatchByIds(headFields);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
