package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.exception.SaTokenException;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.DeleteAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryResourceFileListDTO;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.handing.*;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.ExperimentInfo;
import com.singularity.pojo.entity.TaskQueue;
import com.singularity.pojo.vo.attachment.QueryResourceFileListVO;
import com.singularity.pojo.vo.hading.*;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.ExperimentService;
import com.singularity.service.HandingService;
import com.singularity.service.TaskQueueService;
import com.singularity.util.FileUtil;
import com.singularity.util.TimedTaskUtil;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.UUID;


@Slf4j
@RestController
@Api(tags = "操作稳定性相关接口")
@RequestMapping(value = "/api/handing")
public class HandingController {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private HandingService handingService;

    @Resource
    private TimedTaskUtil timedTaskUtil;

    @Resource
    private ExperimentService experimentService;

    @Resource
    private TaskQueueService taskQueueService;

    @Value("${file.path}")
    private String filePath;

    @LogOperation(message = "上传操纵稳定性试验结果文件")
    @ApiOperation("批量上传操纵稳定性试验结果文件")
    @PostMapping(value = "/uploadHandingFile")
    public Result<?> uploadHandingFile(@Validated UploadHandingFileDTO dto) {
        MultipartFile multipartFile = dto.getMultipartFile();
        // 创建队列信息
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(dto.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(dto.getCarId());
        taskQueue.setStat(1);
        taskQueue.setOperId(dto.getOperId());
        taskQueueService.save(taskQueue);
        dto.setTaskId(taskQueue.getId());
        // 批量定时上传
        if(ObjectUtil.isNotEmpty(dto.getStartTime())){
            try {
                dto.setOriginalFilename(multipartFile.getOriginalFilename());
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
                FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
                dto.setLocalFilePath(fileNameAndPath);
                dto.setSize(multipartFile.getSize());
            } catch (IOException e) {
                throw new SaTokenException("临时保存文件失败！");
            }
            // 定时执行需要将文件临时存放到本地
            timedTaskUtil.startCron(
                    () -> handingService.uploadHandingFileList(dto),dto.getStartTime()
            );
        }else{
            // 批量上传
            handingService.uploadHandingFileList(dto);
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @LogOperation(message = "删除操纵稳定性数据文件")
    @ApiOperation("批量删除操作稳定性数据文件")
    @PostMapping(value = "/deleteHandingFile")
    public Result<?> deleteHandingFile(@Validated @RequestBody DeleteHandingFileDTO dto) {
        handingService.deleteHandingFile(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询操作稳定性数据文件列表")
    @PostMapping(value = "/queryHandingFileList")
    public Result<List<QueryHandingFileListVO>> queryHandingFileList(@Validated @RequestBody QueryHandingFileListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,attachmentService.queryHandingFileList(dto));
    }

    @ApiOperation("生成操作稳定性试验报告")
    @LogOperation(message = "生成操纵稳定性试验报告")
    @PostMapping(value = "/createHadingReport")
    public Result<?> createHadingReport(@Validated @RequestBody CreateHadingReportDTO dto) {
        handingService.createHadingReport(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询操作稳定性试验数据表")
    @PostMapping(value = "/queryHandingTableData")
    public Result<List<QueryTableDataVO>> queryHandingTableData(@Validated @RequestBody QueryTableDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,handingService.queryHandingTableData(dto));
    }

    @ApiOperation("对比操作稳定性试验数据表")
    @PostMapping(value = "/compareHandingTableData")
    public Result<CompareHandingTableDataVO> compareHandingTableData(@Validated @RequestBody CompareHandingTableDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,handingService.compareHandingTableData(dto));
    }

    @ApiOperation("修改操作稳定性试验数据表")
    @PostMapping(value = "/updateHandingTableData")
    public Result<?> updateHandingTableData(@Validated @RequestBody UpdateHandingTableDataDTO dto) {
        handingService.updateHandingTableData(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询操作稳定性图像数据")
    @PostMapping(value = "/querySnakeTestChartData")
    public Result<List<QueryHandingChartDataVO>> querySnakeTestChartData(@Validated @RequestBody QueryHandingChartDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,handingService.queryHandingChartData(dto));
    }

    @ApiOperation("对比操作稳定性图像数据")
    @PostMapping(value = "compareHandingChartData")
    public Result<List<QueryHandingChartDataVO>> compareHandingChartData(@Validated @RequestBody CompareHandingChartDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,handingService.compareHandingChartData(dto));
    }

    @ApiOperation("查询试验类别信息")
    @PostMapping(value = "/queryExpTypeInfo")
    public Result<List<QueryExpTypeInfoVO>> queryExpTypeInfo(@Validated @RequestBody QueryExpTypeInfoDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,handingService.queryExpTypeInfo(dto));
    }

    @ApiOperation("行驶性能操纵稳定性试验报告主列表分页查询")
    @PostMapping(value = "/queryHandingFileReportList")
    public Result<List<QueryAttachmentListVO>> queryHandingFileReportList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(dto.getBusinessId());
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }

    @ApiOperation("上传行驶性能操纵稳定性试验报告文件")
    @LogOperation(message = "上传行驶性能操纵稳定性试验报告文件")
    @PostMapping(value = "/uploadHandingFileReport")
    public Result<List<QueryAttachmentListVO>> uploadHandingFileReport(@Validated UploadAttachmentDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        dto.setBusinessType(FileBusinessConstant.HANDING_REPORT_FILE);
        attachmentService.uploadAttachment(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除行驶性能操纵稳定性试验报告文件")
    @LogOperation(message = "删除行驶性能操纵稳定性试验报告文件")
    @PostMapping(value = "/deleteHandingFileReport")
    public Result<List<QueryAttachmentListVO>> deleteHandingFileReport(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());

        // 删除试验场景信息
        LambdaQueryWrapper<ExperimentInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(ExperimentInfo::getAttachId,dto.getAttachIdList());
        experimentService.remove(queryWrapper);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("上传操纵稳定性原始数据文件")
    @LogOperation(message = "上传操纵稳定性原始数据文件")
    @PostMapping("/uploadReSourceFile")
    public Result<List<Attachment>> uploadReSourceFile(@Validated UploadReSourceFileDTO dto) {
        MultipartFile multipartFile = dto.getMultipartFile();
        // 创建队列信息
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(dto.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(dto.getOwnerId());
        taskQueue.setStat(1);
        taskQueue.setOperId(dto.getOperId());
        taskQueueService.save(taskQueue);
        dto.setTaskId(taskQueue.getId());
        // 批量定时上传
        if(ObjectUtil.isNotEmpty(dto.getStartTime())){
            try {
                dto.setOriginalFilename(multipartFile.getOriginalFilename());
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
                FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
                dto.setLocalFilePath(fileNameAndPath);
                dto.setSize(multipartFile.getSize());
            } catch (IOException e) {
                throw new SaTokenException("临时保存文件失败！");
            }
            // 定时执行需要将文件临时存放到本地
            timedTaskUtil.startCron(
                    () -> handingService.uploadReSourceFile(dto),dto.getStartTime()
            );
        }else{
            // 批量上传
            handingService.uploadReSourceFile(dto);
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("行驶性能操纵稳定性原始数据列表分页查询")
    @PostMapping(value = "/queryHandingResourceFileList")
    public Result<List<QueryResourceFileListVO>> queryHandingResourceFileList(@Validated @RequestBody QueryResourceFileListDTO dto) {
        dto.setBusinessId(dto.getBusinessId());
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryResourceFileList(dto));
    }

}
