package com.singularity.controller;

import cn.dev33.satoken.util.SaResult;
import cn.hutool.core.bean.BeanUtil;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.singularity.pojo.dto.permission.MenuDeleteDTO;
import com.singularity.pojo.dto.permission.MenuSetDTO;
import com.singularity.pojo.dto.permission.PermissionInfoDTO;
import com.singularity.pojo.dto.permission.queryMenuTrueListDTO;
import com.singularity.pojo.entity.Permission;
import com.singularity.pojo.param.permission.PermissionAddParam;
import com.singularity.pojo.param.permission.PermissionUpdateParam;
import com.singularity.pojo.vo.permission.QueryMenuDropListVO;
import com.singularity.pojo.vo.permission.QueryMenuTrueListVO;
import com.singularity.service.PermissionService;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api/permission")
@Api(tags = "权限相关")
public class PermissionController {

    private final PermissionService permissionService;

    @Autowired
    public PermissionController(PermissionService permissionService) {
        this.permissionService = permissionService;
    }

    @ApiOperation("获取所有权限")
    @RequestMapping(method = RequestMethod.GET,value = "/permissions/{status}")
    public SaResult getPermission(@PathVariable("status") Integer status) {
        List<PermissionInfoDTO> list = permissionService.getAllPermissionList(status);
        return new SaResult(SaResult.CODE_SUCCESS,"查询成功",list);
    }

    @ApiOperation("根据角色id获取权限")
    @RequestMapping(method = RequestMethod.GET,value = "/getByRoleId/{roleId}")
    public SaResult getPermissionByRoleId(@PathVariable("roleId") Long roleId) {
        List<PermissionInfoDTO> list = permissionService.getPermissionList(roleId);
        return new SaResult(SaResult.CODE_SUCCESS,"查询成功",list);
    }

    @ApiOperation("添加菜单")
    @RequestMapping(method = RequestMethod.POST,value = "/add")
    public SaResult insertPermission(@RequestBody PermissionAddParam permissionAddParam){
        if (permissionAddParam.getPid() == null){
            permissionAddParam.setPid(0L);
        }
        permissionAddParam.setStatus(1);
        Permission permission = BeanUtil.copyProperties(permissionAddParam, Permission.class);
        Boolean insert = permissionService.insert(permission);
        if (insert){
            return SaResult.ok("添加成功");
        }
        return SaResult.error("添加失败");
    }

    @ApiOperation("修改菜单")
    @RequestMapping(method = RequestMethod.POST,value = "/update")
    public SaResult updatePermission(@RequestBody PermissionUpdateParam permissionUpdateParam){
        Permission permission = BeanUtil.copyProperties(permissionUpdateParam, Permission.class);
        Boolean insert = permissionService.update(permission);
        if (insert){
            return SaResult.ok("修改成功");
        }
        return SaResult.error("修改失败");
    }

    @ApiOperation(value = "查询系统菜单树")
    @ApiOperationSupport(author = "windsky")
    @PostMapping("/queryMenuTrueList")
    public Result<List<QueryMenuTrueListVO>> queryMenuTrueList(@RequestBody queryMenuTrueListDTO queryMenuTrueListDTO){
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,permissionService.queryMenuTrueList(queryMenuTrueListDTO));
    }

    @ApiOperation(value = "查询菜单下拉列表")
    @PostMapping("/queryMenuDropList")
    public Result<List<QueryMenuDropListVO>> queryMenuDropList(){
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,permissionService.queryMenuDropList());
    }

    @ApiOperation(value = "新增菜单")
    @ApiOperationSupport(author = "windsky")
    @RequestMapping(method = RequestMethod.POST,value = "/insertMenu")
    public Result<?> insertMenu(@RequestBody MenuSetDTO menuSetDTO){
        permissionService.addMenu(menuSetDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation(value = "修改菜单")
    @ApiOperationSupport(author = "windsky")
    @RequestMapping(method = RequestMethod.POST,value = "/setMenu")
    public Result<?> setMenu(@RequestBody MenuSetDTO menuSetDTO){
        permissionService.setMenu(menuSetDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation(value = "删除菜单")
    @ApiOperationSupport(author = "windsky")
    @RequestMapping(method = RequestMethod.DELETE,value = "/deleteMenus")
    public Result<?> deleteMenus(@RequestBody MenuDeleteDTO menuSetDTO){
        permissionService.deleteMenu(menuSetDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
