package com.singularity.controller;

import cn.dev33.satoken.util.SaResult;
import com.singularity.pojo.dto.handing.QueryRideAbilityReportInfoDTO;
import com.singularity.pojo.dto.ride.CarTestMessageDTO;
import com.singularity.pojo.dto.ride.CompareTestResultDTO;
import com.singularity.pojo.dto.ride.CreateRideTestReportDTO;
import com.singularity.pojo.dto.ride.RideCarIdDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.ExperimentalResults;
import com.singularity.pojo.param.rideAbility.*;
import com.singularity.pojo.vo.attachment.AttachmentVO;
import com.singularity.pojo.vo.hading.QueryRideAbilityReportInfoVO;
import com.singularity.pojo.vo.ride.CarTestMessageVO;
import com.singularity.pojo.vo.ride.CompareTestResultVO;
import com.singularity.service.ExperimentalResultsService;
import com.singularity.service.RideAbilityService;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;

@Slf4j
@RestController
@Api(tags = "行驶性能相关接口")
@RequestMapping(value = "/api/rideAbility")
public class RideAbilityController {

    @Resource
    private  RideAbilityService rideAbilityService;

    @Resource
    private ExperimentalResultsService experimentalResultsService;

    @ApiOperation("上传实验结果文件")
    @PostMapping(value = "/exResultUpload")
    public SaResult exResultUpload(RideAbilityExReUploadFileParam rideAbilityExReUploadFileParam) {
        int result = rideAbilityService.exResultUpload(rideAbilityExReUploadFileParam);
        if(result>0) return SaResult.ok();
        return SaResult.error();
    }

    @ApiOperation("获取实验结果列表")
    @PostMapping(value = "/getExResult")
    public SaResult getExResult(@RequestBody RideAbilityExReParam rideAbilityExReParam) {
        List<ExReParam> list = rideAbilityService.getExResult(rideAbilityExReParam);
        return new SaResult(SaResult.CODE_SUCCESS, "操作成功", list);
    }

    @ApiOperation("修改实验结果列表(不使用)")
    @PostMapping(value = "/setExResult")
    public SaResult setExResult(@RequestBody SetExResultParam setExResultParam) throws IOException {
        AttachmentVO attachmentVO =rideAbilityService.setExResult(setExResultParam);
        return new SaResult(SaResult.CODE_SUCCESS, "操作成功", attachmentVO);
    }

    @ApiOperation("修改实验结果列表")
    @PostMapping(value = "/updateExResult")
    public SaResult updateExResult(@RequestBody UpdateExResultParam updateExResultParam){
        rideAbilityService.updateExResult(updateExResultParam);
        return SaResult.ok();
    }

    @ApiOperation("新增实验结果列表")
    @PostMapping(value = "/addExResult")
    public SaResult addExResult(@RequestBody ExperimentalResults experimentalResults){
        int result=rideAbilityService.addExResult(experimentalResults);
        if(result>0) return SaResult.ok();
        return SaResult.error();
    }



    @ApiOperation("实验结果上传图片")
    @PostMapping(value = "/exRePictureUpload")
    public SaResult exRePictureUpload(RideAbilityExRePictureUploadParam rideAbilityExReUploadFileParam) {
        List<Attachment> attachmentDTO=rideAbilityService.exRePictureUpload(rideAbilityExReUploadFileParam);
        return new SaResult(SaResult.CODE_SUCCESS, "操作成功", attachmentDTO);
    }

    @ApiOperation("获取图片")
    @PostMapping(value = "/getPicture")
    public SaResult getPicture(@RequestBody GetPictureParam getPictureParam) {
        List<AttachmentVO> attachmentVO =rideAbilityService.getPicture(getPictureParam);
        return new SaResult(SaResult.CODE_SUCCESS, "操作成功", attachmentVO);
    }

    @PostMapping("/createRideTestReport")
    @ApiOperation(value = "根据模板生成平顺性试验报告")
    public Result<?> createRideTestReport(@Validated @RequestBody CreateRideTestReportDTO dto) {
        experimentalResultsService.createRideTestReport(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @PostMapping("/compareTestResult")
    @ApiOperation(value = "对比试验结果")
    public Result<CompareTestResultVO> compareTestResult(@Validated @RequestBody CompareTestResultDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,experimentalResultsService.compareTestResult(dto));
    }

    @PostMapping("/queryTestMessage")
    @ApiOperation(value = "查询实验类别弹窗的信息")
    public Result<CarTestMessageVO> queryTestMessage(@Validated @RequestBody RideCarIdDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,experimentalResultsService.queryTestMessage(dto));
    }

    @PostMapping("/setTestMessage")
    @ApiOperation(value = "修改实验类别弹窗的信息")
    public Result<?> setTestMessage(@Validated @RequestBody CarTestMessageDTO dto) {
        experimentalResultsService.setTestMessage(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询行驶性能试验报告信息列表")
    @PostMapping(value = "/queryRideAbilityReportInfo")
    public Result<List<QueryRideAbilityReportInfoVO>> queryRideAbilityReportInfo(@Validated @RequestBody QueryRideAbilityReportInfoDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,rideAbilityService.queryRideAbilityReportInfo(dto));
    }
}
