package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.DeleteAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.damper.UploadDamperReportFileDTO;
import com.singularity.pojo.dto.slide.*;
import com.singularity.service.*;
import com.singularity.pojo.vo.slide.QuerySlideTestConditionListVO;
import com.singularity.pojo.vo.slide.QuerySlideTestResultListVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
@RequestMapping("/api/slide")
@Api(tags = "滑行相关接口")
public class SlideController {
    @Resource
    private AttachmentService attachmentService;

    @Resource
    private SlideTestResultService slideTestResultService;

    @Resource
    private SlideTestConditionService slideTestConditionService;

    @ApiOperation("行驶性能滑行原始数据文件主列表分页查询")
    @PostMapping(value = "/querySlideDataFileList")
    public Result<List<QueryAttachmentListVO>> querySlideDataFileList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(FileBusinessConstant.SLIDE_DATA_FILE);
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }

    @ApiOperation("上传滑行试验数据文件")
    @PostMapping(value = "/uploadSlideDataFile")
    public Result<?> uploadSlideDataFile(@Validated UploadSlideDataFileDTO dto) {
        slideTestResultService.uploadSlideDataFile(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除滑行试验数据文件")
    @PostMapping(value = "/deleteSlideDataFile")
    public Result<?> deleteSlideDataFile(@RequestBody @Validated DeleteSlideDataFileDTO dto) {
        slideTestResultService.deleteSlideDataFile(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询滑行试验条件列表")
    @PostMapping(value = "/querySlideTestConditionList")
    public Result<List<QuerySlideTestConditionListVO>> querySlideTestConditionList(@RequestBody @Validated QuerySlideTestConditionListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,slideTestConditionService.querySlideTestConditionList(dto));
    }

    @ApiOperation("编辑滑行试验条件")
    @PostMapping(value = "/updateSlideTestCondition")
    public Result<?> updateSlideTestCondition(@RequestBody @Validated UpdateSlideTestConditionDTO dto) {
        slideTestConditionService.updateSlideTestCondition(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询滑行道路载荷试验结果列表")
    @PostMapping(value = "/querySlideTestResultList")
    public Result<List<QuerySlideTestResultListVO>> querySlideTestResultList(@RequestBody @Validated QuerySlideTestResultListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,slideTestResultService.querySlideTestResultList(dto));
    }

    @ApiOperation("编辑滑行道路载荷试验结果列表")
    @PostMapping(value = "/updateSlideTestResult")
    public Result<?> updateSlideTestResult(@RequestBody @Validated UpdateSlideTestResultDTO dto) {
        slideTestResultService.updateSlideTestResult(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }


    @ApiOperation("上传滑行试验报告文件")
    @LogOperation(message = "上传滑行试验报告文件")
    @PostMapping(value = "/uploadSlideReportFile")
    public Result<?> uploadSlideReportFile(@Validated UploadDamperReportFileDTO dto) {
        // 文件名验重
        attachmentService.checkRepeatFileName(
                dto.getMultipartFile().getOriginalFilename(),
                dto.getCarId(),
                FileBusinessConstant.SLIDE_REPORT_FILE,
                AttachmentConstant.OWNER_TYPE_CAR
        );

        // 上传文件
        attachmentService.uploadAttachment(
                dto.getMultipartFile(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                FileBusinessConstant.SLIDE_REPORT_FILE,
                dto.getOperId()
        );
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("行驶性能滑行试验报告主列表分页查询")
    @PostMapping(value = "/querySlideReportFileList")
    public Result<List<QueryAttachmentListVO>> querySlideFileReportList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(FileBusinessConstant.SLIDE_REPORT_FILE);
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }

    @SaCheckRole("admin")
    @ApiOperation("删除行驶性能滑行试验报告文件")
    @LogOperation(message = "删除行驶性能滑行试验报告文件")
    @PostMapping(value = "/deleteSlideReportFile")
    public Result<List<QueryAttachmentListVO>> deleteSlideFileReport(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
