package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.exception.SaTokenException;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.aop.LogOperation;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.common.entity.PartChartData;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.DeleteAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.damper.QueryDamperCompareChartDataDTO;
import com.singularity.pojo.dto.spring.QuerySpringChartDataListDTO;
import com.singularity.pojo.dto.spring.QuerySpringCompareChartDataDTO;
import com.singularity.pojo.dto.spring.UploadSpringResourceFileDTO;
import com.singularity.pojo.dto.spring.UploadSpringResultFileDTO;
import com.singularity.pojo.entity.TaskQueue;
import com.singularity.pojo.vo.damper.QueryDamperCompareChartDataVO;
import com.singularity.pojo.vo.spring.QuerySpringCompareChartDataVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.DamperChartDataService;
import com.singularity.service.SpringService;
import com.singularity.service.TaskQueueService;
import com.singularity.util.FileUtil;
import com.singularity.util.TimedTaskUtil;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

@Api("零部件-弹簧相关接口")
@RestController
@RequestMapping("/api/spring")
public class SpringController {

    @Resource
    private TaskQueueService taskQueueService;

    @Resource
    private TimedTaskUtil timedTaskUtil;

    @Resource
    private SpringService springService;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private DamperChartDataService damperChartDataService;

    @Value("${file.path}")
    private String filePath;

    @LogOperation(message = "上传弹簧原始数据文件")
    @ApiOperation("上传弹簧原始数据文件")
    @PostMapping("/uploadSpringResourceFile")
    public Result<?> uploadSpringResourceFile(@Validated UploadSpringResourceFileDTO dto){
        // 创建队列信息
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(dto.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(dto.getOwnerId());
        taskQueue.setStat(1);
        taskQueue.setOperId(dto.getOperId());
        taskQueueService.save(taskQueue);
        dto.setTaskId(taskQueue.getId());
        if(ObjectUtil.isNotEmpty(dto.getStartTime())){
            try {
                MultipartFile multipartFile = dto.getMultipartFile();
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
                FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
                dto.setLocalFilePath(fileNameAndPath);
            } catch (IOException e) {
                throw new SaTokenException("临时保存文件失败！");
            }
            // 定时执行需要将文件临时存放到本地
            timedTaskUtil.startCron(() -> springService.uploadSpringResourceFile(dto),dto.getStartTime());
        }else{
            springService.uploadSpringResourceFile(dto);
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @LogOperation(message = "上传弹簧试验结果文件")
    @ApiOperation("上传弹簧试验结果文件")
    @PostMapping("/uploadSpringResultFile")
    public Result<?> uploadSpringResultFile(@Validated UploadSpringResultFileDTO dto){
        // 创建队列信息
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(dto.getMultipartFile().getOriginalFilename());
        taskQueue.setCarId(dto.getOwnerId());
        taskQueue.setStat(1);
        taskQueue.setOperId(dto.getOperId());
        taskQueueService.save(taskQueue);
        dto.setTaskId(taskQueue.getId());
        if(ObjectUtil.isNotEmpty(dto.getStartTime())){
            try {
                MultipartFile multipartFile = dto.getMultipartFile();
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
                FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
                dto.setLocalFilePath(fileNameAndPath);
            } catch (IOException e) {
                throw new SaTokenException("临时保存文件失败！");
            }
            // 定时执行需要将文件临时存放到本地
            timedTaskUtil.startCron(() -> springService.uploadSpringResultFile(dto),dto.getStartTime());
        }else{
            springService.uploadSpringResultFile(dto);
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询弹簧试验曲线数据")
    @PostMapping(value = "/querySpringChartDataList")
    public Result<List<ChartDataModel>> querySpringChartDataList(@RequestBody @Validated QuerySpringChartDataListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,springService.querySpringChartDataList(dto));
    }

    @ApiOperation("查询弹簧对比曲线数据")
    @PostMapping(value = "/querySpringCompareChartData")
    public Result<List<QuerySpringCompareChartDataVO>> querySpringCompareChartData(@RequestBody @Validated QuerySpringCompareChartDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,springService.querySpringCompareChartData(dto));
    }


    @ApiOperation("上传弹簧试验报告文件")
    @LogOperation(message = "上传弹簧试验报告文件")
    @PostMapping(value = "/uploadSpringReportFile")
    public Result<List<QueryAttachmentListVO>> uploadSpringReportFile(@Validated UploadAttachmentDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        dto.setBusinessType(FileBusinessConstant.SPRING_REPORT_FILE);
        attachmentService.uploadAttachment(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("行驶性能弹簧文件列表分页查询")
    @PostMapping(value = "/querySpringFileList")
    public Result<List<QueryAttachmentListVO>> querySpringFileList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(dto.getBusinessId());
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }

    @SaCheckRole("admin")
    @ApiOperation("删除行驶性能弹簧试验报告文件")
    @LogOperation(message = "删除行驶性能弹簧试验报告文件")
    @PostMapping(value = "/deleteSpringReportFile")
    public Result<?> deleteSpringReportFile(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除行驶性能弹簧原始数据文件")
    @LogOperation(message = "删除行驶性能弹簧原始数据文件")
    @PostMapping(value = "/deleteSpringResourceFile")
    public Result<?> deleteSpringResourceFile(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @LogOperation(message = "删除行驶性能弹簧试验结果文件")
    @ApiOperation("删除行驶性能弹簧试验结果文件")
    @PostMapping(value = "/deleteSpringResultFile")
    public Result<?> deleteSpringResultFile(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        // 删除文件
        attachmentService.removeByIds(dto.getAttachIdList());
        // 删除绘图数据
        LambdaQueryWrapper<PartChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(PartChartData::getAttachId,dto.getAttachIdList());
        damperChartDataService.remove(queryWrapper);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
