package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.DeleteAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.steering.DeleteSteeringDataFileDTO;
import com.singularity.pojo.dto.steering.QuerySteeringDataDTO;
import com.singularity.pojo.dto.steering.QuerySteeringDataFileListDTO;
import com.singularity.pojo.dto.steering.UploadSteeringDateFileDTO;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.service.AttachmentService;
import com.singularity.service.SteeringChartDataService;
import com.singularity.pojo.vo.steering.QuerySteeringDataFileListVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@Slf4j
@Api(tags = "转向相关接口")
@RestController
@RequestMapping("api/steering")
public class SteeringController {

    @Resource
    private SteeringChartDataService steeringChartDataService;

    @Resource
    private AttachmentService attachmentService;

    @ApiOperation("上传转向试验数据文件")
    @LogOperation(message = "上传转向试验数据文件")
    @PostMapping("/uploadSteeringDateFile")
    public Result<?> uploadSteeringDateFile(@Validated UploadSteeringDateFileDTO dto) {
        steeringChartDataService.uploadSteeringDateFile(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询转向试验曲线数据")
    @PostMapping("/querySteeringData")
    public Result<List<ChartDataModel>> querySteeringData(@RequestBody @Validated QuerySteeringDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, steeringChartDataService.querySteeringData(dto));
    }

    @ApiOperation("查询转向试验数据文件列表")
    @PostMapping(value = "/querySteeringDateFileList")
    public Result<List<QuerySteeringDataFileListVO>> querySteeringDateFileList(@RequestBody @Validated QuerySteeringDataFileListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.querySteeringDataFileList(dto));
    }

    @SaCheckRole("admin")
    @ApiOperation("删除制动性试验数据文件")
    @LogOperation(message = "删除制动性试验数据文件")
    @PostMapping("/deleteSteeringFileList")
    public Result<?> deleteSteeringFileList(@Validated @RequestBody DeleteSteeringDataFileDTO dto) {
        steeringChartDataService.deleteSteeringFileList(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }


    // 试验报告
    @ApiOperation("行驶性能转向试验报告主列表分页查询")
    @PostMapping(value = "/querySteeringReportFileList")
    public Result<List<QueryAttachmentListVO>> querySteeringReportFileList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(FileBusinessConstant.STEERING_REPORT_FILE);
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }

    @ApiOperation("上传行驶性能转向试验报告")
    @LogOperation(message = "上传行驶性能制动性试验报告")
    @PostMapping(value = "/uploadSteeringFileReport")
    public Result<?> uploadSteeringFileReport(@Validated UploadAttachmentDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        dto.setBusinessType(FileBusinessConstant.STEERING_REPORT_FILE);
        attachmentService.uploadAttachment(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除行驶性能转向试验报告")
    @LogOperation(message = "删除行驶性能制动性试验报告")
    @PostMapping(value = "/deleteSteeringFileReport")
    public Result<?> deleteBarkingFileReport(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
