package com.singularity.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.pojo.dto.template.GetOneTemplateDTO;
import com.singularity.pojo.dto.template.GetTempURLByBusinessTypeDTO;
import com.singularity.pojo.dto.template.QueryTempAttachmentListDTO;
import com.singularity.pojo.dto.template.UploadTemplateDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.vo.template.GetOneTemplateVO;
import com.singularity.pojo.vo.template.GetTempURLByBusinessTypeVO;
import com.singularity.pojo.vo.template.QueryTempAttachmentListVO;
import com.singularity.service.AttachmentService;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/api/template")
@Api(tags = "模板管理相关接口")
public class TemplateController {

    @Resource
    private AttachmentService attachmentService;

    @ApiOperation("批量上传模板文件")
    @PostMapping(value = "/uploadTemplate")
    public Result<?> uploadTemplate(@Validated UploadTemplateDTO dto) {
        attachmentService.uploadAttachmentList(dto.getMultipartFileList(),dto.getBusinessType());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("根据业务类型查询模板列表")
    @PostMapping(value = "/getTempURLByBusinessType")
    public Result<List<GetTempURLByBusinessTypeVO>> getTempURLByBusinessType(
            @Validated @RequestBody GetTempURLByBusinessTypeDTO dto) {
        List<Attachment> attachmentList;
        if(dto.getBusinessType() != null){
            attachmentList = attachmentService.getAttachment(dto.getBusinessType());
        }else {
            attachmentList = attachmentService.getAttachment(dto.getBusinessTypeList());
        }
        List<GetTempURLByBusinessTypeVO> result = new ArrayList<>();
        for (Attachment attachment : attachmentList) {
            GetTempURLByBusinessTypeVO getTempURLByBusinessTypeVO = new GetTempURLByBusinessTypeVO();
            getTempURLByBusinessTypeVO.setObjName(attachment.getObjName());
            getTempURLByBusinessTypeVO.setTempName(attachment.getName());
            getTempURLByBusinessTypeVO.setPath(attachment.getPath());
            result.add(getTempURLByBusinessTypeVO);
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,result);
    }

    @ApiOperation("模板文件列表主查询")
    @PostMapping(value = "/queryTempAttachmentList")
    public Result<List<QueryTempAttachmentListVO>> queryTempAttachmentList(@Validated @RequestBody QueryTempAttachmentListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryTempAttachmentList(dto));
    }

    @ApiOperation("根据业务类型查询唯一模板")
    @PostMapping(value = "/getOneTemplate")
    public Result<GetOneTemplateVO> getOneTemplate(@Validated @RequestBody GetOneTemplateDTO dto) {
        List<Attachment> attachmentList = attachmentService.getAttachment(dto.getBusinessId());
        if(CollectionUtil.isEmpty(attachmentList)){
            throw new ToolsBusinessWarn("请先上传该模板文件！");
        }
        GetOneTemplateVO getOneTemplateVO = new GetOneTemplateVO();
        BeanUtil.copyProperties(attachmentList.get(0),getOneTemplateVO);
        getOneTemplateVO.setTempName(attachmentList.get(0).getName());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,getOneTemplateVO);
    }

}
