package com.singularity.impl;

import cn.dev33.satoken.stp.StpInterface;
import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.singularity.dao.PermissionMapper;
import com.singularity.dao.RoleMapper;
import com.singularity.dao.RolePermissionMapper;
import com.singularity.dao.UserRoleMapper;
import com.singularity.pojo.entity.Permission;
import com.singularity.pojo.entity.Role;
import com.singularity.pojo.entity.RolePermission;
import com.singularity.pojo.entity.UserRole;
import com.singularity.util.RedisLettuceUtil;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class StpInterfaceImpl implements StpInterface {

    @Resource
    RolePermissionMapper rolePermissionMapper;

    @Resource
    private UserRoleMapper userRoleMapper;

    @Resource
    PermissionMapper permissionMapper;

    @Resource
    RoleMapper roleMapper;

    @Resource
    private RedisLettuceUtil redisLettuceUtil;

    @Override
    public List<String> getPermissionList(Object loginId, String loginType) {
        String[] spiltStr = loginId.toString().split("-");
        Integer userId = Integer.parseInt(spiltStr[spiltStr.length - 1]);
        // 从redis缓存中获取权限集合
        String jsonString = redisLettuceUtil.get(userId.toString() + "_permission");
        List<String> permissionList = JSONObject.parseArray(jsonString, String.class);
        if(CollectionUtil.isNotEmpty(permissionList))return permissionList;
        List<UserRole> roleIds = userRoleMapper.selectList(
                new LambdaQueryWrapper<UserRole>()
                .eq(UserRole::getUserId,userId));
        List<Long> rolesList = roleIds.stream().map(UserRole::getRoleId).collect(Collectors.toList());
        if (!(rolesList.size() >0)) return null;
        List<String> list = new ArrayList<>();
        rolesList.forEach(roleId ->{
            // 根据角色id从角色权限表中获取权限id
            List<RolePermission> rolePermissions = rolePermissionMapper.selectList(new LambdaQueryWrapper<RolePermission>().
                    eq(RolePermission::getRoleId, roleId));
            // 根据权限id从权限表中获取权限名称
            rolePermissions.forEach(permissionsId->{
                Permission permissions = permissionMapper.selectById(permissionsId.getPermissionId());
                list.add(permissions.getName());
            });
        });

        // 更新缓存
        jsonString = JSON.toJSONString(list);
        redisLettuceUtil.set(userId + "_permission",jsonString);
        return list;
    }

    /**
     * 返回一个账号所拥有的角色标识集合 (权限与角色可分开校验)
     */
    @Override
    public List<String> getRoleList(Object loginId, String loginType) {
        String[] spiltStr = loginId.toString().split("-");
        Integer userId = Integer.parseInt(spiltStr[spiltStr.length - 1]);
        // 从redis缓存中获取角色集合
        String jsonString = redisLettuceUtil.get(userId.toString() + "_role");
        List<String> permissionList = JSONObject.parseArray(jsonString, String.class);
        if(CollectionUtil.isNotEmpty(permissionList))return permissionList;
        List<UserRole> roleIds = userRoleMapper.selectList(
                new LambdaQueryWrapper<UserRole>()
                .eq(UserRole::getUserId,userId));
        List<String> list = new ArrayList<>();

        if (!(roleIds.size() >0)){
//            用户没有分配角色
            return null;
        }
        roleIds.forEach(roleId ->{
            Role byId = roleMapper.selectById(roleId.getRoleId());
            list.add(byId.getName());
        });

        // 更新缓存
        jsonString = JSON.toJSONString(list);
        redisLettuceUtil.set(userId + "_role",jsonString);

        return list;



    }

}
