package com.singularity.listener;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.excel.util.ListUtils;
import com.singularity.pojo.entity.CarKeyIndex;
import com.singularity.pojo.param.kc.CarKeyIndexParam;
import com.singularity.service.impl.CarKeyIndexServiceImpl;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
public class CarKeyIndexListener implements ReadListener<CarKeyIndexParam> {

    /**
     * 单次最大读取量
     */
    private static final int BATCH_COUNT = 50;

    /**
     * 缓存的数据
     */
    private List<CarKeyIndexParam> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);

    /**
     * 所属文件的id
     */
    private final Long id;


    private final CarKeyIndexServiceImpl carKeyIndexServiceImpl;

    public CarKeyIndexListener(Long id, CarKeyIndexServiceImpl carKeyIndexServiceImpl) {
        this.id = id;
        this.carKeyIndexServiceImpl=carKeyIndexServiceImpl;
    }


    @Override
    public void invoke(CarKeyIndexParam carKeyIndexParam, AnalysisContext analysisContext) {
        cachedDataList.add(carKeyIndexParam);
        // 需要存储一次数据库
        if (cachedDataList.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        saveData();
    }


    /**
     * 保存到数据库
     */
    private void saveData() {
        List<CarKeyIndex> carKeyIndices = BeanUtil.copyToList(cachedDataList, CarKeyIndex.class);
        carKeyIndices.forEach(item -> item.setId(id));
        log.info("{}条数据，开始存储数据库！", cachedDataList.size());
        int i = carKeyIndexServiceImpl.saveData(carKeyIndices);
        if(i<0) throw new RuntimeException("数据错误");
        log.info("存储数据库成功！");
    }


}
