package com.singularity.listener;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.fastjson.JSONObject;
import com.singularity.pojo.entity.TableData;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Data
@Slf4j
public class HandingReadFileListener implements ReadListener<Map<Integer,String>> {

    // 创建list集合封装最终的数据
    private List<TableData> tableDataList = new ArrayList<>();

    // 数据文件id
    private Long carId;

    // 表头id
    private Integer tableHeadId;

    // 表头id
    private Integer attachId;

    // 结束行
    private Integer endCol;

    // 上一次不包含nullMap
    Map<Integer,String> lastMap = new HashMap<>();

    private final LocalDateTime localDate = LocalDateTime.now();

    public HandingReadFileListener(Long carId,Integer tableHeadId,Integer attachId,Integer endCol){
        this.carId = carId;
        this.tableHeadId = tableHeadId;
        this.attachId = attachId;
        this.endCol = endCol;
    }

    @Override
    public void invoke(Map<Integer,String> map, AnalysisContext context) {
        int currentRow = context.readRowHolder().getRowIndex() + 1;
        if(ObjectUtil.isNotNull(endCol) && currentRow > endCol){
            return;
        }
        if(!map.containsValue(null)){
            lastMap = map;
        }else{ // 存在null则将上一次的数据插入
            for (Integer key : lastMap.keySet()) {
                if(map.get(key) == null)
                    map.put(key,lastMap.get(key));
            }
        }
        TableData tableData = new TableData();
        tableData.setRowInfo(JSONObject.toJSONString(map));
        tableData.setCarId(carId);
        tableData.setUpdateTime(localDate);
        tableData.setTableHeadId(tableHeadId);
        tableData.setCreateTime(localDate);
        tableData.setAttachId(attachId);
        tableDataList.add(tableData);
    }


    // 读取完成后执行
    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        log.info("excel读取完毕");
    }

}
