package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.pojo.entity.RideData;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Data
@Slf4j
public class RideReadPulseDataListener implements ReadListener<Map<Integer,String>> {

    private List<RideData> rideDataList = new ArrayList<>();

    private Integer attachId;

    private Integer testType;

    private Integer firstCarSpeed;

    private Integer secondCarSpeed;

    private LocalDateTime localDateTime = LocalDateTime.now();

    public RideReadPulseDataListener(Integer attachId,Integer testType){
        this.attachId = attachId;
        this.testType = testType;
    }

    @Override
    public void invoke(Map<Integer,String> map, AnalysisContext context) {
        int currentRowNum = context.getCurrentRowNum();
        if(currentRowNum == 1 || currentRowNum == 17 || currentRowNum == 33){
            firstCarSpeed = Integer.parseInt(map.get(0).replaceAll("\\D", ""));
            secondCarSpeed = Integer.parseInt(map.get(8).replaceAll("\\D", ""));
            return;
        }
        RideData rideData = new RideData();
        rideData.setTestIndex(map.get(0));
        rideData.setFirstData(Integer.parseInt(map.get(1)));
        rideData.setSecondData(Integer.parseInt(map.get(2)));
        rideData.setThirdData(Integer.parseInt(map.get(3)));
        rideData.setFirthData(Integer.parseInt(map.get(4)));
        rideData.setFifthData(Integer.parseInt(map.get(5)));
        rideData.setAverageData(Double.parseDouble(map.get(6)));
        rideData.setCarSpeed(firstCarSpeed);
        rideData.setAttachId(attachId);
        rideData.setTestType(testType);
        rideData.setCreateTime(localDateTime);
        rideData.setUpdateTime(localDateTime);
        rideDataList.add(rideData);

        RideData rideData2 = new RideData();
        rideData2.setTestIndex(map.get(8));
        rideData2.setFirstData(Integer.parseInt(map.get(9)));
        rideData2.setSecondData(Integer.parseInt(map.get(10)));
        rideData2.setThirdData(Integer.parseInt(map.get(11)));
        rideData2.setFirthData(Integer.parseInt(map.get(12)));
        rideData2.setFifthData(Integer.parseInt(map.get(13)));
        rideData2.setAverageData(Double.parseDouble(map.get(14)));
        rideData2.setCarSpeed(secondCarSpeed);
        rideData2.setAttachId(attachId);
        rideData2.setTestType(testType);
        rideData2.setCreateTime(localDateTime);
        rideData2.setUpdateTime(localDateTime);
        rideDataList.add(rideData2);
    }


    // 读取完成后执行
    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        log.info("excel读取完毕");
    }
}
