package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.pojo.entity.RideData;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Data
@Slf4j
public class RideReadRandomDataListener implements ReadListener<Map<Integer, String>> {

    private List<RideData> rideDataList = new ArrayList<>();

    private Integer attachId;

    private Integer testType;

    private Map<Integer, String> carSpeedMap;

    private LocalDateTime localDateTime = LocalDateTime.now();

    public RideReadRandomDataListener(Integer attachId, Integer testType) {
        this.attachId = attachId;
        this.testType = testType;
    }

    @Override
    public void invoke(Map<Integer, String> map, AnalysisContext context) {
        int currentRowNum = context.getCurrentRowNum();
        if (currentRowNum == 1) {
            carSpeedMap = map;
            return;
        }
        for (Integer key : map.keySet()) {
            if (key == 0) continue;
            RideData rideData = new RideData();
            rideData.setTestIndex(map.get(0));
            rideData.setFirstData(Integer.parseInt(map.get(key)));
            rideData.setAverageData(Double.parseDouble(map.get(key)));
            rideData.setCarSpeed(Integer.parseInt(carSpeedMap.get(key)));
            rideData.setAttachId(attachId);
            rideData.setTestType(testType);
            rideData.setCreateTime(localDateTime);
            rideData.setUpdateTime(localDateTime);
            rideDataList.add(rideData);
        }
    }


    // 读取完成后执行
    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        log.info("excel读取完毕");
    }
}
