package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.pojo.entity.SlideTestResult;
import com.singularity.service.SlideTestResultService;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
public class SlideTestResultListener implements ReadListener<Map<Integer, String>> {

    private final SlideTestResultService slideTestResultService;

    // 附件id
    private final Long attachId;

    // 车辆id
    private final Long carId;

    // 当前时间
    LocalDateTime localDateTime = LocalDateTime.now();

    // 减振器数据集合
    List<SlideTestResult> slideTestResultList = new ArrayList<>();

    public SlideTestResultListener(SlideTestResultService slideTestResultService, Long attachId, Long carId) {
        this.slideTestResultService = slideTestResultService;
        this.attachId = attachId;
        this.carId = carId;
    }

    @Override
    public void invoke(Map<Integer, String> data, AnalysisContext context) {
        SlideTestResult slideTestResult = new SlideTestResult();
        slideTestResult.setCarId(carId);
        slideTestResult.setAttachId(attachId);
        slideTestResult.setAt(Double.parseDouble(data.get(0)));
        slideTestResult.setBt(Double.parseDouble(data.get(1)));
        slideTestResult.setCt(Double.parseDouble(data.get(2)));
        slideTestResult.setCreateTime(localDateTime);
        slideTestResult.setUpdateTime(localDateTime);
        slideTestResultList.add(slideTestResult);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 滑行道路载荷试验结果
        slideTestResultService.saveBatch(slideTestResultList);
    }
}