package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.pojo.entity.SubjectiveEvaluation;
import com.singularity.pojo.param.subjectiveEvaluation.SubjectiveEvaluationCommodityExcelParam;
import com.singularity.service.SubjectiveEvaluationService;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

@Slf4j
public class SubjectiveEvaluationCommodityListener implements ReadListener<SubjectiveEvaluationCommodityExcelParam> {

    /**
     * 缓存的数据
     */
    private List<SubjectiveEvaluation> subjectiveEvaluationList = new ArrayList<>();

    /**
     * 所属车辆的id
     */
    private final Long carId;

    /**
     * 文件类型(0:商务车 1:乘用车)
     */
    private final Integer reviewType;


    private final SubjectiveEvaluationService subjectiveEvaluationService;

    private Long gradeOneSerial;

    private Long gradeTwoSerial;

    private Long serialNumber = 1L;

    private String modelReviews;


    public SubjectiveEvaluationCommodityListener(Long carId, Integer reviewType, SubjectiveEvaluationService subjectiveEvaluationService) {
        this.carId = carId;
        this.reviewType = reviewType;
        this.subjectiveEvaluationService = subjectiveEvaluationService;
    }


    @Override
    public void invoke(SubjectiveEvaluationCommodityExcelParam subjectiveEvaluationCommodityExcelParam, AnalysisContext analysisContext) {
        SubjectiveEvaluation subjectiveEvaluation = SubjectiveEvaluation.builder().carId(carId).reviewType(reviewType).build();

        if (subjectiveEvaluationCommodityExcelParam.getRow1() != null) {

            subjectiveEvaluation.setTestName(subjectiveEvaluationCommodityExcelParam.getRow2());
            subjectiveEvaluation.setGrade(1L);
            subjectiveEvaluation.setSerialNumber(serialNumber);
            gradeOneSerial = serialNumber++;
            gradeTwoSerial = null;


        } else if (subjectiveEvaluationCommodityExcelParam.getRow4() != null) {
            subjectiveEvaluation.setTestName(subjectiveEvaluationCommodityExcelParam.getRow4());
            subjectiveEvaluation.setGrade(3L);
            subjectiveEvaluation.setSerialNumber(serialNumber++);
            if (gradeTwoSerial == null)
                subjectiveEvaluation.setBelongSerial(gradeOneSerial);
            else
                subjectiveEvaluation.setBelongSerial(gradeTwoSerial);

            if (subjectiveEvaluationCommodityExcelParam.getRow5() != null)
                modelReviews = subjectiveEvaluationCommodityExcelParam.getRow5();

            subjectiveEvaluation.setModelReviews(modelReviews);
        } else {
            subjectiveEvaluation.setTestName(subjectiveEvaluationCommodityExcelParam.getRow3());
            subjectiveEvaluation.setGrade(2L);
            subjectiveEvaluation.setSerialNumber(serialNumber);
            subjectiveEvaluation.setBelongSerial(gradeOneSerial);
            gradeTwoSerial = serialNumber++;

        }
        subjectiveEvaluationList.add(subjectiveEvaluation);

    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
       saveData();

    }


    /**
     * 保存到数据库
     */
    private void saveData() {
        subjectiveEvaluationService.saveBatch(subjectiveEvaluationList);
    }


}
