package com.singularity.pojo.dto.ride;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.constraints.Future;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

@Data
@ApiModel(value = "UploadRideFileDTO",description = "上传平顺性试验数据文件入参")
public class UploadRideFileDTO {

    @NotNull(message = "请选择您需要上传的附件")
    @ApiModelProperty(value = "平顺性试验数据文件，格式为.xlsx .xls .csv",required = true)
    private MultipartFile multipartFile;

    @NotNull(message = "车辆id不能为空")
    @ApiModelProperty(value = "车辆id",required = true)
    private Long carId;

    @NotNull(message = "操作人id不能为空")
    @ApiModelProperty(value = "操作人id",required = true)
    private Integer operId;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @ApiModelProperty(value = "开始时间")
    @Future(message = "任务开始时间应该在当前时间之后")
    private Date startTime;

    @ApiModelProperty(value = "任务id,后端参数，前端忽略",hidden = true)
    private Integer taskId;

    @ApiModelProperty(value = "本地文件路径,后端参数，前端忽略",hidden = true)
    private String localFilePath;

    @ApiModelProperty(value = "本地压缩包内文件路径,后端参数，前端忽略",hidden = true)
    private List<String> localFilePathList;

    @ApiModelProperty(value = "文件名称,后端参数，前端忽略",hidden = true)
    private String originalFilename;

    @ApiModelProperty(value = "文件大小,后端参数，前端忽略",hidden = true)
    private Long size;


    // 试验场景相关接口
    @ApiModelProperty(value = "工况id")
    private Integer conditionId;

    @ApiModelProperty(value = "驾驶模式(0-标准)")
    private Integer drivingMode;

    @ApiModelProperty(value = "载荷（0-空载，1-半载，2-满载）")
    private Integer carLoad;


}
