package com.singularity.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.car.ride.RideMatlabFunction;
import com.mathworks.toolbox.javabuilder.MWNumericArray;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.dao.BarkingChartDataMapper;
import com.singularity.pojo.dto.barking.CompareBarkingDataDTO;
import com.singularity.pojo.dto.barking.DeleteBarkingDataFileDTO;
import com.singularity.pojo.dto.barking.QueryBarkingDataDTO;
import com.singularity.pojo.dto.barking.UploadBarkingDateFileDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.common.entity.BarkingChartData;
import com.singularity.pojo.entity.CarBasicParam;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.service.AttachmentService;
import com.singularity.service.BarkingChartDataService;
import com.singularity.service.CarBasicParamService;
import com.singularity.pojo.vo.barking.CompareBarkingDataVO;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.util.FileUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class BarkingChartDataServiceImpl extends ServiceImpl<BarkingChartDataMapper, BarkingChartData> implements BarkingChartDataService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @SneakyThrows
    @Transactional
    public void uploadBarkingFile(UploadBarkingDateFileDTO dto) {
        // 判断文件名是否重复
        LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Attachment::getName,dto.getOriginalFilename())
                .eq(Attachment::getObjName,dto.getCarId())
                .eq(Attachment::getOwnerType, AttachmentConstant.OWNER_TYPE_CAR);
        if(attachmentService.count(queryWrapper) > 0){
            throw new ToolsBusinessWarn("文件名称重复");
        }

        // 将文件上传到minio
        InputStream inputStream = new FileInputStream(dto.getLocalFilePath());
        attachmentService.uploadAttachment(
                inputStream,
                dto.getOriginalFilename(),
                dto.getSize(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                FileBusinessConstant.BARKING_DATA_FILE);
        inputStream.close();

        // 调用easyExcel函数读取衬套excel文件


        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setBrakingUploadStat(2);
        carBasicParam.setId(dto.getCarId());
        carBasicParamService.updateById(carBasicParam);
    }

    /**
     * 批量删除衬套试验数据文件
     */
    @Override
    public void deleteFileList(DeleteBarkingDataFileDTO dto) {
        // 删除数据库绘图数据
        LambdaQueryWrapper<BarkingChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(BarkingChartData::getAttachId,dto.getAttachIdList());
        this.remove(queryWrapper);

        // 删除附件
        attachmentService.removeByIds(dto.getAttachIdList());
    }

    /**
     * 查看衬套试验数据详情
     */
    @Override
    @SneakyThrows
    public ChartDataModel queryBarkingData(QueryBarkingDataDTO dto) {
        LambdaQueryWrapper<BarkingChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(BarkingChartData::getAttachId,dto.getAttachId())
                .eq(BarkingChartData::getChartType,dto.getTestType())
                .eq(BarkingChartData::getChartName,dto.getChartName());
        BarkingChartData BarkingChartData = this.getOne(queryWrapper);
        if(ObjectUtil.isEmpty(BarkingChartData)){
            return null;
        }
        ChartDataModel chartDataModel = (ChartDataModel)FileUtil.byteToObject(BarkingChartData.getChartData());

        // 拟合阶数不等于默认值3，重新调用matlab函数计算拟合函数数据
        if(dto.getOrder() != 3){
            log.info("调用matlab函数重新计算拟合数据");
            try{
                RideMatlabFunction rideMatlabFunction = new RideMatlabFunction();
                double[][] data = chartDataModel.getData();
                Object[] result = rideMatlabFunction.getFittingData(1,data[0],data[1],dto.getOrder());
                MWNumericArray mwNumericArray = (MWNumericArray)result[0];
                double[][] fittingY = (double[][])mwNumericArray.toDoubleArray();
                data[3] = fittingY[0];
                chartDataModel.setData(data);
            }catch (Exception e){
                e.printStackTrace();
            }
            log.info("调用matlab函数重新计算拟合数据成功");
        }
        return chartDataModel;
    }

    /**
     * 对比衬套试验数据结果
     */
    @Override
    @SneakyThrows
    public CompareBarkingDataVO compareBarkingData(CompareBarkingDataDTO dto) {
        LambdaQueryWrapper<BarkingChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(BarkingChartData::getAttachId,dto.getMap().keySet())
                .eq(BarkingChartData::getChartType,dto.getTestType())
                .eq(BarkingChartData::getChartName,dto.getChartName());
        List<BarkingChartData> BarkingChartDataList = this.list(queryWrapper);
        Map<Integer,List<BarkingChartData>> map = BarkingChartDataList.stream()
                .collect(Collectors.groupingBy(BarkingChartData::getAttachId));
        Map<Integer,Map<Double,Double>> dataMap = new HashMap<>();
        for (Integer key : map.keySet()){
            List<BarkingChartData> currentList = map.get(key);
            Map<Double,Double> currentMap = new HashMap<>();
            for (BarkingChartData BarkingChartData : currentList) {
                ChartDataModel chartDataModel = (ChartDataModel)FileUtil.byteToObject(BarkingChartData.getChartData());
                double[] xData = chartDataModel.getXData();
                double[] yData = chartDataModel.getYData();
                for (int i = 0; i < xData.length && i < yData.length; i++) {
                    currentMap.put(xData[i],yData[i]);
                }
                dataMap.put(dto.getMap().get(key),currentMap);
            }
        }
        CompareBarkingDataVO compareBarkingDataVO = new CompareBarkingDataVO();
        compareBarkingDataVO.setDataMap(dataMap);
        return compareBarkingDataVO;
    }
}
