package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.singularity.pojo.dto.condition.ConditionInfoDeleteDTO;
import com.singularity.pojo.dto.condition.ConditionInfoListQueryDTO;
import com.singularity.pojo.dto.condition.ConditionInfoAddDTO;
import com.singularity.pojo.dto.condition.ConditionInfoUpdateDTO;
import com.singularity.pojo.dto.condition.ConditionTypeDTO;
import com.singularity.dao.AttachmentMapper;
import com.singularity.dao.ConditionInfoMapper;
import com.singularity.dao.MeasureInfoMapper;
import com.singularity.pojo.dto.kc.QueryKCAttachmentListDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.ConditionInfo;
import com.singularity.pojo.entity.MeasureInfo;
import com.singularity.service.AttachmentService;
import com.singularity.service.ConditionInfoService;
import com.singularity.pojo.vo.condition.ConditionInfoListQueryVO;
import com.singularity.pojo.vo.condition.ConditionNameVO;
import com.singularity.pojo.vo.kc.QueryKCAttachmentListVO;
import com.singularity.pojo.vo.kc.QueryKCConditionInfoVO;
import com.singularity.exception.ToolsBusinessWarn;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ConditionInfoServiceImpl extends ServiceImpl<ConditionInfoMapper, ConditionInfo> implements ConditionInfoService {


    @Resource
    private ConditionInfoMapper conditionInfoMapper;

    @Resource
    private MeasureInfoMapper measureInfoMapper;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private AttachmentMapper attachmentMapper;

    /**
     * 工况列表分页查询
     */
    @Override
    public List<ConditionInfoListQueryVO> queryConditionInfoList(ConditionInfoListQueryDTO dto){
        // 不分页是弹窗下拉框查询，只查询正常状态数据
        if (dto.getPageSize() == 0)dto.setStatus(1);
        PageHelper.startPage(dto.getPageNum(),dto.getPageSize());
        return conditionInfoMapper.queryConditionInfoList(dto);
    }

    /**
     * 增加工况信息
     */
    @Override
    public void ddConditionInfo(ConditionInfoAddDTO dto){
        // 判断code是否重复
        LambdaQueryWrapper<ConditionInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ConditionInfo::getConditionCode,dto.getConditionCode());
        if(this.count(queryWrapper) > 0){
            throw new ToolsBusinessWarn("工况简写重复");
        }
        ConditionInfo conditionInfo = new ConditionInfo();
        LocalDate date = LocalDate.now();
        BeanUtil.copyProperties(dto,conditionInfo);
        conditionInfo.setCreateTime(date);
        conditionInfo.setUpdateTime(date);
        conditionInfo.setStatus(1);
        this.save(conditionInfo);
    }

    /**
     * 删除工况信息
     */
    @Override
    public void deleteConditionInfo(ConditionInfoDeleteDTO dto){
        // 判断此工况是否有测点关联
        QueryWrapper<MeasureInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("condition_id",dto.getIdList());
        if(measureInfoMapper.selectCount(queryWrapper) > 0){
            throw new ToolsBusinessWarn("请先删除下级测点");
        }
        this.removeBatchByIds(dto.getIdList());
    }

    /**
     * 修改工况信息
     */
    @Override
    public void updateConditionInfo(ConditionInfoUpdateDTO dto){
        // 判断code是否重复
        LambdaQueryWrapper<ConditionInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ConditionInfo::getConditionCode,dto.getConditionCode());
        if(this.count(queryWrapper) > 0){
            throw new ToolsBusinessWarn("工况简写重复");
        }
        ConditionInfo conditionInfo = new ConditionInfo();
        BeanUtil.copyProperties(dto,conditionInfo);
        LocalDate date = LocalDate.now();
        conditionInfo.setUpdateTime(date);
        this.updateById(conditionInfo);
    }

    @Override
    public List<ConditionNameVO> selectConditionByType(ConditionTypeDTO conditionTypeDTO) {
        return conditionInfoMapper.selectConditionByType(conditionTypeDTO);
    }

    @Override
    public Integer selectIdByConditionName(String s) {
        return conditionInfoMapper.selectIdByConditionName(s);
    }

    /**
     * 查询K&C工况测点信息
     */
    @Override
    public List<QueryKCConditionInfoVO> queryKCConditionInfo() {
        return conditionInfoMapper.queryKCConditionInfo();
    }

    /**
     * 批量查询K&C数据文件列表
     */
    @Override
    public QueryKCAttachmentListVO queryKCDataFileList(QueryKCAttachmentListDTO dto) {
        List<Attachment> attachmentList = attachmentMapper.queryKCDataFileList(dto);
        Map<Long,List<Attachment>> attachmentMap = attachmentList.stream()
                .collect(Collectors.groupingBy(Attachment::getOwnerId));
        QueryKCAttachmentListVO queryKCAttachmentListVO = new QueryKCAttachmentListVO();
        queryKCAttachmentListVO.setAttachmentMap(attachmentMap);
        return queryKCAttachmentListVO;
    }

}
