package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.dao.DepartmentMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.pojo.dto.department.DepartmentAddDTO;
import com.singularity.pojo.dto.department.DepartmentDeleteDTO;
import com.singularity.pojo.dto.department.DepartmentUpdateDTO;
import com.singularity.pojo.dto.department.QueryDepartmentTreeListDTO;
import com.singularity.pojo.entity.Department;
import com.singularity.pojo.vo.department.QueryDepartmentTreeListVO;
import com.singularity.service.DepartmentService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;


@Service
public class DepartmentServiceImpl extends ServiceImpl<DepartmentMapper, Department> implements DepartmentService {

    @Resource
    private DepartmentMapper departmentMapper;

    @Override
    public void addDepartment(DepartmentAddDTO dto) {
        // 部门名称唯一校验
        LambdaQueryWrapper<Department> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(Department::getDeptName,dto.getDeptName());
        if(departmentMapper.selectCount(queryWrapper) > 0){
            throw new ToolsBusinessWarn("部门名称不能重复");
        }
        Department department = BeanUtil.copyProperties(dto, Department.class);
        departmentMapper.insert(department);
    }


    @Override
    public void updateDepartment(DepartmentUpdateDTO departmentUpdateDTO) {
        Department department = BeanUtil.copyProperties(departmentUpdateDTO, Department.class);
        departmentMapper.updateById(department);
    }

    @Override
    @Transactional
    public void deleteDepartmentList(DepartmentDeleteDTO departmentDeleteDTO) {
        // 是否包含下级部门校验
        LambdaQueryWrapper<Department> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(Department::getParentId, departmentDeleteDTO.getIdList());
        if(departmentMapper.selectCount(queryWrapper) > 0){
            throw new ToolsBusinessWarn("请先删除下级部门");
        }
        this.removeBatchByIds(departmentDeleteDTO.getIdList());
    }

    @Override
    public List<QueryDepartmentTreeListVO> queryDepartmentTreeList(QueryDepartmentTreeListDTO dto) {
        // 查询所有部门信息
        LambdaQueryWrapper<Department> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.eq(StringUtils.isNotEmpty(dto.getDeptName()),Department::getDeptName,dto.getDeptName())
                .eq(ObjectUtil.isNotEmpty(dto.getState()),Department::getState,dto.getState());
        List<Department> departmentList  = this.list(queryWrapper2);
        List<QueryDepartmentTreeListVO> queryDepartmentTreeListVOList = BeanUtil.copyToList(departmentList,QueryDepartmentTreeListVO.class);
        // 将部门信息封装为树
        return this.convertMenuTree(queryDepartmentTreeListVOList);
    }

    /**
     * 两层循环封装菜单树
     *
     * @param treeNodes 需要封装的菜单集合
     */
    private List<QueryDepartmentTreeListVO> convertMenuTree(List<QueryDepartmentTreeListVO> treeNodes) {
        List<QueryDepartmentTreeListVO> trees = new ArrayList<>();
        for (QueryDepartmentTreeListVO treeNode : treeNodes) {
            if (Objects.equals(treeNode.getParentId(), 0)) {
                trees.add(treeNode);
            }
            for (QueryDepartmentTreeListVO it : treeNodes) {
                if (Objects.equals(treeNode.getId(), it.getParentId())) {
                    if (treeNode.getChildren() == null || treeNode.getChildren().size() == 0) {
                        treeNode.setChildren(new ArrayList<>());
                    }
                    List<QueryDepartmentTreeListVO> treeNodeChildren = treeNode.getChildren();
                    treeNodeChildren.add(it);
                    treeNode.setChildren(treeNodeChildren);
                }
            }
        }
        return trees;
    }
}
