package com.singularity.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.dao.ExperimentInfoMapper;
import com.singularity.dao.ProcessingMapper;
import com.singularity.pojo.dto.experiment.ExperimentFileDTO;
import com.singularity.pojo.dto.experiment.PreprocessingDTO;
import com.singularity.pojo.dto.experiment.SelectPreprocessingDTO;
import com.singularity.pojo.dto.experiment.TestConditionDTO;
import com.singularity.pojo.entity.ExperimentInfo;
import com.singularity.pojo.enumeration.FileTitleEnum;
import com.singularity.pojo.vo.attachment.UploadAttachmentVO;
import com.singularity.pojo.vo.experiment.PreprocessingVO;
import com.singularity.service.ExperimentService;
import com.singularity.service.MinioService;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class ExperimentServiceImpl extends ServiceImpl<ExperimentInfoMapper, ExperimentInfo> implements ExperimentService {
    @Resource
    private ProcessingMapper processingMapper;

    private final MinioService minioService;

    @Autowired
    public ExperimentServiceImpl(MinioService minioService) {
        this.minioService = minioService;
    }

    @Override
    public void updatepreprocessing(PreprocessingDTO preprocessingDTO) {
        processingMapper.updatePreprocessing(preprocessingDTO);
    }

    @Override
    public PreprocessingVO selectPreprocessing(SelectPreprocessingDTO selectPreprocessingDTO) {
        return processingMapper.selectPreprocessing(selectPreprocessingDTO);
    }

    @Override
    @SneakyThrows
    public void uploadTestConditions(ExperimentFileDTO experimentFileDTO) {
        UploadAttachmentVO uploadAttachmentVO = minioService.storeFile(experimentFileDTO.getMultipartFile());
        TestConditionDTO testConditionDTO = new TestConditionDTO();
        testConditionDTO.setName(uploadAttachmentVO.getName());
        testConditionDTO.setObjName(uploadAttachmentVO.getObjName());
        testConditionDTO.setPath(uploadAttachmentVO.getPath());
        testConditionDTO.setType(uploadAttachmentVO.getType());
        testConditionDTO.setSize(uploadAttachmentVO.getSize());
        testConditionDTO.setOwnerType(experimentFileDTO.getOwnerType());
        testConditionDTO.setOwnerId(experimentFileDTO.getOwnerId());
        testConditionDTO.setBusinessType(experimentFileDTO.getBusinessType());

        String[] split = testConditionDTO.getName().split("_");
        if (split.length > 3){
            //设置载荷
            testConditionDTO.setCarLoad(FileTitleEnum.getCodeByName(split[1]));

            //设置试验条件模式
            if ("标准模式".equals(split[2])){
                testConditionDTO.setDrivingMode(0);
            }

            Pattern p = Pattern.compile("[^0-9]");
            Matcher m = p.matcher(split[3].split("\\.")[0]);
            testConditionDTO.setTestSpeed(m.replaceAll("").trim());
        }

        processingMapper.insertAttachmentFile(testConditionDTO);

        testConditionDTO.setAttachId(Math.toIntExact(testConditionDTO.getId()));

        processingMapper.insertExperimentSettings(testConditionDTO);
    }
}
