package com.singularity.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.dao.ExperimentalResultsMapper;
import com.singularity.pojo.dto.ride.CarTestMessageDTO;
import com.singularity.pojo.dto.ride.CompareTestResultDTO;
import com.singularity.pojo.dto.ride.CreateRideTestReportDTO;
import com.singularity.pojo.dto.ride.RideCarIdDTO;
import com.singularity.pojo.entity.CarTest;
import com.singularity.pojo.entity.ExperimentalResults;
import com.singularity.pojo.vo.ride.CarTestMessageVO;
import com.singularity.pojo.vo.ride.CompareTestResultVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.CarTestService;
import com.singularity.service.ExperimentalResultsService;
import com.singularity.util.FileUtil;
import com.singularity.util.WordUtil;
import lombok.SneakyThrows;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


@Service
public class ExperimentalResultsServiceImpl extends ServiceImpl<ExperimentalResultsMapper, ExperimentalResults> implements ExperimentalResultsService {

    @Resource
    private MinioServiceImpl minioService;

    @Resource
    private AttachmentService attachmentService;

    @Value("${file.path}")
    private String filePath;

    @Resource
    private CarTestService carTestService;

    /**
     * 对比试验结果
     */
    @Override
    public CompareTestResultVO compareTestResult(CompareTestResultDTO dto) {

        // 1、根据车辆idList查询所有试验结果信息
        LambdaQueryWrapper<ExperimentalResults> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(ExperimentalResults::getCarId, dto.getCarIdList());
        List<ExperimentalResults> experimentalResults = this.list(queryWrapper);

        // 通过车辆id分组
        Map<Long, List<ExperimentalResults>> map = experimentalResults.stream()
                .collect(Collectors.groupingBy(ExperimentalResults::getCarId));

        if (map.size() == 0)
            return null;

        // 2、定义表头
        Optional<List<ExperimentalResults>> first = map.values().stream().findFirst();
        List<String> indexTableHead = first.get().stream().map(ExperimentalResults::getTarget
        ).collect(Collectors.toList());

        List<String> unitTableHead = first.get().stream().map(ExperimentalResults::getUnit
        ).collect(Collectors.toList());

        List<String> paramTableHead = new ArrayList<>();
        while (paramTableHead.size() != indexTableHead.size() * 2) {
            paramTableHead.add("S30.txt");
            paramTableHead.add("平均值");
        }
        CompareTestResultVO compareTestResultVO = new CompareTestResultVO();
        compareTestResultVO.setIndexTableHead(indexTableHead);
        compareTestResultVO.setUnitTableHead(unitTableHead);
        compareTestResultVO.setParamTableHead(paramTableHead);

        // 3、定义试验结果
        Map<Long, Map<String, List<BigDecimal>>> data = new HashMap<>();
        for (Integer carId : dto.getCarIdList()) {
            List<ExperimentalResults> paramList = map.get((long) carId);
            if (Objects.isNull(paramList) || paramList.size() < 1) {
                continue;
            }
            Map<String, List<BigDecimal>> paramMap = new HashMap<>();
            for (ExperimentalResults e : paramList) {
                List<BigDecimal> list = new ArrayList<>();
                list.add(e.getS30());
                list.add(e.getAverage());
                paramMap.put(e.getTarget(), list);
            }
            data.put((long) carId, paramMap);
        }
        compareTestResultVO.setData(data);
        return compareTestResultVO;
    }

    /**
     * 根据模板生成word文档
     */
    @Override
    @SneakyThrows
    public void createRideTestReport(CreateRideTestReportDTO dto) {
        // 1、根据模板id查询模板信息
        InputStream tempInputStream = minioService.getFile(dto.getObjName());
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String tempPathAndName = filePath + uuid + ".docx";
        uuid = UUID.randomUUID().toString().replaceAll("-", "");
        // 试验报告名称
        String fileName = "平顺性试验报告.docx";
        String tempOutPath = filePath + uuid + "-" + fileName;
        FileUtil.downloadToLocal(tempInputStream, tempPathAndName);
        tempInputStream.close();

        // 2、查询图片数据
        List<InputStream> inputStreamList = new ArrayList<>();
        for (String objName : dto.getObjNameList()) {
            InputStream inputStream = minioService.getFile(objName);
            inputStreamList.add(inputStream);
        }
        Map<String, Object> params = new HashMap<>();
        int index = 1;
        for (InputStream inputStream : inputStreamList) {
            // excel数据文件地址
            uuid = UUID.randomUUID().toString().replaceAll("-", "");
            String localFileName = filePath + uuid + "-" + ".jpg";
            FileUtil.downloadToLocal(inputStream, localFileName);
            inputStream.close();
            Map<String, Object> jpeg = new HashMap<>();
            jpeg.put("width", 100);
            jpeg.put("height", 150);
            jpeg.put("type", "jpg");
            jpeg.put("content", WordUtil.inputStream2ByteArray(new FileInputStream(localFileName), true));
            params.put("photo" + index++, jpeg);
            FileUtil.deleteFile(localFileName);
        }

        // 3、查询表格数据
        LambdaQueryWrapper<ExperimentalResults> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ExperimentalResults::getCarId, dto.getCarId());
        List<ExperimentalResults> experimentalResults = this.list(queryWrapper);
        List<String[]> data = new ArrayList<>();
        for (ExperimentalResults experimentalResult : experimentalResults) {
            String[] str = new String[5];
            str[0] = experimentalResult.getId().toString();
            str[1] = experimentalResult.getTarget();
            str[2] = experimentalResult.getUnit();
            str[3] = experimentalResult.getAverage().toString();
            str[4] = experimentalResult.getS30().toString();
            data.add(str);
        }

        // 4、插入图片与表格数据并生成试验报告
        WordUtil.changWord(tempPathAndName, tempOutPath, params, data, 1, 0);

        // 5、将试验报告保存至minio
        File file = new File(tempOutPath);
        InputStream fileInputStream = new FileInputStream(file);
        attachmentService.uploadAttachment(fileInputStream,fileName,file.length(),"car_info",(long)dto.getCarId(),4);
        fileInputStream.close();

        // 6、删除本地试验报告模板与试验报告文件
        FileUtil.deleteFile(tempPathAndName);
        FileUtil.deleteFile(tempOutPath);


    }

    @Override
    public CarTestMessageVO queryTestMessage(RideCarIdDTO dto) {
        CarTestMessageVO carTestMessageVO = new CarTestMessageVO();
        LambdaQueryWrapper<CarTest> qw = new LambdaQueryWrapper<>();
        qw.eq(CarTest::getCarId,dto.getCarId());
        qw.select().orderByDesc(CarTest::getUpdateTime);
        qw.last("limit 1");
        BeanUtils.copyProperties(carTestService.getOne(qw),carTestMessageVO);
        return carTestMessageVO;
    }

    @Override
    public void setTestMessage(CarTestMessageDTO dto) {
        CarTest carTest = new CarTest();
        BeanUtils.copyProperties(dto,carTest);
        Integer id = carTestService.selectTestMessage(dto);
        carTest.setCarId(carTest.getId());
        carTest.setId(id);
        carTest.setUpdateTime(new Date());
        carTestService.saveOrUpdate(carTest);
    }
}
