package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.car.ride.RideMatlabFunction;
import com.mathworks.toolbox.javabuilder.MWNumericArray;
import com.singularity.common.entity.RidingChartData;
import com.singularity.pojo.bo.handing.TableHeadFiledBO;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.dto.handing.*;
import com.singularity.pojo.vo.hading.*;
import com.singularity.dao.TableHeadMapper;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.enumeration.ExperimentEnum;
import com.singularity.pojo.enumeration.HandingResultEnum;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.service.*;
import com.singularity.util.FileUtil;
import com.singularity.util.WordUtil;
import com.singularity.listener.HandingReadChartDataListener;
import com.singularity.listener.HandingReadFileListener;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.IOUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.*;
import java.io.File;
import java.nio.charset.Charset;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import static java.lang.Double.parseDouble;

@Slf4j
@Service
public class HandingServiceImpl implements HandingService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private TableDataService tableDataService;

    @Resource
    private TableHeadService tableHeadService;

    @Resource
    private TableHeadMapper tableHeadMapper;

    @Resource
    private MinioServiceImpl minioService;

    @Resource
    private TableChartService tableChartService;

    @Resource
    private FileBusinessService fileBusinessService;

    @Resource
    private RidingChartDataService ridingChartDataService;

    @Resource
    private TaskQueueService taskQueueService;

    @Resource
    private ExperimentService experimentInfoService;

    @Value("${file.path}")
    private String filePath;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource
    private ExperimentService experimentService;

    /**
     * 保存操作稳定性试验结果与实验曲线数据
     */
    private void readHandingTestDataFile(UploadHandingFileDTO dto) throws Exception{
        MultipartFile multipartFile = dto.getMultipartFile();
        // 1、根据试验工况查询该试验有哪些数据表
        LambdaQueryWrapper<TableHead> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(TableHead::getId, TableHead::getSheetName, TableHead::getTableType)
                .eq(TableHead::getBusiness_id, dto.getBusinessType())
                .eq(TableHead::getStat, 1);
        List<TableHead> tableHeadList = tableHeadService.list(queryWrapper);

        // 2、文件名验重
        attachmentService.checkRepeatFileName(
                dto.getOriginalFilename(),
                dto.getCarId(),
                dto.getBusinessType(),
                AttachmentConstant.OWNER_TYPE_CAR
        );

        // 3、将文件上传至minio
        Attachment attachment = attachmentService.uploadAttachment(
                multipartFile.getInputStream(),
                dto.getOriginalFilename(),
                dto.getSize(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                dto.getBusinessType(),
                dto.getOperId()
        );


        // 4、保存文件试验场景信息
        ExperimentInfo experimentInfo = ExperimentInfo.builder()
                .attachId(attachment.getId())
                .carLoad(dto.getCarLoad())
                .build();
        experimentInfoService.save(experimentInfo);

        // 5、读取excel中的关键指标数据
        for (TableHead tableHead : tableHeadList) {
            // 0代表是绘图数据文件，读取绘图数据保存到数据库
            if (tableHead.getTableType() == 0) {
                // 查询改数据表查询绘图数据读取方式
                LambdaQueryWrapper<TableChart> queryWrapper3 = Wrappers.lambdaQuery();
                queryWrapper3.eq(TableChart::getHeadId, tableHead.getId());
                List<TableChart> tableChartList = tableChartService.list(queryWrapper3);

                // 使用EasyExcel读取数据文件，一个数据表可以绘制多个图像，每个图像对应一个tableChart数据读取方式
                HandingReadChartDataListener handingReadChartDataListener = new HandingReadChartDataListener();
                EasyExcel.read(multipartFile.getInputStream(), handingReadChartDataListener)
                        .headRowNumber(0).sheet(tableHead.getSheetName()).doRead();
                List<Map<Integer, String>> chartDataList = handingReadChartDataListener.getChartDataList();

                // 根据每种图像读取方式读取List的数据
                List<QueryHandingChartDataSon> queryHandingChartDataSonArrayList = new ArrayList<>();
                for (TableChart tableChart : tableChartList) {
                    // 根据条件读取数据
                    QueryHandingChartDataSon queryHandingChartDataSon =
                            new QueryHandingChartDataSon();
                    double[][] data = new double[6][];
                    List<Double> dataX = new ArrayList<>();
                    List<Double> dataY = new ArrayList<>();
                    // x轴数据所在列
                    int colX = tableChart.getColX();
                    // y轴数据所在列
                    int colY = tableChart.getColY();
                    for (int i = tableChart.getStartRow(); i < chartDataList.size(); i += tableChart.getRowInterval()) {
                        // 根据行号获取数据表该行数据
                        Map<Integer, String> dataMap = chartDataList.get(i);
                        if (dataMap.containsKey(colX))
                            dataX.add(parseDouble(dataMap.get(colX)));
                        if (dataMap.containsKey(colY))
                            dataY.add(parseDouble(dataMap.get(colY)));
                    }
                    int minLength = Math.min(dataX.size(), dataY.size());
                    data[0] = listToDouble(dataX, minLength);
                    data[1] = listToDouble(dataY, minLength);

                    // 调用matlab函数计算拟合曲线
                    if(dto.getBusinessType() == 30){ // 转向盘中心区试验
                        List<List<Double>> fittingList = this.getFittingData(tableChart.getFittingStart(),tableChart.getFittingEnd(),data[0],data[1]);
                        RideMatlabFunction rideMatlabFunction = new RideMatlabFunction();
                        double[] x1 = fittingList.get(0).stream().mapToDouble(Double::doubleValue).toArray();
                        double[] y1 = fittingList.get(1).stream().mapToDouble(Double::doubleValue).toArray();
                        double[] x2 = fittingList.get(2).stream().mapToDouble(Double::doubleValue).toArray();
                        double[] y2 = fittingList.get(3).stream().mapToDouble(Double::doubleValue).toArray();
                        Object[] result = rideMatlabFunction.getFittingData(1, x1,y1,3);
                        Object[] result2 = rideMatlabFunction.getFittingData(1, x2,y2,3);
                        data[2] = x1;
                        MWNumericArray mwNumericArray = (MWNumericArray) result[0];
                        double[][] fittingY = (double[][]) mwNumericArray.toDoubleArray();
                        data[3] = fittingY[0];
                        data[4] = x2;
                        MWNumericArray mwNumericArray2 = (MWNumericArray) result2[0];
                        double[][] fittingY2 = (double[][]) mwNumericArray2.toDoubleArray();
                        data[5] = fittingY2[0];
                    }else{
                        RideMatlabFunction rideMatlabFunction = new RideMatlabFunction();
                        Object[] result = rideMatlabFunction.getFittingData(1, data[0], data[1], 3);
                        data[2] = data[0];
                        MWNumericArray mwNumericArray = (MWNumericArray) result[0];
                        double[][] fittingY = (double[][]) mwNumericArray.toDoubleArray();
                        data[3] = fittingY[0];
                    }
                    queryHandingChartDataSon.setData(data);
                    queryHandingChartDataSon.setXLabel(tableChart.getLabelX());
                    queryHandingChartDataSon.setYLabel(tableChart.getLabelY());
                    queryHandingChartDataSon.setTitle(tableChart.getTitle());
                    queryHandingChartDataSonArrayList.add(queryHandingChartDataSon);
                }

                // 将绘图数据保存至数据库
                LocalDateTime localDateTime = LocalDateTime.now();
                RidingChartData ridingChartData = new RidingChartData();
                ridingChartData.setCarId(dto.getCarId());
                ridingChartData.setAttachId(attachment.getId().intValue());
                ridingChartData.setHeadId(tableHead.getId());
                ridingChartData.setCreateTime(localDateTime);
                ridingChartData.setUpdateTime(localDateTime);
                ridingChartData.setChartData(FileUtil.objectToByte(queryHandingChartDataSonArrayList));
                ridingChartDataService.save(ridingChartData);
            } else {
                // 非0代表是试验数据文件与试验结果文件，将其保存到数据库
                HandingReadFileListener handingReadFileListener = new HandingReadFileListener(
                        dto.getCarId(),
                        tableHead.getId(),
                        attachment.getId().intValue(),
                        tableHead.getEndCol()
                );
                EasyExcel.read(multipartFile.getInputStream(), handingReadFileListener)
                        .headRowNumber(0).sheet(tableHead.getSheetName()).doRead();
                List<TableData> tableDataList = handingReadFileListener.getTableDataList();
                tableDataService.saveBatch(tableDataList);
            }
        }
    }

    /**
     * 从左至右遍历
     */
    private List<List<Double>> getFittingData(Integer start,Integer end,double[] y,double[] x){
        List<List<Double>> result = new ArrayList<>();
        List<Double> y1 = new ArrayList<>();
        List<Double> x1 = new ArrayList<>();
        List<Double> y2 = new ArrayList<>();
        List<Double> x2 = new ArrayList<>();
        for(int i = 1; i < y.length; i++){
            if(y[i] - y[i - 1] > 0 && x[i] > start && x[i] < end){
                y1.add(y[i]);
                x1.add(x[i]);
            }else if (y[i] - y[i - 1] <= 0 && x[i] > start && x[i] < end){
                y2.add(y[i]);
                x2.add(x[i]);
            }
        }
        result.add(x1);
        result.add(y1);
        result.add(x2);
        result.add(y2);
        return result;
    }


    /**
     * 批量上传操纵稳定性试验结果文件
     */
    @SneakyThrows
    @Transactional
    public void uploadHandingFileList(UploadHandingFileDTO dto) {
        MultipartFile multipartFile = StringUtils.isNotEmpty(dto.getLocalFilePath()) ? FileUtil.fileToMultipartFile(dto.getLocalFilePath()) : dto.getMultipartFile();
        LocalDateTime startTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setId(dto.getTaskId());
        try {
            if(multipartFile.getOriginalFilename().contains(".zip")){ //批量上传
                InputStream inputStream = multipartFile.getInputStream();
                ZipInputStream zipInputStream = new ZipInputStream(inputStream, Charset.forName("GBK"));
                ZipEntry zipEntry = zipInputStream.getNextEntry();
                while (zipEntry != null) {
                    if (!zipEntry.isDirectory()) {
                        // 获取文件名称
                        String[] currentFilePath = zipEntry.getName().split("/");
                        String fileName = currentFilePath[currentFilePath.length - 1];
                        dto.setOriginalFilename(fileName);
                        if (currentFilePath.length != 4)
                            throw new ToolsBusinessWarn("压缩包格式错误，请使用导入模板导入数据");
                        Integer businessId = HandingResultEnum.getCodeByName(currentFilePath[currentFilePath.length - 3]);
                        Integer carLoad = ExperimentEnum.LoadEnum.getCodeByName(currentFilePath[currentFilePath.length - 2]);
                        dto.setCarLoad(carLoad);
                        if (ObjectUtil.isNull(businessId))
                            throw new ToolsBusinessWarn("压缩包格式错误，请使用导入模板导入数据");
                        dto.setBusinessType(businessId);
                        byte[] bytes = IOUtils.toByteArray(zipInputStream);
                        InputStream zipEntryStream = new ByteArrayInputStream(bytes);
                        dto.setSize(zipEntry.getSize());
                        String fileType = fileName.substring(fileName.lastIndexOf(".") + 1);
                        dto.setMultipartFile(new MockMultipartFile(fileName, fileName, fileType, zipEntryStream));
                        this.readHandingTestDataFile(dto);
                    }
                    zipInputStream.closeEntry();
                    zipEntry = zipInputStream.getNextEntry();
                }
                zipInputStream.close();
                inputStream.close();
            }else{ //单个上传
                dto.setOriginalFilename(multipartFile.getOriginalFilename());
                dto.setSize(multipartFile.getSize());
                this.readHandingTestDataFile(dto);
            }

        }catch(Exception e){
            e.printStackTrace();
            taskQueue.setRemark(e.getMessage());
            taskQueue.setStat(3);
        }finally{
            Duration duration = Duration.between(startTime,LocalDateTime.now());
            taskQueue.setConsumeTime(duration.getSeconds());
            taskQueueService.updateById(taskQueue);
            // 删除临时文件
            if(StringUtils.isNotEmpty(dto.getLocalFilePath()))
                FileUtil.deleteFile(dto.getLocalFilePath());
        }
    }

    /**
     * 生成操作稳定性试验报告
     */
    @Override
    @SneakyThrows
    @Transactional
    public void createHadingReport(CreateHadingReportDTO dto){
        // 1、查询模板信息
        InputStream tempInputStream = minioService.getFile(dto.getObjName());
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String fileName = "操作稳定性试验报告.docx";
        String outPath = filePath + uuid + "-" + fileName;

        // 2、查询用户选择数据表
        LambdaQueryWrapper<TableHead> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(TableHead::getId,TableHead::getTableName)
                .in(dto.getHeadIdList().size() > 0,TableHead::getId,dto.getHeadIdList())
                .eq(TableHead::getStat,1);
        List<TableHead> tableHeadList = tableHeadService.list(queryWrapper);
        // headId 与 tableName映射Map
        Map<Integer,String> idToNameMap = tableHeadList.stream()
                .collect(Collectors.toMap(TableHead::getId,TableHead::getTableName,(key1, key2) -> key2));

        // 3、根据headId分组
        List<Integer> tableHeadIdList = tableHeadList.stream().map(TableHead::getId).collect(Collectors.toList());

        // 4、查询所有数据表数据
        LambdaQueryWrapper<TableData> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.select(TableData::getTableHeadId,TableData::getRowInfo)
                .eq(TableData::getCarId,dto.getCarId())
                .in(TableData::getTableHeadId,tableHeadIdList);
        List<TableData> tableDataList = tableDataService.list(queryWrapper2);
        // 根据数据表id分组
        Map<Integer,List<TableData>> groupByHeadIdMap = tableDataList.stream()
                .collect(Collectors.groupingBy(TableData::getTableHeadId));

        // 5、封装当前试验类别数据表数据
        Map<String,List<Map<Integer,String>>> tableList = new HashMap<>();
        for(Integer tableHeadId : groupByHeadIdMap.keySet()){
            List<TableData> tableDataList2 = groupByHeadIdMap.get(tableHeadId);
            List<Map<Integer,String>> data = new ArrayList<>();
            for (TableData tableData : tableDataList2) {
                Map map = JSONObject.parseObject(tableData.getRowInfo(), Map.class);
                data.add(map);
            }
            tableList.put(idToNameMap.get(tableHeadId),data);
        }

        // 6、封装当前试验类别图像数据
        Map<String,String> svgDataUrlMap = dto.getSvgDataUrlMap();
        Map<String, Object> photoMap = new HashMap<>();
        if(CollectionUtil.isNotEmpty(svgDataUrlMap)){
            for (String key : svgDataUrlMap.keySet()) {
                String svgDataUrl = svgDataUrlMap.get(key);
                // 将eCharts svg格式dataUrl转换为字符数组
                Map<String,Object> jpeg = new HashMap<>();
                jpeg.put("width", 600);
                jpeg.put("height", 300);
                jpeg.put("type", "svg");
                jpeg.put("content", Base64.getDecoder().decode(svgDataUrl.replace("data:image/png;base64,","")));
                photoMap.put(key,jpeg);
            }
        }


        // 7、将word模板中的占位符替换为图片与表格
        if(!WordUtil.changWord(tempInputStream, outPath, photoMap, tableList))
            throw new ToolsBusinessWarn("生成试验报告失败");


        // 8、上传文件到minio
        File file = new File(outPath);
        InputStream fileInputStream = new FileInputStream(file);
        attachmentService.uploadAttachment(
                fileInputStream,
                fileName,
                file.length(),
                AttachmentConstant.OWNER_TYPE_CAR,
                (long)dto.getCarId(),
                9
        );
        fileInputStream.close();

        // 9、删除本地报告文件
        FileUtil.deleteFile(outPath);
    }

    /**
     * 查询操作稳定性试验数据表
     */
    @Override
    public List<QueryTableDataVO> queryHandingTableData(QueryTableDataDTO dto){
        // 1、根据条件查询操作稳定性试验中间数据表与试验结果表
        // 根据附件id查询业务类型id
        Attachment attachment = attachmentService.getById(dto.getId());
        LambdaQueryWrapper<TableHead> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.select(TableHead::getId,TableHead::getTableName)
                .eq(TableHead::getBusiness_id,attachment.getBusinessType())
                .notIn(TableHead::getTableType,0);
        List<TableHead> tableHeadList = tableHeadService.list(queryWrapper2);
        if(tableHeadList.size() < 1)return new ArrayList<>();
        List<Integer> tableHeadIdList = tableHeadList.stream().map(TableHead::getId).collect(Collectors.toList());
        // headId 与 tableName映射Map
        Map<Integer,String> idToNameMap = tableHeadList.stream()
                .collect(Collectors.toMap(TableHead::getId,TableHead::getTableName,(key1, key2) -> key2));


        // 2、查询操作稳定性所有数据表-表头集合
        List<TableHeadFiledBO> tableHeadFiledBOList = tableHeadMapper.getTableFieldByHeadIdList(tableHeadIdList);
        Map<Integer,List<TableHeadFiledBO>> tableHeadMap = tableHeadFiledBOList.stream()
                .collect(Collectors.groupingBy(TableHeadFiledBO::getHeadId));


        // 3、查询操作稳定性所有数据表-数据集合
        LambdaQueryWrapper<TableData> queryWrapper3 = Wrappers.lambdaQuery();
        queryWrapper3.eq(TableData::getCarId,dto.getCarId())
                .eq(TableData::getAttachId,dto.getId())
                .in(TableData::getTableHeadId,tableHeadIdList);
        List<TableData> tableDataList = tableDataService.list(queryWrapper3);
        Map<Integer,List<TableData>> map = tableDataList.stream()
                .collect(Collectors.groupingBy(TableData::getTableHeadId));


        // 4、根据表头id将每组表头与数据封装为一个对象
        List<QueryTableDataVO> queryTableDataVOList = new ArrayList<>();
        for(Integer tableHeadId : map.keySet()){
            // 4.1、tableData
            QueryTableDataVO queryTableDataVO = new QueryTableDataVO();
            List<TableData> tableDataList2 = map.get(tableHeadId);
            List<QueryTableDataVO.QueryTableDataSon> sonList = new ArrayList<>();
            for (TableData tableData : tableDataList2) {
                QueryTableDataVO.QueryTableDataSon queryTableDataSon
                        = new QueryTableDataVO.QueryTableDataSon();
                Map dataMap = JSONObject.parseObject(tableData.getRowInfo(), Map.class);
                queryTableDataSon.setTableDataMap(dataMap);
                queryTableDataSon.setId(tableData.getId());
                sonList.add(queryTableDataSon);
            }

            // 4.2、tableHead
            List<TableHeadFiledBO> tableHeadFiledBOList2 = tableHeadMap.get(tableHeadId);
            List<String> headList  = tableHeadFiledBOList2.stream()
                    .map(TableHeadFiledBO::getFieldName).collect(Collectors.toList());
            int i = 0;
            Map<Integer,String> headMap = new HashMap<>();
            for (String s : headList) {
                headMap.put(i++,s);
            }

            // 4.3、VO
            queryTableDataVO.setTableDataList(sonList);
            queryTableDataVO.setTableName(idToNameMap.get(tableHeadId));
            queryTableDataVO.setTableHeadMap(headMap);
            queryTableDataVOList.add(queryTableDataVO);
        }

        return queryTableDataVOList;
    }

    /**
     * 修改操作稳定性试验数据表
     */
    @Override
    public void updateHandingTableData(UpdateHandingTableDataDTO dto){
        LocalDate localDate = LocalDate.now();
        LambdaUpdateWrapper<TableData> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.eq(TableData::getId,dto.getId())
                .set(TableData::getRowInfo, JSON.toJSONString(dto.getTableDataMap()))
                .set(TableData::getUpdateTime,localDate);
        tableDataService.update(updateWrapper);
    }

    /**
     * 查询操作稳定性图像数据
     */
    @Override
    public List<QueryHandingChartDataVO> queryHandingChartData(QueryHandingChartDataDTO dto){
        List<QueryHandingChartDataVO> queryHandingTableDataVOList = new ArrayList<>();
        // 1、操作稳定性试验数据表IdList
        // 根据附件id查询业务类型id
        Attachment attachment = attachmentService.getById(dto.getId());
        LambdaQueryWrapper<TableHead> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.select(TableHead::getId,TableHead::getTableName)
                .eq(TableHead::getBusiness_id,attachment.getBusinessType())
                .eq(TableHead::getTableType,0);
        List<TableHead> tableHeadList = tableHeadService.list(queryWrapper2);

        // 2、根据数据表id查询图像数据
        for (TableHead tableHead : tableHeadList) {
            // 查询该数据表的绘图数据
            LambdaQueryWrapper<RidingChartData> queryWrapper3 = Wrappers.lambdaQuery();
            queryWrapper3.select(RidingChartData::getChartData)
                    .eq(RidingChartData::getHeadId,tableHead.getId())
                    .eq(RidingChartData::getAttachId,dto.getId())
                    .eq(RidingChartData::getCarId,dto.getCarId());
            List<RidingChartData> ridingChartDataList = ridingChartDataService.list(queryWrapper3);
            if(CollectionUtil.isEmpty(ridingChartDataList))continue;
            RidingChartData ridingChartData = ridingChartDataList.get(0);
            // 封装绘图数据
            List<QueryHandingChartDataSon> queryHandingChartDataSonList = new ArrayList<>();

            try{
                Object object = FileUtil.byteToObject(ridingChartData.getChartData());
                if (object instanceof ArrayList<?>) {
                    List<?> obj = (List<?>) object;
                    for (Object o : obj) {
                        queryHandingChartDataSonList.add((QueryHandingChartDataSon)o);
                    }
                }
            }catch (Exception e){
                e.printStackTrace();
            }
            // 拟合阶数不等于默认值3，重新调用matlab函数计算拟合函数数据
            if(dto.getOrder() != 3){
                log.info("调用matlab函数重新计算拟合数据");
                for (QueryHandingChartDataSon queryHandingChartDataSon : queryHandingChartDataSonList) {
                    try{
                        RideMatlabFunction rideMatlabFunction = new RideMatlabFunction();
                        double[][] data = queryHandingChartDataSon.getData();
                        Object[] result = rideMatlabFunction.getFittingData(1,data[0],data[1],dto.getOrder());
                        MWNumericArray mwNumericArray = (MWNumericArray)result[0];
                        double[][] fittingY = (double[][])mwNumericArray.toDoubleArray();
                        data[3] = fittingY[0];
                        queryHandingChartDataSon.setData(data);
                    }catch (Exception e){
                        e.printStackTrace();
                    }
                }
                log.info("调用matlab函数重新计算拟合数据成功");
            }

            QueryHandingChartDataVO queryHandingChartDataVO = new QueryHandingChartDataVO();
            queryHandingChartDataVO.setChartDataSonList(queryHandingChartDataSonList);
            queryHandingChartDataVO.setTestName(tableHead.getTableName());
            queryHandingTableDataVOList.add(queryHandingChartDataVO);
        }
        return queryHandingTableDataVOList;
    }

    private double[] listToDouble(List<Double> list, int length){
        double[] result = new double[length];
        for (int i = 0; i < length; i++) {
            result[i] = list.get(i);
        }
        return result;
    }


    /**
     * 批量删除操作稳定性数据文件
     */
    @Override
    public void deleteHandingFile(DeleteHandingFileDTO dto){
        // 1、删除minio上的附件
        attachmentService.removeAttachmentById(dto.getAttachIdList());

        // 2、删除数据文件
        LambdaQueryWrapper<TableData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(TableData::getAttachId,dto.getAttachIdList());
        tableDataService.remove(queryWrapper);

        // 3、删除绘图数据
        LambdaQueryWrapper<RidingChartData> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.in(RidingChartData::getAttachId,dto.getAttachIdList());
        ridingChartDataService.remove(queryWrapper2);

        // 4、删除试验场景信息
        LambdaQueryWrapper<ExperimentInfo> queryWrapper3 = Wrappers.lambdaQuery();
        queryWrapper3.in(ExperimentInfo::getAttachId,dto.getAttachIdList());
        experimentInfoService.remove(queryWrapper3);
    }

    /**
     * 查询试验类别信息
     */
    @Override
    public List<QueryExpTypeInfoVO> queryExpTypeInfo(QueryExpTypeInfoDTO dto){
        LambdaQueryWrapper<FileBusiness> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(FileBusiness::getId,FileBusiness::getBusinessName)
                .eq(FileBusiness::getParentId,dto.getBusinessId())
                .eq(ObjectUtil.isNotEmpty(dto.getBusinessType()),FileBusiness::getBusinessType,dto.getBusinessType())
                .eq(FileBusiness::getStat,1);
        List<FileBusiness> fileBusinessList = fileBusinessService.list(queryWrapper);
        return BeanUtil.copyToList(fileBusinessList,QueryExpTypeInfoVO.class);
    }

    /**
     * 对比操作稳定性图像数据
     */
    @Override
    public List<QueryHandingChartDataVO> compareHandingChartData(CompareHandingChartDataDTO dto) {
        List<QueryHandingChartDataVO> queryHandingTableDataVOList = new ArrayList<>();
        // 1、操作稳定性试验数据表IdList
        LambdaQueryWrapper<TableHead> queryWrapper1 = Wrappers.lambdaQuery();
        queryWrapper1.select(TableHead::getId,TableHead::getTableName)
                .eq(TableHead::getBusiness_id,dto.getBusinessId())
                .eq(TableHead::getTableType,0);
        List<TableHead> tableHeadList = tableHeadService.list(queryWrapper1);

        // 2、根据数据表id查询图像数据
        for (TableHead tableHead : tableHeadList) {
            // 2.1查询该数据表的绘图数据
            LambdaQueryWrapper<RidingChartData> queryWrapper2 = Wrappers.lambdaQuery();
            queryWrapper2.select(RidingChartData::getHeadId,RidingChartData::getChartData)
                    .eq(RidingChartData::getHeadId,tableHead.getId())
                    .in(RidingChartData::getCarId,dto.getCarIdList());
            List<RidingChartData> ridingChartDataList = ridingChartDataService.list(queryWrapper2);
            // 该数据表没有绘图数据，跳过
            if(CollectionUtil.isEmpty(ridingChartDataList))continue;

            // 通过数据表id分组
            Map<Integer,List<RidingChartData>> groupMap = ridingChartDataList.stream()
                    .collect(Collectors.groupingBy(RidingChartData::getHeadId));
            // 遍历map,将所有车辆绘图数据封装到一个数组中
            for (Integer key : groupMap.keySet()) {
                List<RidingChartData> groupList = groupMap.get(key);
                // 这个map等于List
                Map<String,QueryHandingChartDataSon> map = new HashMap<>();
                // 一个ridingChartData就是一辆车一个数据表下的一组图像
                for (RidingChartData ridingChartData : groupList) {
                    try{
                        Object object = FileUtil.byteToObject(ridingChartData.getChartData());
                        if (object instanceof ArrayList<?>) {
                            List<?> obj = (List<?>) object;
                            for (Object o : obj) {
                                QueryHandingChartDataSon queryHandingChartDataSon = (QueryHandingChartDataSon)o;
                                if(!map.containsKey(queryHandingChartDataSon.getTitle()))
                                    map.put(queryHandingChartDataSon.getTitle(),queryHandingChartDataSon);
                                else{
                                    // 添加曲线数据
                                    QueryHandingChartDataSon now = map.get(queryHandingChartDataSon.getTitle());
                                    int length = now.getData().length + queryHandingChartDataSon.getData().length;
                                    double[][] data = new double[length][];
                                    System.arraycopy(now.getData(),0,data,0,now.getData().length );
                                    System.arraycopy(queryHandingChartDataSon.getData(),0,
                                            data,now.getData().length,queryHandingChartDataSon.getData().length);
                                    now.setData(data);
                                }

                            }
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                    }
                }
                // map转List
                List<QueryHandingChartDataSon> queryHandingChartDataSonList = new ArrayList<>(map.values());

                QueryHandingChartDataVO queryHandingChartDataVO = new QueryHandingChartDataVO();
                queryHandingChartDataVO.setChartDataSonList(queryHandingChartDataSonList);
                queryHandingChartDataVO.setTestName(tableHead.getTableName());
                queryHandingTableDataVOList.add(queryHandingChartDataVO);
            }



        }
        return queryHandingTableDataVOList;
    }

    @Override
    public CompareHandingTableDataVO compareHandingTableData(CompareHandingTableDataDTO dto) {
        CompareHandingTableDataVO compareHandingTableDataVO = new CompareHandingTableDataVO();
        // 1、根据试验项目试验结果表
        LambdaQueryWrapper<TableHead> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(TableHead::getId)
                .eq(TableHead::getTableType,2)
                .eq(TableHead::getBusiness_id,dto.getBusinessId());
        TableHead tableHead = tableHeadService.getOne(queryWrapper);

        // 3、根据试验结果表id查询实验结果表数据
        LambdaQueryWrapper<TableData> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.select(TableData::getCarId,TableData::getRowInfo)
                .eq(TableData::getTableHeadId,tableHead.getId())
                .in(TableData::getCarId,dto.getCarIdList());
        List<TableData> tableDataList = tableDataService.list(queryWrapper2);
        // 根据车辆id分组
        Map<Long,List<TableData>> groupMap = tableDataList.stream().collect(Collectors.groupingBy(TableData::getCarId));

        // 取第一列封装表头
        List<String> tableHeadList = new ArrayList<>();
        int a = 0;
        for (TableData tableData : groupMap.get(dto.getCarIdList().get(0))) {
            if(a++ < 2)continue; // 去除前两行非表头数据
            Map dataMap = JSONObject.parseObject(tableData.getRowInfo(), Map.class);
            String head = "";
            if(dataMap.containsKey(0))
                head = dataMap.get(0).toString();
            tableHeadList.add(head);
        }
        compareHandingTableDataVO.setTableHeadList(tableHeadList);

        // 封装数据行
        Map<Long, Map<String,String>> data = new HashMap<>();
        for (Long key : groupMap.keySet()) {
            List<TableData> currentList = groupMap.get(key);
            Map<String,String> currentMap = new HashMap<>();
            int j = 0;
            for (TableData tableData : currentList) {
                if(j++ < 2)continue;
                Map dataMap = JSONObject.parseObject(tableData.getRowInfo(), Map.class);
                if(dataMap.containsKey(1) && dataMap.containsKey(0))
                    currentMap.put(dataMap.get(0).toString(),dataMap.get(1).toString());
            }
            data.put(key,currentMap);
        }
        compareHandingTableDataVO.setData(data);
        return compareHandingTableDataVO;
    }

    /**
     * 批量上传操纵稳定性原始数据文件
     */
    @Override
    public void uploadReSourceFile(UploadReSourceFileDTO dto) {
        // 若localFilePath不为空表示异步上传
        MultipartFile multipartFile = StringUtils.isNotEmpty(dto.getLocalFilePath()) ? FileUtil.fileToMultipartFile(dto.getLocalFilePath()) : dto.getMultipartFile();
        LocalDateTime startTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setId(dto.getTaskId());
        try{
            if(multipartFile.getOriginalFilename().contains(".zip")){
                dto.setMultipartFile(multipartFile);
                attachmentService.uploadHandingZipAttachment(dto);
            }else{
                // 上传文件
                Attachment attachment = attachmentService.uploadAttachment(
                        dto.getMultipartFile(),
                        AttachmentConstant.OWNER_TYPE_CAR,
                        dto.getOwnerId(),
                        dto.getBusinessType(),
                        dto.getOperId()
                );

                // 保存试验场景信息
                ExperimentInfo experimentInfo = ExperimentInfo.builder()
                        .attachId(attachment.getId())
                        .testSpeed(dto.getTestSpeed())
                        .dataType(dto.getDataType())
                        .steering(dto.getSteering())
                        .build();
                experimentService.save(experimentInfo);
            }
            taskQueue.setStat(2);
        }catch(Exception e){
            e.printStackTrace();
            taskQueue.setRemark(e.getMessage());
            taskQueue.setStat(3);
        }finally{
            Duration duration = Duration.between(startTime,LocalDateTime.now());
            taskQueue.setConsumeTime(duration.getSeconds());
            taskQueueService.updateById(taskQueue);
            // 删除临时文件
            if(StringUtils.isNotEmpty(dto.getLocalFilePath()))
                FileUtil.deleteFile(dto.getLocalFilePath());
        }
        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setHandingUploadStat(2);
        carBasicParam.setId(dto.getOwnerId());
        carBasicParamService.updateById(carBasicParam);
    }


}
