package com.singularity.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.dao.CarBasicParamMapper;
import com.singularity.dao.MenuInfoMapper;
import com.singularity.pojo.dto.homePage.GetTrendsDTO;
import com.singularity.pojo.dto.homePage.TestsNumberDTO;
import com.singularity.pojo.dto.homePage.TrendsDTO;
import com.singularity.pojo.entity.CarBasicParam;
import com.singularity.pojo.entity.MenuInfo;
import com.singularity.service.CarBasicParamService;
import com.singularity.service.HomePageService;
import com.singularity.pojo.vo.nvh.QueryTestCountVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.List;

@Service
public class HomePageServiceImpl implements HomePageService {

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource
    private MenuInfoMapper menuInfoMapper;

    @Resource
    private CarBasicParamMapper carBasicParamMapper;

    @Override
    public TestsNumberDTO getTestsNumber() {

        Date currentDate = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        String currentMonth = sdf.format(currentDate);

        TestsNumberDTO testsNumberDTO = new TestsNumberDTO();
        // 试验总数量
        testsNumberDTO.setTotalNumberOfTests(this.getTestTotalNumber());

        // 本月试验总数量
        testsNumberDTO.setNewTotalNumberOfTests(this.getNewTestNumber(currentMonth));

        // 车辆总数量
        testsNumberDTO.setVehicleFleet(carBasicParamService.count());

        // 本月新增车辆总数量
        LambdaQueryWrapper<CarBasicParam> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.like(CarBasicParam::getCreatedAt,currentMonth);
        testsNumberDTO.setNewVehicleFleet(carBasicParamService.count(queryWrapper2));

        return testsNumberDTO;
    }

    private Long getTestTotalNumber(){

        LambdaQueryWrapper<MenuInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(MenuInfo::getLevel,3);
        Long nvhCount = menuInfoMapper.selectCount(queryWrapper);

        // 查询K&C试验分布情况，数量为已上传车辆总数
        LambdaQueryWrapper<CarBasicParam> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.eq(CarBasicParam::getKcUploadStat,2);
        Long kcCount = carBasicParamMapper.selectCount(queryWrapper2);

        // 查询行驶性能试验分布情况
        Long drivingCount = carBasicParamMapper.queryDrivingTestCountNum(null);

        // 查询主观评价分布情况，数量为已上传车辆总数
        Long subjectiveCount = carBasicParamMapper.querySubjectiveTestCountNum(null);

        return nvhCount + kcCount + drivingCount + subjectiveCount;
    }

    private Long getNewTestNumber(String currentMonth){
        LambdaQueryWrapper<MenuInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.like(MenuInfo::getCreateTime,currentMonth);
        Long nvhCount = menuInfoMapper.selectCount(queryWrapper);

        // 查询K&C试验分布情况，数量为已上传车辆总数
        LambdaQueryWrapper<CarBasicParam> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.like(CarBasicParam::getCreatedAt,currentMonth);
        Long kcCount = carBasicParamMapper.selectCount(queryWrapper2);

        // 查询行驶性能试验分布情况
        Long drivingCount = carBasicParamMapper.queryDrivingTestCountNum(currentMonth);

        // 查询主观评价分布情况，数量为已上传车辆总数
        Long subjectiveCount = carBasicParamMapper.querySubjectiveTestCountNum(currentMonth);

        return nvhCount + kcCount + drivingCount + subjectiveCount;
    }

    @Override
    public TrendsDTO getTrends(GetTrendsDTO dto) {

        Map<String,Integer> kc = new LinkedHashMap<>();
        for(int i = 1; i < 13; i++){
            kc.put(i + "月",0);
        }
        Map<String,Integer> nvh = new LinkedHashMap<>(kc);
        Map<String,Integer> subjective = new LinkedHashMap<>(kc);
        Map<String,Integer> ride = new LinkedHashMap<>(kc);

        // 查询NVH试验分布情况,数量为工况菜单数
        List<QueryTestCountVO> nvhCountList = menuInfoMapper.queryNVHTestCount(dto.getYear());
        for (QueryTestCountVO queryTestCountVO : nvhCountList) {
            nvh.put(queryTestCountVO.getMonth() + "月", queryTestCountVO.getCount());
        }

        // 查询K&C试验分布情况，数量为已上传车辆总数
        List<QueryTestCountVO> kcCountList = carBasicParamMapper.queryKCTestCount(dto.getYear());
        for (QueryTestCountVO queryTestCountVO : kcCountList) {
            kc.put(queryTestCountVO.getMonth() + "月", queryTestCountVO.getCount());
        }

        // 查询行驶性能试验分布情况
        List<QueryTestCountVO> drivingCountList = carBasicParamMapper.queryDrivingTestCount(dto.getYear());
        for (QueryTestCountVO queryTestCountVO : drivingCountList) {
            ride.put(queryTestCountVO.getMonth() + "月", queryTestCountVO.getCount());
        }

        // 查询主观评价分布情况，数量为已上传车辆总数
        List<QueryTestCountVO> subjectiveCountList = carBasicParamMapper.querySubjectiveTestCount(dto.getYear());
        for (QueryTestCountVO queryTestCountVO : subjectiveCountList) {
            subjective.put(queryTestCountVO.getMonth() + "月", queryTestCountVO.getCount());
        }

        TrendsDTO trendsDTO = new TrendsDTO();
        trendsDTO.setKC(kc);
        trendsDTO.setNVH(nvh);
        trendsDTO.setRide(ride);
        trendsDTO.setSubjective(subjective);
        return trendsDTO;
    }

}
