package com.singularity.service.impl;

import cn.dev33.satoken.exception.SaTokenException;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.car.nvh.DrawNVHChart;
import com.mathworks.toolbox.javabuilder.MWCellArray;
import com.singularity.common.entity.ChartData;
import com.singularity.common.vo.nvh.ReadNVHFileVO;
import com.singularity.dao.AttachmentMapper;
import com.singularity.dao.MenuInfoMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.listener.NVHObjectiveElectricListener;
import com.singularity.listener.NVHObjectiveFuelListener;
import com.singularity.listener.NVHSubjectiveFuelListener;
import com.singularity.pojo.bo.attachment.AttachmentBO;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.dto.menuInfo.MenuInfoDTO;
import com.singularity.pojo.dto.nvh.*;
import com.singularity.pojo.entity.TaskQueue;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.enumeration.NVHChartTypeEnum;
import com.singularity.pojo.param.nvh.NVHContentAddParam;
import com.singularity.pojo.param.nvh.NVHSelectParam;
import com.singularity.pojo.param.nvh.NVHUpdateParam;
import com.singularity.pojo.param.nvh.NVHUploadFileParam;
import com.singularity.pojo.vo.nvh.*;
import com.singularity.service.*;
import com.singularity.util.FileUtil;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.*;
import java.lang.reflect.Method;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

@Log4j2
@Service
public class NVHServiceImpl implements NVHService {

    @Resource
    private MenuInfoMapper menuInfoMapper;

    @Resource
    private AttachmentService attachmentService;

    @Value("${file.path}")
    private String filePath;

    @Resource
    private ChartDataService chartDataService;

    @Resource
    private TableDataService tableDataService;

    @Resource(name = "threadPool")
    private ThreadPoolTaskExecutor threadPool;

    @Resource
    private MenuInfoService menuInfoService;

    @Resource
    private TaskQueueService taskQueueService;

    @Resource
    private NVHEvaluationService nvhEvaluationService;

    @Resource
    private AttachmentMapper attachmentMapper;

    @Resource
    private CarBasicParamService carBasicParamService;

    /**
     * 查询NVH菜单树
     */
    @Override
    public List<GetNVHMenuTreeVO> getNVHMeuTreeList(GetNVHMenuTreeDTO dto) {
        // 1、分页查询第一级菜单树
        List<GetNVHMenuTreeVO> menuInfoList = menuInfoMapper.selectFirstMenuInfo(dto);

        // 2、根据第一级菜单查询下级菜单
        if (menuInfoList.size() < 1) return null;
        List<Integer> menuIdList = menuInfoList.stream().map(GetNVHMenuTreeVO::getId).collect(Collectors.toList());
        LambdaQueryWrapper<MenuInfo> queryWrapper2 = Wrappers.lambdaQuery();
        for (Integer integer : menuIdList) {
            queryWrapper2.or().like(MenuInfo::getParentIds, "," + integer);
        }
        queryWrapper2.or().in(MenuInfo::getId, menuIdList);
        List<MenuInfo> allMenuList = menuInfoMapper.selectList(queryWrapper2);

        // 3、根据所有菜单idList查询所有附件列表
        List<Integer> allMenuIdList = allMenuList.stream().map(MenuInfo::getId).collect(Collectors.toList());
        LambdaQueryWrapper<Attachment> queryWrapper3 = Wrappers.lambdaQuery();
        queryWrapper3.in(Attachment::getOwnerId, allMenuIdList)
                .in(Attachment::getBusinessType, 2, 58, 59);
        List<Attachment> attachment = attachmentService.list(queryWrapper3);

        // 4、分组得到每个菜单对应的附件集合
        Map<Long, List<Attachment>> map = attachment.stream().collect(Collectors.groupingBy(Attachment::getOwnerId));
        List<GetNVHMenuTreeVO> nvhMenuTreeVOArrayList = new ArrayList<>();
        for (MenuInfo menuInfo : allMenuList) {
            GetNVHMenuTreeVO getNVHMenuTreeVO = new GetNVHMenuTreeVO();
            BeanUtil.copyProperties(menuInfo, getNVHMenuTreeVO);
            if (map.containsKey((long) menuInfo.getId()))
                getNVHMenuTreeVO.setAttachmentList(map.get((long) menuInfo.getId()));
            nvhMenuTreeVOArrayList.add(getNVHMenuTreeVO);
        }

        // 5、将所有菜单封装为菜单树
        return this.convertMenuTree(nvhMenuTreeVOArrayList);
    }

    /**
     * 两层循环封装菜单树
     *
     * @param treeNodes 需要封装的菜单集合
     */
    private List<GetNVHMenuTreeVO> convertMenuTree(List<GetNVHMenuTreeVO> treeNodes) {
        List<GetNVHMenuTreeVO> trees = new ArrayList<>();
        for (GetNVHMenuTreeVO treeNode : treeNodes) {
            if (Objects.equals(treeNode.getParentId(), 0)) {
                trees.add(treeNode);
            }
            for (GetNVHMenuTreeVO it : treeNodes) {
                if (Objects.equals(treeNode.getId(), it.getParentId())) {
                    if (treeNode.getChildren() == null || treeNode.getChildren().size() == 0) {
                        treeNode.setChildren(new ArrayList<>());
                    }
                    List<GetNVHMenuTreeVO> treeNodeChildren = treeNode.getChildren();
                    treeNodeChildren.add(it);
                    treeNode.setChildren(treeNodeChildren);
                }
            }
        }
        return trees;
    }

    @Override
    public void getMenuInfoDTOS(List<MenuInfoDTO> result, List<MenuInfoDTO> allRightMenu, Long parentId, Long level) {
        List<MenuInfoDTO> collect = allRightMenu.stream()
                .filter(m -> Objects.equals(m.getParentId(), parentId) && Objects.equals(m.getLevel(), level))
                .collect(Collectors.toList());
        result.addAll(collect);
//        if (level == 5) {
//            getFileList(collect);
//            return;
//        }
        for (MenuInfoDTO menuInfoDTO : collect) {
            getMenuInfoDTOS(menuInfoDTO.getMenuInfoDTOList(), allRightMenu, menuInfoDTO.getId(), level + 1);
        }
    }

    @Override
    public long addNVHContentList(NVHContentAddParam addParam) {
        // 查询父菜单信息
        LambdaQueryWrapper<MeasureInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(MeasureInfo::getId, addParam.getParentId());
        MenuInfo parentMenuInfo = menuInfoMapper.selectById(addParam.getParentId());

        MenuInfo menuInfo = BeanUtil.copyProperties(addParam, MenuInfo.class);
        LocalDateTime localDateTime = LocalDateTime.now();
        menuInfo.setCreateTime(localDateTime);
        menuInfo.setUpdateTime(localDateTime);
        String parentIds = ObjectUtil.isEmpty(parentMenuInfo) ? "0" : parentMenuInfo.getParentIds() + "," + parentMenuInfo.getId();
        menuInfo.setParentIds(parentIds);
        menuInfoService.save(menuInfo);
        return menuInfo.getId();
    }

    @Override
    public List<MenuInfoDTO> getNVHContentListForCompare(NVHSelectParam selectParam) {
        // 根据车辆idList查询所有关联菜单
        LambdaQueryWrapper<MenuInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(CollectionUtil.isNotEmpty(selectParam.getCarIdList()),MenuInfo::getCarId,selectParam.getCarIdList());
        List<MenuInfo> menuInfoList = menuInfoService.list(queryWrapper);

        // 3、根据所有菜单idList查询所有附件列表
        List<Integer> allMenuIdList = menuInfoList.stream().map(MenuInfo::getId).collect(Collectors.toList());
        LambdaQueryWrapper<Attachment> queryWrapper3 = Wrappers.lambdaQuery();
        queryWrapper3.in(Attachment::getOwnerId, allMenuIdList)
                .in(Attachment::getBusinessType, 2, 20, 21);
        List<Attachment> attachment = attachmentService.list(queryWrapper3);

        // 分组得到每个菜单对应的附件集合
        Map<Long, List<Attachment>> map = attachment.stream().collect(Collectors.groupingBy(Attachment::getOwnerId));
        List<MenuInfoDTO> menuInfoDTOList = new ArrayList<>();
        for (MenuInfo menuInfo : menuInfoList) {
            MenuInfoDTO menuInfoDTO = new MenuInfoDTO();
            BeanUtil.copyProperties(menuInfo, menuInfoDTO);
            if (map.containsKey((long) menuInfo.getId()))
                menuInfoDTO.setFileList(map.get((long) menuInfo.getId()));
            menuInfoDTOList.add(menuInfoDTO);
        }

        // 5、将所有菜单封装为菜单树
        return this.convertMenuTree2(menuInfoDTOList);
    }

    /**
     * 两层循环封装菜单树
     *
     * @param treeNodes 需要封装的菜单集合
     */
    private List<MenuInfoDTO> convertMenuTree2(List<MenuInfoDTO> treeNodes) {
        List<MenuInfoDTO> trees = new ArrayList<>();
        for (MenuInfoDTO treeNode : treeNodes) {
            if (Objects.equals(treeNode.getParentId(), 0L)) {
                trees.add(treeNode);
            }
            for (MenuInfoDTO it : treeNodes) {
                if (Objects.equals(treeNode.getId(), it.getParentId())) {
                    if (treeNode.getMenuInfoDTOList() == null || treeNode.getMenuInfoDTOList().size() == 0) {
                        treeNode.setMenuInfoDTOList(new ArrayList<>());
                    }
                    List<MenuInfoDTO> treeNodeChildren = treeNode.getMenuInfoDTOList();
                    treeNodeChildren.add(it);
                    treeNode.setMenuInfoDTOList(treeNodeChildren);
                }
            }
        }
        return trees;
    }

    @SneakyThrows
    @Override
    public void uploadNVHFile(NVHUploadFileParam dto) {
        // 上传NVH性能客观评价文件
        if(dto.getFileType() == 1){
            // 上传原始数据文件
            log.info("读取NVH关键指标数据文件");
            Attachment attachment = attachmentService.uploadAttachment(
                    dto.getFile(),
                    AttachmentConstant.OWNER_TYPE_MENU,
                    dto.getOwnerId(),
                    dto.getBusinessId()
            );
            if(dto.getBusinessId() == 59){
                // 读取电动车关键指标数据,并保存至数据库，用作数据对比使用
                NVHObjectiveElectricListener electricListener = new NVHObjectiveElectricListener(attachment.getId());
                EasyExcel.read(dto.getFile().getInputStream(), electricListener)
                        .headRowNumber(2).sheet().doRead();
                nvhEvaluationService.saveBatch(electricListener.getNvhEvaluationList());
            }
            if(dto.getBusinessId() == 58){
                // 读取燃油车关键指标数据
                NVHObjectiveFuelListener fuelListener = new NVHObjectiveFuelListener(attachment.getId());
                EasyExcel.read(dto.getFile().getInputStream(), fuelListener)
                        .headRowNumber(2).sheet().doRead();
                nvhEvaluationService.saveBatch(fuelListener.getNvhEvaluationList());
            }
        }else{ // 上传NVH绘图数据文件
            try{
                log.info("开始调用matlab函数读取绘图数据");
                InputStream inputStream = dto.getFile().getInputStream();
                String drawType = dto.getDrawType();
                if(StringUtils.isEmpty(drawType)){
                    throw new ToolsBusinessWarn("绘图类型错误");
                }
                switch (drawType) {
                    case "readLineFile":
                        this.readLineFile(dto.getOwnerId(),drawType, inputStream);
                        break;
                    case "readStatFile":
                        this.readStatFile(dto.getOwnerId(),drawType, inputStream);
                        break;
                    case "read3DFile":
                        this.read3DFile(dto.getOwnerId(),drawType, inputStream);
                        break;
                }
                log.info("读取绘图数据成功");
            }catch (Exception e){
                e.printStackTrace();
                throw new ToolsBusinessWarn("调用matlab读取绘图数据失败");
            }
        }

        MenuInfo menuInfo = menuInfoService.getById(dto.getOwnerId());

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setNvhUploadStat(2);
        carBasicParam.setId((long)menuInfo.getCarId());
        carBasicParamService.updateById(carBasicParam);
        log.info("文件上传完毕");
    }

    @Override
    @Transactional
    public void deleteNVHContentList(Long id) {
        //判断是否存在子菜单
        LambdaQueryWrapper<MenuInfo> lqw = new LambdaQueryWrapper<>();
        lqw.eq(MenuInfo::getParentId, id);
        if (menuInfoMapper.exists(lqw)) {
            throw new SaTokenException("存在子菜单，无法删除");
        }
        //更新菜单
        menuInfoMapper.deleteById(id);
        //删除测点数据
        chartDataService.removeByOwner(AttachmentConstant.OWNER_TYPE_MENU,id);
        //删除文件
        attachmentService.removeAttachment(AttachmentConstant.OWNER_TYPE_MENU, id);
    }

    @Override
    public int updateNVHContentList(NVHUpdateParam updateParam) {
        MenuInfo menuInfo = BeanUtil.copyProperties(updateParam, MenuInfo.class);

        //要删除的
        if (updateParam.getDeleteAttachmentIds().size() != 0) {
            attachmentService.removeAttachmentById(updateParam.getDeleteAttachmentIds());
        }
        //要修改的
        if (updateParam.getAttachmentIds().size() != 0) {
            attachmentService.updateAttachment(updateParam.getAttachmentIds(), AttachmentConstant.OWNER_TYPE_MENU, updateParam.getId());
        }
        return menuInfoMapper.updateById(menuInfo);
    }

    @Override
    public int deleteNVHFile(Long id) {
        //1、删除附件数据文件
        Attachment attachment = attachmentService.getAttachmentById(id.intValue());
        LambdaQueryWrapper<Attachment> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ObjectUtil.isNotEmpty(attachment.getOwnerId()), Attachment::getOwnerId, attachment.getId())
                .eq(Attachment::getOwnerType, "attachment");
        attachmentService.remove(queryWrapper);

        // 2、删除附件关键指标数据
        LambdaQueryWrapper<TableData> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.eq(TableData::getCarId,-id);
        tableDataService.remove(queryWrapper2);

        // 2、删除附件
        return attachmentService.removeAttachmentById(id);
    }

    /**
     * 异步批量上传文件
     */
    @Override
    public void asyncUploadFileList(UploadFileListDTO dto){
        // 创建队列信息
        LocalDateTime localDateTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setFileName(dto.getOriginalFilename());
        taskQueue.setStat(1);
        taskQueue.setOperId(dto.getOperId());
        taskQueueService.save(taskQueue);
        dto.setStartTime(localDateTime);
        dto.setTaskId(taskQueue.getId());
        // 异步调用任务
        threadPool.execute(() -> {
            try {
                // 核心任务
                this.uploadFileList(dto);

                // 任务执行完毕修改任务状态
                log.info("文件上传完毕");
                LocalDateTime now = LocalDateTime.now();
                Duration duration = Duration.between(dto.getStartTime(),now);
                LambdaUpdateWrapper<TaskQueue> updateWrapper = Wrappers.lambdaUpdate();
                updateWrapper.set(TaskQueue::getStat,2)
                        .set(TaskQueue::getConsumeTime,duration.toMillis()/1000)
                        .eq(TaskQueue::getId,dto.getTaskId());
                taskQueueService.update(updateWrapper);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
                // 修改任务状态
                LambdaUpdateWrapper<TaskQueue> updateWrapper = Wrappers.lambdaUpdate();
                updateWrapper.set(TaskQueue::getStat,3)
                        .set(TaskQueue::getRemark,e.getMessage())
                        .eq(TaskQueue::getId,taskQueue.getId());
                taskQueueService.update(updateWrapper);
            }finally {
                FileUtil.deleteFile(dto.getLocalFilePath());
            }
        });
        log.info("操作成功");
    }

    @SneakyThrows
    @Transactional
    public void uploadFileList(UploadFileListDTO dto) {
        log.info("开始异步上传NVHzip数据文件");
        // 1、获取文件格式
        File file = new File(dto.getLocalFilePath());
        Map<String, Integer> map = new HashMap<>();
        InputStream inputStream = new FileInputStream(file);
        ZipInputStream zipInputStream = new ZipInputStream(inputStream, Charset.forName("GBK"));
        ZipEntry zipEntry = zipInputStream.getNextEntry();
        while (zipEntry != null) {
            if (zipEntry.isDirectory()) {
                String[] filePath = zipEntry.getName().split("/");
                String menuName = filePath[filePath.length - 1];
                MenuInfo menuInfo = new MenuInfo();
                LocalDateTime localDateTime = LocalDateTime.now();
                menuInfo.setCreateTime(localDateTime);
                menuInfo.setUpdateTime(localDateTime);
                menuInfo.setMenuName(menuName);
                menuInfo.setCarId(dto.getCarId());
                menuInfo.setLevel(filePath.length);
                if(filePath.length == 1){
                    if(map.containsKey(filePath[0] + (0))){
                        throw new ToolsBusinessWarn("文件夹名称重复");
                    }
                    menuInfo.setParentIds("0");
                    menuInfo.setParentId(0);
                    menuInfoService.save(menuInfo);
                    map.put(filePath[0] + 0,menuInfo.getId());
                }else{
                    Integer parentId = map.get(filePath[filePath.length - 2] + (filePath.length - 2));
                    menuInfo.setParentId(parentId);
                    StringBuilder parentIdList = new StringBuilder();
                    parentIdList.append("0,");
                    for (int i = 0; i < filePath.length - 1; i++) {
                        if (i != 0) {
                            parentIdList.append(",");
                        }
                        parentIdList.append(map.get(filePath[i] + i));
                    }
                    menuInfo.setParentIds(parentIdList.toString());
                    menuInfoService.save(menuInfo);
                    if(ObjectUtil.isNotEmpty(map.get(menuInfo.getMenuName()))){
                        throw new ToolsBusinessWarn("文件夹名称重复");
                    }
                    map.put(menuName + (filePath.length - 1), menuInfo.getId());
                }
            } else { // 如果是文件则添加文件
                String[] currentFilePath = zipEntry.getName().split("/");
                Long ownerId = (long)map.get(currentFilePath[currentFilePath.length - 2] + (currentFilePath.length - 2));
                String fileName = currentFilePath[currentFilePath.length - 1];
                String[] splitFileName = fileName.split("_");
                String chartTypeName = splitFileName[0];

                // 将zip中的文件下载到本地
                InputStream zipEntryStream = new ByteArrayInputStream(IOUtils.toByteArray(zipInputStream));
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String localFilePath = filePath + uuid + ".xlsx";
                File localFile = new File(localFilePath);
                try(FileOutputStream outputStream = new FileOutputStream(localFile)){
                    byte[] buff = new byte[1024];
                    int len;
                    while((len = zipEntryStream.read(buff)) != -1) {
                        outputStream.write(buff,0,len);
                    }
                }catch (Exception e){
                    throw new ToolsBusinessWarn("文件格式转换失败");
                }

                try{
                    if(fileName.contains("NVH性能客观评价电动及混动版")){
                        InputStream localFileInputStream = new FileInputStream(localFile);
                        Attachment attachment = attachmentService.uploadAttachment(
                                localFileInputStream,
                                fileName,
                                localFile.length(),
                                AttachmentConstant.OWNER_TYPE_MENU,
                                ownerId,
                                21);
                        localFileInputStream.close();
                        // 读取电动车关键指标数据,并保存至数据库，用作数据对比使用
                        NVHObjectiveElectricListener electricListener = new NVHObjectiveElectricListener(attachment.getId());
                        EasyExcel.read(new FileInputStream(localFile), electricListener)
                                .headRowNumber(14).sheet().doRead();
                        nvhEvaluationService.saveBatch(electricListener.getNvhEvaluationList());
                        zipInputStream.closeEntry();
                        zipEntry = zipInputStream.getNextEntry();
                        continue;
                    }
                    if(fileName.contains("NVH性能客观评价燃油版")){
                        InputStream localFileInputStream = new FileInputStream(localFile);
                        Attachment attachment = attachmentService.uploadAttachment(
                                localFileInputStream,
                                fileName,
                                localFile.length(),
                                AttachmentConstant.OWNER_TYPE_MENU,
                                ownerId,
                                20);
                        localFileInputStream.close();
                        // 读取燃油车关键指标数据
                        NVHSubjectiveFuelListener fuelListener = new NVHSubjectiveFuelListener(attachment.getId());
                        EasyExcel.read(new FileInputStream(localFile), fuelListener)
                                .headRowNumber(48).sheet().doRead();
                        nvhEvaluationService.saveBatch(fuelListener.getNvhEvaluationList());
                        zipInputStream.closeEntry();
                        zipEntry = zipInputStream.getNextEntry();
                        continue;
                    }
                }catch (Exception e){
                    e.printStackTrace();
                    log.error("读取NVH性能客观评价文件失败");
                }


                // 2、调用matlab函数读取文件得到绘图数据
                log.info("开始调用matlab函数读取绘图数据");
                FileInputStream fileInputStream = new FileInputStream(localFile);
                String drawType = NVHChartTypeEnum.getCodeByName(chartTypeName);
                if(StringUtils.isEmpty(drawType)){
                    zipInputStream.closeEntry();
                    zipEntry = zipInputStream.getNextEntry();
                    continue;
                }
                try{
                    switch (Objects.requireNonNull(drawType)) {
                        case "readLineFile":
                            this.readLineFile(ownerId,drawType, fileInputStream);
                            break;
                        case "readStatFile":
                            this.readStatFile(ownerId,drawType, fileInputStream);
                            break;
                        case "read3DFile":
                            this.read3DFile(ownerId,drawType, fileInputStream);
                            break;
                    }
                    log.info("读取绘图数据成功");
                }catch (Exception e){
                    e.printStackTrace();
                    log.info("错误文件路径" + zipEntry.getName());
                    throw new ToolsBusinessWarn("调用matlab读取绘图数据错误，错误文件"+ zipEntry.getName());
                }

            }
            zipInputStream.closeEntry();
            zipEntry = zipInputStream.getNextEntry();
        }
        zipInputStream.close();
        inputStream.close();

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setNvhUploadStat(2);
        carBasicParam.setId((long)dto.getCarId());
        carBasicParamService.updateById(carBasicParam);
    }

    /**
     * 读取NVH数据文件
     */
    @Override
    @SneakyThrows
    public List<ReadNVHFileVO> readNVHFile(ReadNVHFileDTO dto) {
        List<ReadNVHFileVO> readNVHFileVOList = new ArrayList<>();
        LambdaQueryWrapper<ChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(ChartData::getAttachId, dto.getAttachIdList());
        List<ChartData> chartDataList = chartDataService.list(queryWrapper);
        for (ChartData chartData : chartDataList) {
            ReadNVHFileVO readNVHFileVO = (ReadNVHFileVO) FileUtil.byteToObject(chartData.getChartData());
            readNVHFileVOList.add(readNVHFileVO);
        }
        return readNVHFileVOList;
    }

    /**
     * 批量读取NVH数据文件
     */
    @Override
    @SneakyThrows
    public ReadNVHFileListVO readNVHFileList(ReadNVHFileListDTO dto) {
        // 1、根据附件idList查询数据对象
        ReadNVHFileListVO readNVHFileListVO = new ReadNVHFileListVO();
        LambdaQueryWrapper<ChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(ChartData::getAttachId, dto.getAttachIdList());
        List<ChartData> chartDataList = chartDataService.list(queryWrapper);
        List<String> legendList = new ArrayList<>();

        // 2、遍历数据对象拼接数据集合
        for (ChartData chartData : chartDataList) {
            ReadNVHFileVO readNVHFileVO = (ReadNVHFileVO) FileUtil.byteToObject(chartData.getChartData());
            if (readNVHFileListVO.getData() != null) {
                readNVHFileListVO.setData(ArrayUtils.addAll(readNVHFileVO.getData(), readNVHFileListVO.getData()));
            } else {
                readNVHFileListVO.setData(readNVHFileVO.getData());
            }
            legendList.add(readNVHFileVO.getLegend());

            readNVHFileListVO.setLabelX(readNVHFileVO.getLabelX());
            readNVHFileListVO.setLabelY(readNVHFileVO.getLabelY());
            readNVHFileListVO.setChartType(readNVHFileVO.getChartType());
        }
        readNVHFileListVO.setLegendList(legendList);
        return readNVHFileListVO;
    }

    @Override
    public CompareNVHSubjectiveFileVO compareNVHSubjectiveFile(CompareNVHSubjectiveFileDTO dto) {
        // 根据附件idList查询所有主观评价信息
        LambdaQueryWrapper<NVHEvaluation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(NVHEvaluation::getAttachId,dto.getMap().keySet());
        List<NVHEvaluation> nvhEvaluationList = nvhEvaluationService.list(queryWrapper);

        Map<Long, List<NVHEvaluation>> map = nvhEvaluationList.stream()
                .collect(Collectors.groupingBy(NVHEvaluation::getAttachId));

        Map<Integer, Map<String, List<String>>> data  = new HashMap<>();
        for (Long attachId : dto.getMap().keySet()) {
            List<NVHEvaluation> paramList = map.get(attachId);
            if (Objects.isNull(paramList) || paramList.size() < 1) {
                continue;
            }
            Map<String, List<String>> paramMap = new HashMap<>();
            for (NVHEvaluation nvhEvaluation : paramList) {
                List<String> list = new ArrayList<>();
                list.add(nvhEvaluation.getScore().toString());
                list.add(nvhEvaluation.getRemark());
                StringBuilder stringBuilder = new StringBuilder();
                if(StringUtils.isNotEmpty(nvhEvaluation.getFirstCondition()))stringBuilder.append(nvhEvaluation.getFirstCondition());
                if(StringUtils.isNotEmpty(nvhEvaluation.getSecondCondition()))stringBuilder.append(nvhEvaluation.getSecondCondition());
                if(StringUtils.isNotEmpty(nvhEvaluation.getItem()))stringBuilder.append(nvhEvaluation.getItem());
                paramMap.put(stringBuilder.toString() , list);
            }
            data.put(dto.getMap().get(attachId), paramMap);
        }
        CompareNVHSubjectiveFileVO compareKCEvaluationInfoVO = new CompareNVHSubjectiveFileVO();
        compareKCEvaluationInfoVO.setData(data);
        return compareKCEvaluationInfoVO;
    }

    @Override
    public QueryNVHDataFileListVO queryNVHDataFileList(QueryNVHDataFileListDTO dto) {
        // 查询与NVH菜单关联的NVH性能主观评价附件
        List<AttachmentBO> attachmentList = attachmentMapper.queryNVHDataFileList(dto);
        Map<Long,List<AttachmentBO>> attachmentMap = attachmentList.stream()
                .collect(Collectors.groupingBy(AttachmentBO::getCarId));
        QueryNVHDataFileListVO queryNVHDataFileListVO = new QueryNVHDataFileListVO();
        queryNVHDataFileListVO.setAttachmentMap(attachmentMap);
        return queryNVHDataFileListVO;
    }

    @Override
    public CompareNVHObjectiveFileVO compareNVHObjectiveFile(CompareNVHObjectiveFileDTO dto) {
        // 根据附件idList查询所有主观评价信息
        LambdaQueryWrapper<NVHEvaluation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(NVHEvaluation::getAttachId,dto.getMap().keySet());
        List<NVHEvaluation> nvhEvaluationList = nvhEvaluationService.list(queryWrapper);

        Map<Long, List<NVHEvaluation>> map = nvhEvaluationList.stream()
                .collect(Collectors.groupingBy(NVHEvaluation::getAttachId));

        Map<Integer, Map<String, List<String>>> data  = new HashMap<>();
        for (Long attachId : dto.getMap().keySet()) {
            List<NVHEvaluation> paramList = map.get(attachId);
            if (Objects.isNull(paramList) || paramList.size() < 1) {
                continue;
            }
            Map<String, List<String>> paramMap = new HashMap<>();
            for (NVHEvaluation nvhEvaluation : paramList) {
                List<String> list = new ArrayList<>();
                list.add(nvhEvaluation.getTestValue());
                StringBuilder stringBuilder = new StringBuilder();
                if(nvhEvaluation.getFirstCondition() != null)stringBuilder.append(nvhEvaluation.getFirstCondition());
                if(nvhEvaluation.getTestIndex() != null){
                    stringBuilder.append(nvhEvaluation.getTestIndex());
                }else{
                    stringBuilder.append(nvhEvaluation.getSecondCondition());
                }
                paramMap.put(stringBuilder.toString() , list);
            }
            data.put(dto.getMap().get(attachId), paramMap);
        }
        CompareNVHObjectiveFileVO compareNVHObjectiveFileVO = new CompareNVHObjectiveFileVO();
        compareNVHObjectiveFileVO.setData(data);
        return compareNVHObjectiveFileVO;
    }

    @SneakyThrows
    private void readStatFile(Long ownerId,String drawType, InputStream inputStream) {
        // 1、io流转换为本地文件
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String localFileName = filePath + uuid + "-" + drawType + ".xlsx";
        FileUtil.downloadToLocal(inputStream, localFileName);

        // 2.调用matlab函数读取excel读取数据
        DrawNVHChart drawNVHChart = new DrawNVHChart();
        Method method = drawNVHChart.getClass().getMethod(drawType, int.class, Object[].class);
        Object[] object = {
                localFileName
        };
        Object[] result = (Object[]) method.invoke(drawNVHChart, 7, object);
        // 4.封装结果集
        MWCellArray mwCellArray = (MWCellArray) result[0];
        double[][] resultList = new double[mwCellArray.numberOfElements()][];
        for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
            resultList[i] = (double[]) mwCellArray.getCell(new int[]{1, i + 1}).toArray()[0];
        }

        Object legendObject = result[1];
        List<String> legendList = new ArrayList<>();
        if(!"[]".equals(legendObject.toString())){
            MWCellArray mwCellArray1 = (MWCellArray) legendObject;
            for (int i = 0; i < mwCellArray1.numberOfElements(); i++) {
                byte[] receivedBytes = (byte[])mwCellArray1.getCell(new int[]{1, i + 1}).toArray()[0]; // 从网络或文件读取的字节
                legendList.add(new String(receivedBytes, StandardCharsets.UTF_8));
            }
        }

        MWCellArray mwCellArray2 = (MWCellArray) result[2];
        String[] xUnitList = new String[mwCellArray2.numberOfElements()];
        for (int i = 0; i < mwCellArray2.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray2.getCell(new int[]{1, i + 1}).toArray()[0];
            xUnitList[i] = new String(current);
        }

        MWCellArray mwCellArray3 = (MWCellArray) result[3];
        String[] yUnitList = new String[mwCellArray3.numberOfElements()];
        for (int i = 0; i < mwCellArray3.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray3.getCell(new int[]{1, i + 1}).toArray()[0];
            yUnitList[i] = new String(current);
        }

        MWCellArray mwCellArray4 = (MWCellArray) result[4];
        String[] xLabelList = new String[mwCellArray4.numberOfElements()];
        for (int i = 0; i < mwCellArray4.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray4.getCell(new int[]{1, i + 1}).toArray()[0];
            xLabelList[i] = new String(current);
        }

        MWCellArray mwCellArray5 = (MWCellArray) result[5];
        String[] yLabelList = new String[mwCellArray5.numberOfElements()];
        for (int i = 0; i < mwCellArray5.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray5.getCell(new int[]{1, i + 1}).toArray()[0];
            yLabelList[i] = new String(current);
        }

        MWCellArray mwCellArray6 = (MWCellArray) result[6];
        String[] filePathList = new String[mwCellArray6.numberOfElements()];
        for (int i = 0; i < mwCellArray6.numberOfElements(); i++) {
            byte[] receivedBytes = (byte[])mwCellArray6.getCell(new int[]{1, i + 1}).toArray()[0]; // 从网络或文件读取的字节
            filePathList[i] = new String(receivedBytes, StandardCharsets.UTF_8);
        }

        // 保存绘图数据
        int index = 0;
        for (int i = 0; i < resultList.length; i+=2) {
            double[][] chartData = new double[2][];
            chartData[0] = resultList[i];
            chartData[1] = resultList[i + 1];
            ReadNVHFileVO readNVHFileVO = new ReadNVHFileVO();
            readNVHFileVO.setData(chartData);
            if(ObjectUtil.isNotEmpty(legendList)){
                readNVHFileVO.setLegend(legendList.get(index));
            }
            readNVHFileVO.setUnitX(xUnitList[index]);
            readNVHFileVO.setUnitY(yUnitList[index]);
            readNVHFileVO.setLabelX(xLabelList[index]);
            readNVHFileVO.setLabelY(yLabelList[index]);
            readNVHFileVO.setChartType("bar");

            // 上传原始数据文件
            String path = filePathList[index++];
            File file = new File(path);
            InputStream localFileInputStream = new FileInputStream(file);
            Attachment attachment = attachmentService.uploadAttachment(
                    localFileInputStream,
                    file.getName(),
                    file.length(),
                    AttachmentConstant.OWNER_TYPE_MENU,
                    ownerId,
                    2);
            localFileInputStream.close();

            // 保存曲线数据
            this.saveNVHChartDataToDB(attachment.getId(),readNVHFileVO);

            // 删除本地原始数据文件
            FileUtil.deleteFile(path);
        }

        // 5.删除本地文件
        FileUtil.deleteFile(localFileName);
    }

    @SneakyThrows
    private void read3DFile(Long ownerId, String drawType,InputStream inputStream) {
        // 1.io流转换为本地文件
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String localFileName = filePath + uuid + "-" + drawType + ".xlsx";
        FileUtil.downloadToLocal(inputStream, localFileName);

        // 2.调用matlab函数读取excel读取数据
        DrawNVHChart drawNVHChart = new DrawNVHChart();
        Method method = drawNVHChart.getClass().getMethod(drawType, int.class, Object[].class);
        Object[] object = {
                localFileName
        };
        Object[] result = (Object[]) method.invoke(drawNVHChart, 10, object);

        // 3.调整不同数据集频率
        MWCellArray mwCellArray = (MWCellArray) result[0];
        double[][][] resultList = new double[mwCellArray.numberOfElements()][][];
        for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
            resultList[i] = (double[][]) mwCellArray.getCell(new int[]{1, i + 1}).toArray();
        }

        MWCellArray mwCellArray1 = (MWCellArray) result[1];
        String[] xUnitList = new String[mwCellArray1.numberOfElements()];
        for (int i = 0; i < mwCellArray1.numberOfElements(); i++) {
            xUnitList[i] = mwCellArray1.getCell(new int[]{1, i + 1}).toArray()[0].toString();
        }

        MWCellArray mwCellArray2 = (MWCellArray) result[2];
        String[] yUnitList = new String[mwCellArray2.numberOfElements()];
        for (int i = 0; i < mwCellArray2.numberOfElements(); i++) {
            yUnitList[i] = mwCellArray2.getCell(new int[]{1, i + 1}).toArray()[0].toString();
        }


        MWCellArray mwCellArray3 = (MWCellArray) result[3];
        String[] zUnitList = new String[mwCellArray3.numberOfElements()];
        for (int i = 0; i < mwCellArray3.numberOfElements(); i++) {
            zUnitList[i] = mwCellArray3.getCell(new int[]{1, i + 1}).toArray()[0].toString();
        }

        MWCellArray mwCellArray4 = (MWCellArray) result[4];
        String[] xLabelList = new String[mwCellArray4.numberOfElements()];
        for (int i = 0; i < mwCellArray4.numberOfElements(); i++) {
            xLabelList[i] = mwCellArray4.getCell(new int[]{1, i + 1}).toArray()[0].toString();
        }

        MWCellArray mwCellArray5 = (MWCellArray) result[5];
        String[] yLabelList = new String[mwCellArray5.numberOfElements()];
        for (int i = 0; i < mwCellArray5.numberOfElements(); i++) {
            yLabelList[i] = mwCellArray5.getCell(new int[]{1, i + 1}).toArray()[0].toString();
        }

        MWCellArray mwCellArray6 = (MWCellArray) result[6];
        String[] zLabelList = new String[mwCellArray6.numberOfElements()];
        for (int i = 0; i < mwCellArray6.numberOfElements(); i++) {
            zLabelList[i] = mwCellArray6.getCell(new int[]{1, i + 1}).toArray()[0].toString();
        }

        MWCellArray mwCellArray7 = (MWCellArray) result[7];
        double[] ZMaxList = new double[mwCellArray7.numberOfElements()];
        for (int i = 0; i < mwCellArray7.numberOfElements(); i++) {
            double[] current = (double[]) mwCellArray7.getCell(new int[]{1, i + 1}).toArray()[0];
            ZMaxList[i] = current[0];
        }

        MWCellArray mwCellArray8 = (MWCellArray) result[8];
        double[] ZMinList = new double[mwCellArray8.numberOfElements()];
        for (int i = 0; i < mwCellArray8.numberOfElements(); i++) {
            double[] current = (double[]) mwCellArray8.getCell(new int[]{1, i + 1}).toArray()[0];
            ZMinList[i] = current[0];
        }

        MWCellArray mwCellArray9 = (MWCellArray) result[9];
        String[] filePathList = new String[mwCellArray5.numberOfElements()];
        for (int i = 0; i < mwCellArray9.numberOfElements(); i++) {
            filePathList[i] = mwCellArray9.getCell(new int[]{1, i + 1}).toArray()[0].toString();
        }

        // 保存绘图数据
        for (int i = 0; i < resultList.length; i++) {
            double[][] chartData = resultList[i];

            ReadNVHFileVO readNVHFileVO = new ReadNVHFileVO();
            readNVHFileVO.setData(chartData);
            readNVHFileVO.setUnitX(xUnitList[i]);
            readNVHFileVO.setUnitY(yUnitList[i]);
            readNVHFileVO.setUnitZ(zUnitList[i]);
            readNVHFileVO.setLabelX(xLabelList[i]);
            readNVHFileVO.setLabelY(yLabelList[i]);
            readNVHFileVO.setLabelZ(zLabelList[i]);
            readNVHFileVO.setMaxZ(ZMaxList[i]);
            readNVHFileVO.setMinZ(ZMinList[i]);
            readNVHFileVO.setChartType("heatmap");
//
//            // 上传原始数据文件
            String path = filePathList[i];
            File file = new File(path);
            InputStream localFileInputStream = new FileInputStream(file);
            Attachment attachment = attachmentService.uploadAttachment(
                    localFileInputStream,
                    file.getName(),
                    file.length(),
                    AttachmentConstant.OWNER_TYPE_MENU,
                    ownerId,
                    2);
            localFileInputStream.close();

            // 保存曲线数据
            this.saveNVHChartDataToDB(attachment.getId(),readNVHFileVO);

            // 删除本地原始数据文件
            FileUtil.deleteFile(path);
        }
        // 删除本地文件
        FileUtil.deleteFile(localFileName);
    }

    /**
     * 调用matlab函数读取折线图数据
     *
     * @param drawType    matlab函数名
     */
    @SneakyThrows
    private void readLineFile(Long ownerId,String drawType,InputStream inputStream) {
        // 1、io流转换为本地文件
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String localFileName = filePath + uuid + "-" + drawType + ".xlsx";
        FileUtil.downloadToLocal(inputStream, localFileName);

        // 2.调用matlab函数读取excel读取数据
        DrawNVHChart drawNVHChart = new DrawNVHChart();
        Method method = drawNVHChart.getClass().getMethod(drawType, int.class, Object[].class);
        Object[] object = {
                localFileName
        };
        Object[] result = (Object[]) method.invoke(drawNVHChart, 7, object);
        // 4.封装结果集
        MWCellArray mwCellArray = (MWCellArray) result[0];
        double[][] resultList = new double[mwCellArray.numberOfElements()][];
        for (int i = 0; i < mwCellArray.numberOfElements(); i++) {
            resultList[i] = (double[]) mwCellArray.getCell(new int[]{1, i + 1}).toArray()[0];
        }

        Object legendObject = result[1];
        List<String> legendList = new ArrayList<>();
        if(!"[]".equals(legendObject.toString())){
            MWCellArray mwCellArray1 = (MWCellArray) legendObject;
            for (int i = 0; i < mwCellArray1.numberOfElements(); i++) {
                byte[] receivedBytes = (byte[])mwCellArray1.getCell(new int[]{1, i + 1}).toArray()[0]; // 从网络或文件读取的字节
                legendList.add(new String(receivedBytes, StandardCharsets.UTF_8));
            }
        }

        MWCellArray mwCellArray2 = (MWCellArray) result[2];
        String[] xUnitList = new String[mwCellArray2.numberOfElements()];
        for (int i = 0; i < mwCellArray2.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray2.getCell(new int[]{1, i + 1}).toArray()[0];
            xUnitList[i] = new String(current);
        }

        MWCellArray mwCellArray3 = (MWCellArray) result[3];
        String[] yUnitList = new String[mwCellArray3.numberOfElements()];
        for (int i = 0; i < mwCellArray3.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray3.getCell(new int[]{1, i + 1}).toArray()[0];
            yUnitList[i] = new String(current);
        }

        MWCellArray mwCellArray4 = (MWCellArray) result[4];
        String[] xLabelList = new String[mwCellArray4.numberOfElements()];
        for (int i = 0; i < mwCellArray4.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray4.getCell(new int[]{1, i + 1}).toArray()[0];
            xLabelList[i] = new String(current);
        }

        MWCellArray mwCellArray5 = (MWCellArray) result[5];
        String[] yLabelList = new String[mwCellArray5.numberOfElements()];
        for (int i = 0; i < mwCellArray5.numberOfElements(); i++) {
            char[] current = (char[])mwCellArray5.getCell(new int[]{1, i + 1}).toArray()[0];
            yLabelList[i] = new String(current);
        }

        MWCellArray mwCellArray6 = (MWCellArray) result[6];
        String[] filePathList = new String[mwCellArray6.numberOfElements()];
        for (int i = 0; i < mwCellArray6.numberOfElements(); i++) {
            byte[] receivedBytes = (byte[])mwCellArray6.getCell(new int[]{1, i + 1}).toArray()[0]; // 从网络或文件读取的字节
            filePathList[i] = new String(receivedBytes, StandardCharsets.UTF_8);
        }

        // 保存绘图数据
        int index = 0;
        for (int i = 0; i < resultList.length; i+=2) {
            double[][] chartData = new double[2][];
            chartData[0] = resultList[i];
            chartData[1] = resultList[i + 1];
            ReadNVHFileVO readNVHFileVO = new ReadNVHFileVO();
            readNVHFileVO.setData(chartData);
            if(ObjectUtil.isNotEmpty(legendList)){
                readNVHFileVO.setLegend(legendList.get(index));
            }
            readNVHFileVO.setUnitX(xUnitList[index]);
            readNVHFileVO.setUnitY(yUnitList[index]);
            readNVHFileVO.setLabelX(xLabelList[index]);
            readNVHFileVO.setLabelY(yLabelList[index]);
            readNVHFileVO.setChartType("line");

            // 上传原始数据文件
            String path = filePathList[index++];
            File file = new File(path);
            InputStream localFileInputStream = new FileInputStream(file);
            Attachment attachment = attachmentService.uploadAttachment(
                    localFileInputStream,
                    file.getName(),
                    file.length(),
                    AttachmentConstant.OWNER_TYPE_MENU,
                    ownerId,
                    2);
            localFileInputStream.close();

            // 保存曲线数据
            this.saveNVHChartDataToDB(attachment.getId(),readNVHFileVO);

            // 删除本地原始数据文件
            FileUtil.deleteFile(path);
        }

        // 5.删除本地文件
        FileUtil.deleteFile(localFileName);
    }

    private void saveNVHChartDataToDB(Long attachId,ReadNVHFileVO readNVHFileVO){
        LocalDate localDate = LocalDate.now();
        ChartData chartData = new ChartData();
        chartData.setCreateTime(localDate);
        chartData.setUpdateTime(localDate);
        chartData.setAttachId(attachId);
        try {
            chartData.setChartData(FileUtil.objectToByte(readNVHFileVO));
        } catch (IOException e) {
            e.printStackTrace();
        }
        chartDataService.save(chartData);
    }

}
