package com.singularity.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.dao.SlideTestResultMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.listener.SlideTestConditionListener;
import com.singularity.listener.SlideTestResultListener;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.slide.DeleteSlideDataFileDTO;
import com.singularity.pojo.dto.slide.QuerySlideTestResultListDTO;
import com.singularity.pojo.dto.slide.UpdateSlideTestResultDTO;
import com.singularity.pojo.dto.slide.UploadSlideDataFileDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.CarBasicParam;
import com.singularity.pojo.entity.SlideTestCondition;
import com.singularity.pojo.entity.SlideTestResult;
import com.singularity.pojo.vo.slide.QuerySlideTestResultListVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.CarBasicParamService;
import com.singularity.service.SlideTestConditionService;
import com.singularity.service.SlideTestResultService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.List;

@Service
public class SlideTestResultServiceImpl extends ServiceImpl<SlideTestResultMapper, SlideTestResult> implements SlideTestResultService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private SlideTestConditionService slideTestConditionService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Override
    @Transactional
    public void uploadSlideDataFile(UploadSlideDataFileDTO dto) {
        // 文件名验重
        attachmentService.checkRepeatFileName(
                dto.getMultipartFile().getOriginalFilename(),
                dto.getCarId(),
                FileBusinessConstant.SLIDE_DATA_FILE,
                AttachmentConstant.OWNER_TYPE_CAR
        );

        // 将附件信息保存至数据库与minio
        Attachment attachment = attachmentService.uploadAttachment(
                dto.getMultipartFile(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                FileBusinessConstant.SLIDE_DATA_FILE,
                dto.getOperId()
        );

        // 读取滑行试验条件数据
        SlideTestConditionListener slideTestConditionListener = new SlideTestConditionListener(
                slideTestConditionService, attachment.getId(), dto.getCarId()
        );
        try (InputStream inputStream = dto.getMultipartFile().getInputStream()) {
            EasyExcel.read(inputStream, slideTestConditionListener)
                    .sheet("试验条件")
                    .headRowNumber(0)
                    .doRead();
        } catch (Exception e) {
            throw new ToolsBusinessWarn("文件上传失败");
        }

        // 读取滑行道路载荷试验结果数据
        SlideTestResultListener slideTestResultListener = new SlideTestResultListener(
                this, attachment.getId(), dto.getCarId()
        );
        try (InputStream inputStream = dto.getMultipartFile().getInputStream()) {
            EasyExcel.read(inputStream, slideTestResultListener)
                    .sheet("道路载荷试验结果")
                    .headRowNumber(1)
                    .doRead();
        } catch (Exception e) {
            e.printStackTrace();
            throw new ToolsBusinessWarn("文件格式错误！");
        }

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setCoastUploadStat(2);
        carBasicParam.setId(dto.getCarId());
        carBasicParamService.updateById(carBasicParam);
    }

    /**
     * 删除滑行数据文件
     */
    @Override
    public void deleteSlideDataFile(DeleteSlideDataFileDTO dto) {
        // 删除minio附件
        attachmentService.removeAttachmentById(dto.getAttachIdList());

        // 删除滑行试验条件数据
        LambdaQueryWrapper<SlideTestCondition> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(SlideTestCondition::getAttachId,dto.getAttachIdList());
        slideTestConditionService.remove(queryWrapper);

        // 删除滑行道路载荷试验结果数据
        LambdaQueryWrapper<SlideTestResult> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.in(SlideTestResult::getAttachId,dto.getAttachIdList());
        this.remove(queryWrapper2);
    }

    @Override
    public List<QuerySlideTestResultListVO> querySlideTestResultList(QuerySlideTestResultListDTO dto) {
        LambdaQueryWrapper<SlideTestResult> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SlideTestResult::getAttachId,dto.getAttachId());
        List<SlideTestResult> slideTestResultList= this.list(queryWrapper);
        return BeanUtil.copyToList(slideTestResultList, QuerySlideTestResultListVO.class);
    }

    /**
     * 编辑滑行道路载荷试验结果列表
     */
    @Override
    public void updateSlideTestResult(UpdateSlideTestResultDTO dto) {
        SlideTestResult slideTestResult = new SlideTestResult();
        BeanUtil.copyProperties(dto,slideTestResult);
        this.updateById(slideTestResult);
    }
}
