package com.singularity.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.common.entity.PartChartData;
import com.singularity.listener.SpringReadChartDataListener;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.spring.QuerySpringChartDataListDTO;
import com.singularity.pojo.dto.spring.QuerySpringCompareChartDataDTO;
import com.singularity.pojo.dto.spring.UploadSpringResourceFileDTO;
import com.singularity.pojo.dto.spring.UploadSpringResultFileDTO;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.vo.spring.QuerySpringCompareChartDataVO;
import com.singularity.service.*;
import com.singularity.util.FileUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class SpringServiceImpl implements SpringService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private TaskQueueService taskQueueService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource
    private DamperChartDataService damperChartDataService;

    @Override
    public void uploadSpringResourceFile(UploadSpringResourceFileDTO dto) {
        // 若localFilePath不为空表示定时上传
        MultipartFile multipartFile = StringUtils.isNotEmpty(dto.getLocalFilePath()) ? FileUtil.fileToMultipartFile(dto.getLocalFilePath()) : dto.getMultipartFile();
        LocalDateTime startTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setId(dto.getTaskId());
        try{
            if(Objects.requireNonNull(multipartFile.getOriginalFilename()).contains(".zip")){
                attachmentService.uploadResourceFileList(
                        multipartFile,
                        AttachmentConstant.OWNER_TYPE_CAR,
                        dto.getOwnerId(),
                        FileBusinessConstant.SPRING_RESOURCE_FILE,
                        dto.getOperId()
                );
            }else{
                attachmentService.uploadAttachment(
                        multipartFile,
                        AttachmentConstant.OWNER_TYPE_CAR,
                        dto.getOwnerId(),
                        FileBusinessConstant.SPRING_RESOURCE_FILE,
                        dto.getOperId()
                );
            }
            taskQueue.setStat(2);
        }catch(Exception e){
            log.error("错误信息" + e);
            taskQueue.setRemark(e.getMessage());
            taskQueue.setStat(3);
        }finally{
            Duration duration = Duration.between(startTime,LocalDateTime.now());
            taskQueue.setConsumeTime(duration.getSeconds());
            taskQueueService.updateById(taskQueue);
            // 删除临时文件
            if(StringUtils.isNotEmpty(dto.getLocalFilePath()))
                FileUtil.deleteFile(dto.getLocalFilePath());
        }
        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setSpringUploadStat(2);
        carBasicParam.setId(dto.getOwnerId());
        carBasicParamService.updateById(carBasicParam);
    }

    @Override
    public void uploadSpringResultFile(UploadSpringResultFileDTO dto) {
        // 若localFilePath不为空表示定时上传
        MultipartFile multipartFile = StringUtils.isNotEmpty(dto.getLocalFilePath()) ? FileUtil.fileToMultipartFile(dto.getLocalFilePath()) : dto.getMultipartFile();
        LocalDateTime startTime = LocalDateTime.now();
        TaskQueue taskQueue = new TaskQueue();
        taskQueue.setId(dto.getTaskId());
        try{
            Attachment attachment = attachmentService.uploadAttachment(
                    multipartFile,
                    AttachmentConstant.OWNER_TYPE_CAR,
                    dto.getOwnerId(),
                    FileBusinessConstant.SPRING_RESULT_FILE,
                    dto.getOperId()
            );
            this.readResultFile(multipartFile.getInputStream(),attachment.getId(),"前簧");
            this.readResultFile(multipartFile.getInputStream(),attachment.getId(),"后簧");
            taskQueue.setStat(2);
        }catch(Exception e){
            log.error("错误信息" + e);
            taskQueue.setRemark(e.getMessage());
            taskQueue.setStat(3);
        }finally{
            Duration duration = Duration.between(startTime,LocalDateTime.now());
            taskQueue.setConsumeTime(duration.getSeconds());
            taskQueueService.updateById(taskQueue);
            // 删除临时文件
            if(StringUtils.isNotEmpty(dto.getLocalFilePath()))
                FileUtil.deleteFile(dto.getLocalFilePath());
        }
        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setSpringUploadStat(2);
        carBasicParam.setId(dto.getOwnerId());
        carBasicParamService.updateById(carBasicParam);
    }

    @Override
    @SneakyThrows
    public List<ChartDataModel> querySpringChartDataList(QuerySpringChartDataListDTO dto) {
        LambdaQueryWrapper<PartChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(PartChartData::getAttachId,dto.getAttachId());
        List<PartChartData> partChartDataList = damperChartDataService.list(queryWrapper);
        List<ChartDataModel> chartDataModelList = new ArrayList<>();
        for (PartChartData partChartData : partChartDataList) {
            ChartDataModel chartDataModel = (ChartDataModel) FileUtil.byteToObject(partChartData.getChartData());
            chartDataModelList.add(chartDataModel);
        }
        return chartDataModelList;
    }

    @Override
    @SneakyThrows
    public List<QuerySpringCompareChartDataVO> querySpringCompareChartData(QuerySpringCompareChartDataDTO dto) {
        Map<Long,String> transposedMap = new HashMap<>();
        for (Map.Entry<String, Long> entry : dto.getMap().entrySet()) {
            transposedMap.put(entry.getValue(), entry.getKey());
        }
        LambdaQueryWrapper<PartChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(PartChartData::getAttachId,dto.getMap().values());
        List<PartChartData> partChartDataList = damperChartDataService.list(queryWrapper);
        Map<Long,List<PartChartData>> map = partChartDataList.stream().collect(Collectors.groupingBy(PartChartData::getAttachId));
        Map<String, QuerySpringCompareChartDataVO> resultMap = new HashMap<>();
        int index = 0; // 数组起始位置
        for(Map.Entry<Long,List<PartChartData>> listEntry : map.entrySet()){
            List<PartChartData> partChartDataList2 = listEntry.getValue();
            for (PartChartData partChartData : partChartDataList2) {
                ChartDataModel chartDataModel = (ChartDataModel) FileUtil.byteToObject(partChartData.getChartData());
                QuerySpringCompareChartDataVO querySpringCompareChartDataVO;
                List<String> legendList;
                double[][] data;
                if(resultMap.containsKey(chartDataModel.getTitle())){
                    querySpringCompareChartDataVO = resultMap.get(chartDataModel.getTitle());
                    legendList = querySpringCompareChartDataVO.getLegendList();
                    data = querySpringCompareChartDataVO.getData();
                }else{
                    querySpringCompareChartDataVO = new QuerySpringCompareChartDataVO();
                    legendList = new ArrayList<>();
                    data =  new double[chartDataModel.getData().length * map.size()][];
                    querySpringCompareChartDataVO.setChartTitle(chartDataModel.getTitle());
                    querySpringCompareChartDataVO.setXLabel(chartDataModel.getXLabel());
                    querySpringCompareChartDataVO.setYLabel(chartDataModel.getYLabel());
                }
                for (String legend : chartDataModel.getLegendList()) {
                    legendList.add(transposedMap.get(listEntry.getKey()) + legend);
                }
                System.arraycopy(chartDataModel.getData(),0,data,index * chartDataModel.getData().length,chartDataModel.getData().length);
                querySpringCompareChartDataVO.setLegendList(legendList);
                querySpringCompareChartDataVO.setData(data);
                resultMap.put(chartDataModel.getTitle(),querySpringCompareChartDataVO);
            }
            index++;
        }
        return new ArrayList<>(resultMap.values());
    }

    private void readResultFile(InputStream inputStream,Long attachId,String sheetName){
        SpringReadChartDataListener springReadChartDataListener = new SpringReadChartDataListener();
        EasyExcel.read(inputStream, springReadChartDataListener).headRowNumber(0).sheet(sheetName).doRead();
        List<String> legendList = springReadChartDataListener.getLegendList();
        Map<Integer,List<Double>> xData = springReadChartDataListener.getXData();
        Map<Integer,List<Double>> yData = springReadChartDataListener.getYData();
        Map<Integer,List<Double>> yData2 = springReadChartDataListener.getYData2();

        ChartDataModel chartDataModel = new ChartDataModel();
        ChartDataModel chartDataModel2 = new ChartDataModel();
        LocalDateTime localDateTime = LocalDateTime.now();
        double[][] data = new double[legendList.size() * 2][];
        double[][] data2 = new double[legendList.size() * 2][];
        int index = 0;
        for(int i = 1; i <= legendList.size(); i++){
            data[index] = xData.get(i).stream().mapToDouble(Double::doubleValue).toArray();
            data2[index++] = xData.get(i).stream().mapToDouble(Double::doubleValue).toArray();
            data[index] = yData.get(i).stream().mapToDouble(Double::doubleValue).toArray();
            data2[index++] = yData2.get(i).stream().mapToDouble(Double::doubleValue).toArray();
        }
        chartDataModel.setData(data);
        chartDataModel.setTitle(sheetName + "标准高度模式弹簧力曲线");
        chartDataModel.setXLabel("弹簧行程（mm）");
        chartDataModel.setYLabel("弹簧力（N）");
        chartDataModel.setLegendList(legendList);

        chartDataModel2.setData(data2);
        chartDataModel2.setTitle(sheetName + "标准高度模式弹簧刚度曲线");
        chartDataModel2.setXLabel("弹簧行程（mm）");
        chartDataModel2.setYLabel("刚度(N/mm)");
        chartDataModel2.setLegendList(legendList);

        PartChartData partChartData = new PartChartData();
        partChartData.setCreateTime(localDateTime);
        partChartData.setUpdateTime(localDateTime);
        partChartData.setAttachId(attachId);

        PartChartData partChartData2 = new PartChartData();
        partChartData2.setCreateTime(localDateTime);
        partChartData2.setUpdateTime(localDateTime);
        partChartData2.setAttachId(attachId);
        try {
            partChartData.setChartData(FileUtil.objectToByte(chartDataModel));
            partChartData2.setChartData(FileUtil.objectToByte(chartDataModel2));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        damperChartDataService.save(partChartData);
        damperChartDataService.save(partChartData2);
    }
}
