package com.singularity.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.dao.SteeringChartDataMapper;
import com.singularity.pojo.dto.steering.DeleteSteeringDataFileDTO;
import com.singularity.pojo.dto.steering.QuerySteeringDataDTO;
import com.singularity.pojo.dto.steering.UploadSteeringDateFileDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.CarBasicParam;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.pojo.entity.SteeringChartData;
import com.singularity.service.AttachmentService;
import com.singularity.service.CarBasicParamService;
import com.singularity.service.SteeringChartDataService;
import com.singularity.util.FileUtil;
import com.singularity.listener.SteeringChartDataListener;
import lombok.SneakyThrows;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service
public class SteeringChartDataServiceImpl extends ServiceImpl<SteeringChartDataMapper, SteeringChartData> implements SteeringChartDataService {

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private CarBasicParamService carBasicParamService;

    /**
     * 上传转向试验数据文件
     */
    @Override
    @Transactional
    @SneakyThrows
    public void uploadSteeringDateFile(UploadSteeringDateFileDTO dto) {
        // 文件名验重
        attachmentService.checkRepeatFileName(
                dto.getMultipartFile().getOriginalFilename(),
                dto.getCarId(),
                FileBusinessConstant.STEERING_DATA_FILE,
                AttachmentConstant.OWNER_TYPE_CAR
        );

        // 将文件上传到minio
        MultipartFile multipartFile = dto.getMultipartFile();
        Attachment attachment = attachmentService.uploadAttachment(
                multipartFile,
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getCarId(),
                FileBusinessConstant.STEERING_DATA_FILE,
                dto.getOperId()
        );

        // 读取低速转向力数据

        // 读取原地转向力数据

        // 读取其它转向力数据

        // 读取动态转向力数据

        // 读取转向曲线数据
        EasyExcel.read(multipartFile.getInputStream(), new SteeringChartDataListener(this, attachment.getId()))
                .sheet("试验数据")
                .doRead();

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setDamperUploadStat(2);
        carBasicParam.setId(dto.getCarId());
        carBasicParamService.updateById(carBasicParam);
    }

    /**
     * 删除制动性试验数据文件
     */
    @Override
    public void deleteSteeringFileList(DeleteSteeringDataFileDTO dto) {
        // 删除附件
        attachmentService.removeByIds(dto.getAttachIdList());
    }

    /**
     * 查询转向试验曲线数据
     */
    @Override
    public List<ChartDataModel> querySteeringData(QuerySteeringDataDTO dto) {
        LambdaQueryWrapper<SteeringChartData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SteeringChartData::getAttachId,dto.getAttachId());
        List<SteeringChartData> steeringChartDataList = this.list(queryWrapper);
        List<ChartDataModel> chartDataModelList = new ArrayList<>();
        for (SteeringChartData steeringChartData : steeringChartDataList) {
            try {
                ChartDataModel chartDataModel = (ChartDataModel) FileUtil.byteToObject(steeringChartData.getChartData());
                chartDataModelList.add(chartDataModel);
            } catch (IOException | ClassNotFoundException e) {
                throw new RuntimeException(e);
            }
        }
        return chartDataModelList;
    }

}
