package com.singularity.service.impl;

import cn.dev33.satoken.exception.SaTokenException;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.data.*;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.dao.NVHChartDataMapper;
import com.singularity.dao.SubjectiveEvaluationMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.listener.NVHSubjectiveElectricListener;
import com.singularity.listener.NVHSubjectiveFuelListener;
import com.singularity.listener.SubjectiveEvaluationListener;
import com.singularity.listener.SubjectiveSceneListener;
import com.singularity.pojo.bo.subjective.QueryEvaluatorNameListBO;
import com.singularity.pojo.bo.subjective.SubjectiveEvaluationTreeBO;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.DeleteEvaluationDataDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.QueryEvaluatorNameListDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.SubjectiveEvaluationDTO;
import com.singularity.pojo.dto.subjectiveEvaluation.SubjectiveIdDTO;
import com.singularity.pojo.entity.*;
import com.singularity.pojo.enumeration.SubjectiveEvaluationEnum;
import com.singularity.pojo.param.subjectiveEvaluation.*;
import com.singularity.pojo.vo.subjectiveEvaluation.GetSubjectiveEvaluationListVO;
import com.singularity.pojo.vo.subjectiveEvaluation.QueryEvaluatorNameListVO;
import com.singularity.pojo.vo.subjectiveEvaluation.SubjectiveDataVO;
import com.singularity.service.*;
import com.singularity.util.FileUtil;
import com.singularity.util.WordUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.*;
import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class SubjectiveEvaluationServiceImpl extends ServiceImpl<SubjectiveEvaluationMapper, SubjectiveEvaluation> implements SubjectiveEvaluationService {


    @Resource
    private SubjectiveEvaluationMapper subjectiveEvaluationMapper;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private CarColumnService carColumnService;

    @Resource
    private CarBasicParamService carBasicParamService;

    @Resource
    private CarBodyParamService carBodyParamService;

    @Resource
    private CarChassisStructureService carChassisStructureService;

    @Resource
    private NVHEvaluationService nvhEvaluationService;

    @Resource
    private SubjectiveSceneService subjectiveSceneService;

    @Resource
    private NVHChartDataMapper nvhChartDataMapper;


    @Value("${file.path}")
    private String filePath;

    /**
     * 上传主观评价评价结果数据文件
     */
    @Override
    @Transactional
    public void uploadFile(SubjectiveEvaluationUploadFileParam param) {
        // 校验文件后缀xlsx/xls/csv
        String[] split = Objects.requireNonNull(param.getFile().getOriginalFilename()).split("\\.");
        String fileSuffix = split[split.length - 1];
        if (!(fileSuffix.equals("xlsx") || fileSuffix.equals("xls") || fileSuffix.equals("csv"))) {
            throw new ToolsBusinessWarn("传入的评价文件后缀应当为: .xlsx/.xls/.csv");
        }

        // 将上传文件至minio
        Attachment attachment = attachmentService.uploadAttachment(
                param.getFile(),
                AttachmentConstant.OWNER_TYPE_CAR,
                param.getCarId(),
                FileBusinessConstant.SUBJECTIVE_DATA
        );

        // 使用EasyExcel读取数据
        try{
            EasyExcel.read(param.getFile().getInputStream(), SubjectiveEvaluationExcelParam.class,
                    new SubjectiveEvaluationListener(
                            param.getCarId(),
                            attachment.getId(),
                            param.getReviewType(),
                            this
                    ))
                    .headRowNumber(2).sheet(0).doRead();
        }catch (Exception e){
            e.printStackTrace();
            throw new SaTokenException("读取主观评价试验数据失败");
        }

        // 使用EasyExcel读取试验场景
        try{
            SubjectiveSceneListener subjectiveSceneListener = new SubjectiveSceneListener(attachment.getId().intValue());
            EasyExcel.read(param.getFile().getInputStream(), subjectiveSceneListener)
                    .headRowNumber(1).sheet(1).doRead();
            List<SubjectiveScene> subjectiveSceneList = subjectiveSceneListener.getSubjectiveSceneList();
            subjectiveSceneService.saveBatch(subjectiveSceneList);
        }catch (Exception e){
            e.printStackTrace();
            throw new SaTokenException("读取主观评价试验场景失败");
        }

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        switch(param.getReviewType()){
            case 0:
                carBasicParam.setCommercialUploadStat(2);
                break;
            case 1:
                carBasicParam.setPassengerUploadStat(2);
                break;
            case 2:
                carBasicParam.setCommodityUploadStat(2);
                break;
            case 3:
                carBasicParam.setTruckUploadStat(2);
                break;
            case 4:
                carBasicParam.setBusUploadStat(2);
                break;
        }
        carBasicParam.setId(param.getCarId());
        carBasicParamService.updateById(carBasicParam);

    }


    @Override
    public void save(List<SubjectiveEvaluation> subjectiveEvaluationList) {
        this.saveBatch(subjectiveEvaluationList);
    }

    @Override
    public List<GetSubjectiveEvaluationListVO> getSubjectiveEvaluationList(SubjectiveEvaluationParam subjectiveEvaluationParam) {
        // 根据车辆id查询评价数据据
        LambdaQueryWrapper<SubjectiveEvaluation> queryWrapper3 = new LambdaQueryWrapper<>();
            queryWrapper3.eq(SubjectiveEvaluation::getCarId, subjectiveEvaluationParam.getCarId())
                .eq(SubjectiveEvaluation::getReviewType, subjectiveEvaluationParam.getReviewType());
        List<SubjectiveEvaluation> subjectiveEvaluations = subjectiveEvaluationMapper.selectList(queryWrapper3);
        List<SubjectiveEvaluationDTO> subjectiveEvaluationDTOList =
                BeanUtil.copyToList(subjectiveEvaluations,SubjectiveEvaluationDTO.class);
        Map<Long,List<SubjectiveEvaluationDTO>> map = subjectiveEvaluationDTOList.stream()
                .collect(Collectors.groupingBy(SubjectiveEvaluationDTO::getAttachId));
        Set<Long> attachIdSet = map.keySet();
        List<GetSubjectiveEvaluationListVO> result = new ArrayList<>();
        if(CollectionUtil.isEmpty(attachIdSet))return result;
        // 根据附件id查询场景信息
        LambdaQueryWrapper<SubjectiveScene> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.in(SubjectiveScene::getAttachId,attachIdSet);
        if(CollectionUtil.isEmpty(attachIdSet)){
            return result;
        }
        List<SubjectiveScene> subjectiveEvaluationList = subjectiveSceneService.list(queryWrapper2);
        // 根据附件idList查询评价数据
        for (SubjectiveScene subjectiveScene : subjectiveEvaluationList) {
            // 根据附件id查询附件信息
            Attachment attachment = attachmentService.getById(subjectiveScene.getAttachId());
            GetSubjectiveEvaluationListVO getSubjectiveEvaluationListVO = new GetSubjectiveEvaluationListVO();
            BeanUtil.copyProperties(subjectiveScene,getSubjectiveEvaluationListVO);
            getSubjectiveEvaluationListVO.setSubjectiveEvaluationDTOList(
                    this.convertMenuTree(
                            map.get((long)subjectiveScene.getAttachId())
                    )
            );
            getSubjectiveEvaluationListVO.setEvaluatorName(subjectiveScene.getEvaluationName());
            getSubjectiveEvaluationListVO.setFilePath(attachment.getPath());
            getSubjectiveEvaluationListVO.setFileName(attachment.getName());
            result.add(getSubjectiveEvaluationListVO);
        }
        return result;
    }



    /**
     * 两层循环封装菜单树
     *
     * @param treeNodes 需要封装的菜单集合
     */
    private List<SubjectiveEvaluationDTO> convertMenuTree(List<SubjectiveEvaluationDTO> treeNodes) {
        List<SubjectiveEvaluationDTO> trees = new ArrayList<>();
        for (SubjectiveEvaluationDTO treeNode : treeNodes) {
            if (Objects.equals(treeNode.getGrade(), 1L)) {
                trees.add(treeNode);
            }
            for (SubjectiveEvaluationDTO it : treeNodes) {
                if (Objects.equals(treeNode.getSerialNumber(), it.getBelongSerial())) {
                    if (treeNode.getSubjectiveEvaluationList() == null || treeNode.getSubjectiveEvaluationList().size() == 0) {
                        treeNode.setSubjectiveEvaluationList(new ArrayList<>());
                    }
                    List<SubjectiveEvaluationDTO> treeNodeChildren = treeNode.getSubjectiveEvaluationList();
                    treeNodeChildren.add(it);
                    treeNode.setSubjectiveEvaluationList(treeNodeChildren);
                }
            }
        }
        return trees;
    }

    @Override
    public int updateSubjectiveEvaluation(UpdateSubjectiveEvaluationParam updateSubjectiveEvaluationParam) {

        return subjectiveEvaluationMapper.updateSubjectiveEvaluation(updateSubjectiveEvaluationParam);
    }

    @Override
    @SneakyThrows
    public void getSubjectiveEvaluationReports(SubjectiveEvaluationReportsParam subjectiveEvaluationReportsParam) {
        // 1、查询表格数据
        // 1.1查询所有数据
        LambdaQueryWrapper<SubjectiveEvaluation> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SubjectiveEvaluation::getAttachId, subjectiveEvaluationReportsParam.getAttachId());
        List<SubjectiveEvaluation> subjectiveEvaluations = subjectiveEvaluationMapper.selectList(wrapper);

        // 1.2将数据封装为树
        List<SubjectiveEvaluationTreeBO> subjectiveEvaluationTreeBOList =
                BeanUtil.copyToList(subjectiveEvaluations, SubjectiveEvaluationTreeBO.class);
        subjectiveEvaluationTreeBOList = getFirstLevelIndicatorList(subjectiveEvaluationTreeBOList);

        // 1.3将树封装为表格数据 表格<行<列<列号，值>>>
        List<List<Map<Integer, String>>> allData = new ArrayList<>();
        for (SubjectiveEvaluationTreeBO subjectiveEvaluationTreeBO : subjectiveEvaluationTreeBOList) {
            List<Map<Integer, String>> tableData = new ArrayList<>();
            Map<Integer, String> columnData = new HashMap<>();
            // 评分
            columnData.put(1, subjectiveEvaluationTreeBO.getModelRatings());
            // 评语
            columnData.put(2, subjectiveEvaluationTreeBO.getModelReviews());
            // 备注
            columnData.put(3, subjectiveEvaluationTreeBO.getRemark());
            tableData.add(columnData);
            List<SubjectiveEvaluationTreeBO> secondChildren = subjectiveEvaluationTreeBO.getChildren();
            if (CollectionUtil.isEmpty(secondChildren)) {
                allData.add(tableData);
                continue;
            }
            for (SubjectiveEvaluationTreeBO secondChild : secondChildren) {
                columnData = new HashMap<>();
                // 评分
                columnData.put(2, secondChild.getModelRatings());
                // 评语
                columnData.put(3, secondChild.getModelReviews());
                // 备注
                columnData.put(4, secondChild.getRemark());
                tableData.add(columnData);
                List<SubjectiveEvaluationTreeBO> thirdChildren = secondChild.getChildren();
                if (CollectionUtil.isEmpty(thirdChildren)) continue;
                List<Map<Integer, String>> rowData = new ArrayList<>();
                for (SubjectiveEvaluationTreeBO thirdChild : thirdChildren) {
                    columnData = new HashMap<>();
                    // 评分
                    columnData.put(3, thirdChild.getModelRatings());
                    // 评语
                    columnData.put(4, thirdChild.getModelReviews());
                    // 备注
                    columnData.put(5, thirdChild.getRemark());
                    rowData.add(columnData);
                }
                tableData.addAll(rowData);
            }
            allData.add(tableData);
        }

        // 图片数据
        Map<String, Object> params = new HashMap<>();
        String[] dataUrlList = subjectiveEvaluationReportsParam.getBase64();
        if (dataUrlList != null) {
            for (int i = 0; i < dataUrlList.length; i++) {
                // 将eCharts svg格式dataUrl转换为字符数组

                Map<String, Object> jpeg = new HashMap<>();
                jpeg.put("width", 550);
                jpeg.put("height", 400);
                jpeg.put("type", "jpg");
                jpeg.put("content", Base64.getDecoder().decode(subjectiveEvaluationReportsParam.getBase64()[i].replace("data:image/png;base64,", "")));
                String str = "test" + i;
                params.put(str, jpeg);
            }
        }
        // 将minio中的模板下载到本地
        InputStream inputStream = attachmentService.downloadAttachment(subjectiveEvaluationReportsParam.getObjName());
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String localTempPath = filePath + uuid + ".docx";
        FileUtil.downloadToLocal(inputStream, localTempPath);// 这里会自动关闭io流
        String fileName = uuid +
                SubjectiveEvaluationEnum.getNameByCode(subjectiveEvaluationReportsParam.getReviewType()) + "试验报告.docx";
        String outFilePath = filePath + fileName;

        // 将表格与图片插入试验报告中去
        WordUtil.changWord(localTempPath, outFilePath, params, allData);

        if(CollectionUtil.isNotEmpty(subjectiveEvaluationReportsParam.getIdMap())){
            this.getSubjectiveEvaluationReportsWord(
                    subjectiveEvaluationReportsParam.getCarId().intValue(),
                    subjectiveEvaluationReportsParam.getIdMap(),
                    outFilePath
            );
        }

        // 定义主观评价试验类别
        int business_type;
        switch (subjectiveEvaluationReportsParam.getReviewType()){
            case 1:
                business_type = 45;
                break;
            case 2:
                business_type = 46;
                break;
            case 3:
                business_type = 47;
                break;
            case 4:
                business_type = 48;
                break;
            case 5:
                business_type = 49;
                break;
            default:
                business_type = 0;
        }
        if (business_type == 0){
            throw new ToolsBusinessWarn("主观评价试验类别错误");
        }

        // 将本地试验报告上传至minio
        File file = new File(outFilePath);
        try (InputStream in = new FileInputStream(outFilePath)) {
            attachmentService.uploadAttachment(
                    in,
                    fileName,
                    file.length(),
                    "car_info",
                    subjectiveEvaluationReportsParam.getCarId(),
                    business_type);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // 删除本地试验报告文件与模板文件
        FileUtil.deleteFile(localTempPath);
        FileUtil.deleteFile(outFilePath);

    }

    private List<SubjectiveEvaluationTreeBO> getFirstLevelIndicatorList(List<SubjectiveEvaluationTreeBO> treeNodes) {
        List<SubjectiveEvaluationTreeBO> trees = new ArrayList<>();
        for (SubjectiveEvaluationTreeBO treeNode : treeNodes) {
            if (treeNode.getGrade() == 1) {
                trees.add(treeNode);
            }
            for (SubjectiveEvaluationTreeBO it : treeNodes) {
                if (Objects.equals(treeNode.getSerialNumber(), it.getBelongSerial())) {
                    if (treeNode.getChildren() == null || treeNode.getChildren().size() == 0) {
                        treeNode.setChildren(new ArrayList<>());
                    }
                    List<SubjectiveEvaluationTreeBO> treeNodeChildren = treeNode.getChildren();
                    treeNodeChildren.add(it);
                    treeNode.setChildren(treeNodeChildren);
                }
            }
        }
        return trees;
    }

    @Override
    public SubjectiveDataVO getSubjectiveDataListToCompare(SubjectiveIdDTO subjectiveIdDTO) {
        SubjectiveDataVO subjectiveDataVO = new SubjectiveDataVO();
        List<SubjectiveDataVO.SubjectiveDataSon> tableDeadList = new ArrayList<>();
        // 1、查询车辆idList查询所有对比数据
        List<Long> carIdList = new ArrayList<>(subjectiveIdDTO.getCarMap().keySet());
        LambdaQueryWrapper<SubjectiveEvaluation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.in(SubjectiveEvaluation::getCarId,carIdList);
        List<SubjectiveEvaluation> subjectiveEvaluationList = this.list(queryWrapper);

        // 2、封装表头
        Map<Long, List<SubjectiveEvaluation>> map = subjectiveEvaluationList.stream().collect(Collectors.groupingBy(SubjectiveEvaluation::getCarId));
        List<SubjectiveEvaluation> subjectiveDataList = map.get(carIdList.get(0));
        subjectiveDataList.forEach(subjectiveEvaluation -> {
            SubjectiveDataVO.SubjectiveDataSon subjectiveDataSon = new SubjectiveDataVO.SubjectiveDataSon();
            subjectiveDataSon.setGrade(subjectiveEvaluation.getGrade());
            subjectiveDataSon.setTestName(subjectiveEvaluation.getTestName());
            SubjectiveDataVO.SubjectiveDataSon2 subjectiveDataSon2 = new SubjectiveDataVO.SubjectiveDataSon2();
            List<SubjectiveDataVO.SubjectiveDataSon2> SubjectiveDataSon2List = new ArrayList<>();
            SubjectiveDataSon2List.add(subjectiveDataSon2);
            subjectiveDataSon.setSubjectiveDataSon2List(SubjectiveDataSon2List);
            tableDeadList.add(subjectiveDataSon);
        });
        subjectiveDataVO.setTableDeadList(tableDeadList);

        // 3、封装对比数据集合
        Map<Long, Map<String, List<String>>> data = new HashMap<>();
        for (Long carId : carIdList) {
            List<SubjectiveEvaluation> paramList = map.get(carId);
            if (CollectionUtil.isEmpty(paramList)) {
                continue;
            }
            Map<String, List<String>> paramMap = new HashMap<>();
            for (SubjectiveEvaluation subjectiveEvaluation : paramList) {
                List<String> list = new ArrayList<>();
                list.add(subjectiveEvaluation.getModelRatings());
                list.add(subjectiveEvaluation.getModelReviews());
                if (StringUtils.isNotEmpty(subjectiveEvaluation.getTestName()))
                    paramMap.put(subjectiveEvaluation.getTestName(), list);
            }
            data.put(carId, paramMap);
        }
        subjectiveDataVO.setData(data);
        return subjectiveDataVO;
    }

    @Override
    public Map<Integer, List<GetSubjectiveEvaluationListVO>> getSubjectiveEvaluationLists(SubjectiveEvaluationsParam subjectiveEvaluationsParam) {

        // 1、查询该车该评价体系所有人主观评价数据
        List<SubjectiveEvaluationDTO> subjectiveEvaluationList = subjectiveEvaluationMapper.getSubjectiveEvaluationLists(subjectiveEvaluationsParam);

        // 2、通过车辆id进行分组，将每组数据封装为一个List
        Map<Integer, List<GetSubjectiveEvaluationListVO>> result = new HashMap<>();
        Map<Integer, List<SubjectiveEvaluationDTO>> groupByCarIdMap = subjectiveEvaluationList.stream()
                .collect(Collectors.groupingBy(SubjectiveEvaluationDTO::getCarId));
        for (Integer carId : groupByCarIdMap.keySet()) {
            List<SubjectiveEvaluationDTO> groupByCarIdList = groupByCarIdMap.get(carId);
            List<GetSubjectiveEvaluationListVO> currentCarList = new ArrayList<>();
            // 3、通过姓名进行分组，将每组数据封装为一个List
            Map<String, List<SubjectiveEvaluationDTO>> groupMap = groupByCarIdList.stream()
                    .collect(Collectors.groupingBy(SubjectiveEvaluationDTO::getEvaluatorName));
            for (String key : groupMap.keySet()) {
                List<SubjectiveEvaluationDTO> groupSubjectiveEvaluation = groupMap.get(key);
                // 两层循环将List封装为树
                GetSubjectiveEvaluationListVO getSubjectiveEvaluationListVO = new GetSubjectiveEvaluationListVO();
                getSubjectiveEvaluationListVO.setEvaluatorName(key);
                getSubjectiveEvaluationListVO.setSubjectiveEvaluationDTOList(this.convertMenuTree(groupSubjectiveEvaluation));
                currentCarList.add(getSubjectiveEvaluationListVO);
            }
            result.put(carId, currentCarList);
        }
        return result;
    }

    @Override
    public void deleteEvaluationData(DeleteEvaluationDataDTO dto) {
        // 删除minio与数据库附件信息
        attachmentService.removeById(dto.getAttachId());

        // 删除主观评价数据
        LambdaQueryWrapper<SubjectiveEvaluation> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(SubjectiveEvaluation::getAttachId, dto.getAttachId());
        this.remove(queryWrapper);

        // 删除试验场景数据
        LambdaQueryWrapper<SubjectiveScene> queryWrapper2 = Wrappers.lambdaQuery();
        queryWrapper2.eq(SubjectiveScene::getAttachId, dto.getAttachId());
        subjectiveSceneService.remove(queryWrapper2);
    }

    /**
     * 查询评价人列表
     */
    @Override
    public QueryEvaluatorNameListVO queryEvaluatorNameList(QueryEvaluatorNameListDTO dto) {
        QueryEvaluatorNameListVO result = new QueryEvaluatorNameListVO();
        Map<Long, List<String>> map = new HashMap<>();
        // 1、根据车辆idList查询所有评价人姓名
        List<QueryEvaluatorNameListBO> subjectiveEvaluationList = subjectiveEvaluationMapper.queryEvaluatorNameList(dto);

        // 2、通过车辆id分组
        Map<Long, List<QueryEvaluatorNameListBO>> groupMap = subjectiveEvaluationList.stream()
                .collect(Collectors.groupingBy(QueryEvaluatorNameListBO::getCarId));

        for (Long key : groupMap.keySet()) {
            List<QueryEvaluatorNameListBO> currentCarIdList = groupMap.get(key);
            List<String> evaluatorNameList = currentCarIdList.stream().map(QueryEvaluatorNameListBO::getEvaluationName)
                    .collect(Collectors.toList());
            map.put(key, evaluatorNameList);
        }
        result.setMap(map);
        return result;
    }

    @Override
    @Transactional
    @SneakyThrows
    public void uploadNVHSubjectiveFile(UploadAttachmentDTO dto) {
        log.info("读取NVH关键指标数据文件");
        Attachment attachment = attachmentService.uploadAttachment(
                dto.getMultipartFile(),
                AttachmentConstant.OWNER_TYPE_CAR,
                dto.getOwnerId(),
                dto.getBusinessType(),
                dto.getOperId()
        );
        if(dto.getBusinessType() == 21){
            // 读取电动及混动车关键指标数据,并保存至数据库，用作数据对比使用
            NVHSubjectiveElectricListener electricListener = new NVHSubjectiveElectricListener(attachment.getId());
            EasyExcel.read(dto.getMultipartFile().getInputStream(), electricListener)
                    .headRowNumber(14).sheet().doRead();
            nvhEvaluationService.saveBatch(electricListener.getNvhEvaluationList());

            // 读取电动及混动车绘图数据
            ChartDataModel chartDataModel = new ChartDataModel();
            List<String> xData = electricListener.getXData();
            List<Double> yData = electricListener.getYData();
            chartDataModel.setXData2(xData.toArray(new String[0]));
            Double[][] yData2 = new Double[1][];
            yData2[0] = yData.toArray(new Double[0]);
            chartDataModel.setYData2(yData2);
            chartDataModel.setTitle(attachment.getName());
            NVHChartData nvhChartData = new NVHChartData();
            nvhChartData.setAttachId(attachment.getId().intValue());
            nvhChartData.setCarId(dto.getOwnerId().intValue());
            nvhChartData.setBusinessId(attachment.getBusinessType());
            nvhChartData.setChartData(FileUtil.objectToByte(chartDataModel));
            nvhChartDataMapper.insert(nvhChartData);
        }
        if(dto.getBusinessType() == 20){
            // 读取燃油车关键指标数据
            NVHSubjectiveFuelListener fuelListener = new NVHSubjectiveFuelListener(attachment.getId());
            EasyExcel.read(dto.getMultipartFile().getInputStream(), fuelListener)
                    .headRowNumber(32).sheet().doRead();
            nvhEvaluationService.saveBatch(fuelListener.getNvhEvaluationList());

            // 读取电动车绘图数据
            ChartDataModel chartDataModel = new ChartDataModel();
            List<String> xData = fuelListener.getXData();
            List<Double> yData = fuelListener.getYData();
            chartDataModel.setXData2(xData.toArray(new String[0]));
            Double[][] yData2 = new Double[1][];
            yData2[0] = yData.toArray(new Double[0]);
            chartDataModel.setYData2(yData2);
            chartDataModel.setTitle(attachment.getName());
            NVHChartData nvhChartData = new NVHChartData();
            nvhChartData.setAttachId(attachment.getId().intValue());
            nvhChartData.setCarId(dto.getOwnerId().intValue());
            nvhChartData.setBusinessId(attachment.getBusinessType());
            nvhChartData.setChartData(FileUtil.objectToByte(chartDataModel));
            nvhChartDataMapper.insert(nvhChartData);
        }

        // 修改文件上传状态
        CarBasicParam carBasicParam = new CarBasicParam();
        carBasicParam.setNvhSubjectiveStat(2);
        carBasicParam.setId(dto.getOwnerId());
        carBasicParamService.updateById(carBasicParam);
    }

    public void getSubjectiveEvaluationReportsWord(Integer carId,Map<Integer,List<Integer>> idMap,String templatePath) {
        //1.组装数据
        SubjectiveEvaluationReportsWordParam table = assertData(carId,idMap);
        //2.解析模板
        XWPFTemplate template = XWPFTemplate.compile(templatePath);
        // 3.封装模型数据
        HashMap<String, Object> map = new HashMap<>();

//        4.创建车辆基本信息表
        // 创建多行数据
        map.put("carBasicTitle", Texts.of("车辆基本参数信息表").color("000000").bold().create());
        List<RowRenderData> rows0 = new ArrayList<>();

        for (int i = 0; i < table.getCarBasicColumns().size() - 1; i+=2) {
            RowRenderData rowRenderData = Rows.of(
                    table.getCarBasicColumns().get(i).getChineseName(),
                    table.getCarBasicColumns().get(i).getValue(),
                    table.getCarBasicColumns().get(i + 1).getChineseName(),
                    table.getCarBasicColumns().get(i + 1).getValue()
            ).center().create();
            rows0.add(rowRenderData);
        }

        // 将多行数据添加到表格
        map.put("table0", Tables.create(rows0.toArray(new RowRenderData[0])));

//        创建车身基本信息表
        // 创建多行数据
        map.put("carBodyTitle", Texts.of("车辆车身参数信息表").color("000000").bold().create());
        List<RowRenderData> rows1 = new ArrayList<>();

        for (int i = 0; i < table.getCarBodyColumns().size() - 1; i+=2) {
            RowRenderData rowRenderData = Rows.of(
                    table.getCarBodyColumns().get(i).getChineseName(),
                    table.getCarBodyColumns().get(i).getValue(),
                    table.getCarBodyColumns().get(i + 1).getChineseName(),
                    table.getCarBodyColumns().get(i + 1).getValue()
            ).center().create();
            rows1.add(rowRenderData);
        }

        // 将多行数据添加到表格
        map.put("table1", Tables.create(rows1.toArray(new RowRenderData[0])));

//        创建车辆底盘表
        // 创建多行数据
        map.put("carChassisTitle", Texts.of("车辆底盘结构信息表").color("000000").bold().create());
        List<RowRenderData> rows2 = new ArrayList<>();

        for (int i = 0; i < table.getCarChassisColumns().size() - 1; i+=2) {
            RowRenderData rowRenderData = Rows.of(
                    table.getCarChassisColumns().get(i).getChineseName(),
                    table.getCarChassisColumns().get(i).getValue(),
                    table.getCarChassisColumns().get(i + 1).getChineseName(),
                    table.getCarChassisColumns().get(i + 1).getValue()
            ).center().create();
            rows2.add(rowRenderData);
        }

        // 将多行数据添加到表格
        map.put("table2", Tables.create(rows2.toArray(new RowRenderData[0])));

        // 渲染数据
        template.render(map);
        //以文件形式输出
        try {
            template.writeAndClose(new FileOutputStream(templatePath));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

//    获取返回word的信息
    private SubjectiveEvaluationReportsWordParam assertData(Integer carId,Map<Integer,List<Integer>> idMap) {
        SubjectiveEvaluationReportsWordParam subjectiveEvaluationReportsWordParam = new SubjectiveEvaluationReportsWordParam();
        subjectiveEvaluationReportsWordParam.setTitle("主观评价试验报告");
        List<Integer> basicIdList = new ArrayList<>();
        List<Integer> bodyIdList = new ArrayList<>();
        List<Integer> chassisIdList = new ArrayList<>();
//        读取所要查询的字段
        for (Integer key : idMap.keySet()) {
            switch (key){
                case 0:
                    basicIdList = idMap.get(key);
                    break;
                case 1:
                    bodyIdList = idMap.get(key);
                    break;
                case 2:
                    chassisIdList = idMap.get(key);
                    break;
            }
        }
//        查询要返回word文档的值（车辆基本参数）
        if (basicIdList.size() > 0){
//            通过id查询所要显示在word的字段
            QueryWrapper<CarColumn> columnQW = new QueryWrapper<>();
            columnQW.select("name","chinese_name");
            columnQW.in("id",basicIdList);
            List<CarColumn> carColumnList = carColumnService.list(columnQW);
            String[] columnNames = carColumnList.stream().map(CarColumn::getName).toArray(String[]::new);

//            字段所对应的值
            QueryWrapper<CarBasicParam> basicQW = new QueryWrapper<>();
            basicQW.select(columnNames);
            basicQW.eq("id",carId);
            CarBasicParam carBasicParam = carBasicParamService.getOne(basicQW);

            // 通过反射获取字段值
            this.setColumnValue(carColumnList,carBasicParam);

            subjectiveEvaluationReportsWordParam.setCarBasicColumns(carColumnList);
        }

//        查询要返回word文档的值（车身基本参数）
        if (bodyIdList.size() > 0){
//            通过id查询所要显示在word的字段
            QueryWrapper<CarColumn> columnQW = new QueryWrapper<>();
            columnQW.select("name","chinese_name");
            columnQW.in("id",bodyIdList);
            List<CarColumn> carColumnList = carColumnService.list(columnQW);
            String[] columnNames = carColumnList.stream().map(CarColumn::getName).toArray(String[]::new);

//            字段所对应的值
            QueryWrapper<CarBodyParam> bodyQW = new QueryWrapper<>();
            bodyQW.select(columnNames);
            bodyQW.eq("car_id",carId);
            CarBodyParam carBodyParam = carBodyParamService.getOne(bodyQW);

            // 通过反射获取字段值
            this.setColumnValue(carColumnList,carBodyParam);
            subjectiveEvaluationReportsWordParam.setCarBodyColumns(carColumnList);
        }

//        查询要返回word文档的值（底盘结构）
        if (chassisIdList.size() > 0){
//            通过id查询所要显示在word的字段
            QueryWrapper<CarColumn> columnQW = new QueryWrapper<>();
            columnQW.select("name","chinese_name");
            columnQW.in("id",chassisIdList);
            List<CarColumn> carColumnList = carColumnService.list(columnQW);
            String[] columnNames = carColumnList.stream().map(CarColumn::getName).toArray(String[]::new);

//            字段所对应的值
            QueryWrapper<CarChassisStructure> chassisQW = new QueryWrapper<>();
            chassisQW.select(columnNames);
            chassisQW.eq("car_id",carId);
            CarChassisStructure carChassisStructure = carChassisStructureService.getOne(chassisQW);

            // 通过反射获取字段值
            this.setColumnValue(carColumnList,carChassisStructure);
            subjectiveEvaluationReportsWordParam.setCarChassisColumns(carColumnList);
        }
        return subjectiveEvaluationReportsWordParam;
    }

    private void setColumnValue(List<CarColumn> carColumnList,Object object){
        for (CarColumn carColumn : carColumnList) {
            String columnName = getHumpNaming(carColumn.getName());
            if(StringUtils.isEmpty(columnName)){
                carColumn.setValue("");
                continue;
            }
            String columnValue = getChineseValue(object.getClass(), columnName,object);
            carColumn.setValue(columnValue);
        }
    }


//    获取驼峰命名
    public static String getHumpNaming(String name){
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < name.length(); i++) {
            char c = name.charAt(i);
            if (c == '_') {
                sb.append(Character.toUpperCase(name.charAt(i + 1)));
                i++;
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }

    public static String getChineseValue(Class<?> clazz,String value,Object T){
            Field[] fields = clazz.getDeclaredFields();
            for (Field field : fields) {
                if (field.isAnnotationPresent(ApiModelProperty.class) && field.getName().equals(value)) {
                    // 设置为可访问以读取私有字段
                        field.setAccessible(true);
                        // 获取字段的值
                        try {
                            if(ObjectUtil.isEmpty(field.get(T)))return "";
                            return field.get(T).toString();
                        } catch (IllegalAccessException e) {
                            throw new RuntimeException();
                        }

                }
            }
            return null;
    }
}














