package com.singularity.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.singularity.dao.TableHeadMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.pojo.dto.fieldManagement.QueryTableHeadDTO;
import com.singularity.pojo.entity.HeadField;
import com.singularity.pojo.entity.TableData;
import com.singularity.pojo.entity.TableHead;
import com.singularity.service.HeadFieldService;
import com.singularity.service.TableDataService;
import com.singularity.service.TableHeadService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class TableHeadServiceImpl extends ServiceImpl<TableHeadMapper, TableHead> implements TableHeadService {
    @Resource
    private TableDataService tableDataService;

    @Resource
    private HeadFieldService headFieldService;
    @Override
    public List<TableHead> queryForm(QueryTableHeadDTO queryTableHeadDTO) {
        LambdaQueryWrapper<TableHead> qw = new LambdaQueryWrapper<>();
        qw.like(StringUtils.isNotEmpty(queryTableHeadDTO.getTableName()),TableHead::getTableName,queryTableHeadDTO.getTableName())
                .like(StringUtils.isNotEmpty(queryTableHeadDTO.getSheetName()),TableHead::getSheetName,queryTableHeadDTO.getSheetName())
                .eq(queryTableHeadDTO.getStat() != null,TableHead::getStat,queryTableHeadDTO.getStat())
                .ge(queryTableHeadDTO.getStartTime() != null,TableHead::getCreateTime,queryTableHeadDTO.getStartTime())
                .le(queryTableHeadDTO.getEndTime() != null,TableHead::getCreateTime,queryTableHeadDTO.getEndTime());
        return this.list(qw);
    }

    @Override
    public void deleteForm(List<TableHead> tableHeads) {
        List<Integer> idList = tableHeads
                .stream()
                .map(TableHead::getId)
                .collect(Collectors.toList());
        LambdaQueryWrapper<TableData> qw = new LambdaQueryWrapper<>();
        qw.in(TableData::getTableHeadId,idList);
        List<TableData> list = tableDataService.list(qw);
        if (list.size() == 0){
            this.removeBatchByIds(tableHeads);
            LambdaUpdateWrapper<HeadField> qwh = new LambdaUpdateWrapper<>();
            qwh.in(HeadField::getHeadId,idList);
            headFieldService.remove(qwh);
        }else {
            throw new ToolsBusinessWarn("您所选的表中还有数据，不能删除");
        }
    }

    @Override
    public void saveDatas(TableHead tableHead) {
        Date date = new Date();
        tableHead.setCreateTime(date);
        tableHead.setUpdateTime(date);
        tableHead.setStat(true);
        this.save(tableHead);
    }
}
