package com.singularity.service.impl;

import com.singularity.dao.VehicleDataMapper;
import com.singularity.pojo.dto.car.CarTypeDTO;
import com.singularity.pojo.enumeration.CarInfoEnum;
import com.singularity.pojo.vo.car.CarMessageNumberVO;
import com.singularity.pojo.vo.car.CarSingleMessageVO;
import com.singularity.service.VehicleDataService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.stream.Collectors;

@Service
public class VehicleDataServiceImpl implements VehicleDataService {

    @Resource
    private VehicleDataMapper vehicleDataMapper;

    @Override
    public CarMessageNumberVO selectNumberByType(CarTypeDTO carTypeDTO) {
        CarMessageNumberVO carMessageNumberVO = new CarMessageNumberVO();

//        年份
        carMessageNumberVO.setYearMap(vehicleDataMapper.selectYearNumber(carTypeDTO)
                .stream().collect(Collectors.toMap(
                        CarSingleMessageVO::getYearType, CarSingleMessageVO::getYearNums
                )));

//        级别
        carMessageNumberVO.setLevelMap(vehicleDataMapper.selectLevelNumber(carTypeDTO)
                .stream().collect(Collectors.toMap(
                        CarSingleMessageVO::getLevel, CarSingleMessageVO::getLevelNums
                )));

//        类别
        carMessageNumberVO.setCategoryMap(
                vehicleDataMapper.selectCategoryNumber(carTypeDTO).stream()
                        .collect(
                                Collectors.toMap(
                                        carSingleMessageVO-> CarInfoEnum.CategoryEnum.getNameByCode(carSingleMessageVO.getCategory()),
                                        CarSingleMessageVO::getCategoryNums
                                )
                        )
        );

//        国别
        carMessageNumberVO.setCountryMap(vehicleDataMapper.selectCountryNumber(carTypeDTO)
                .stream().collect(Collectors.toMap(s->
                        CarInfoEnum.ProductionPlaceEnum.getNameByCode(s.getProductionPlace()),CarSingleMessageVO::getProductionPlaceNums
                )));

//        能源
        carMessageNumberVO.setEnergyMap(vehicleDataMapper.selectEnergyNumber(carTypeDTO)
                .stream().collect(Collectors.toMap(s->
                        CarInfoEnum.CategoryEnum.getNameByCode(s.getEnergy()),CarSingleMessageVO::getEnergyNums
                )));
        return carMessageNumberVO;
    }
}
