package com.singularity.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.dao.CarDataMapper;
import com.singularity.dao.CarEnergyParamMapper;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.pojo.dto.kc.CarDataDTO;
import com.singularity.pojo.dto.kc.CarFileDTO;
import com.singularity.pojo.dto.vehicle.VehicleInfoDTO;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.entity.CarBasicParam;
import com.singularity.pojo.entity.CarEnergyParam;
import com.singularity.pojo.param.kc.CarDataParam;
import com.singularity.service.AttachmentService;
import com.singularity.service.VehicleInfoService;
import com.singularity.util.page.PageUtil;
import com.singularity.util.page.ResponseDTOWithPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class VehicleInfoServiceImpl implements VehicleInfoService {

    @Resource
    private CarDataMapper carDataMapper;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private CarEnergyParamMapper carEnergyParamMapper;

    @Override
    public ResponseDTOWithPage<CarDataDTO> getCarDataList(CarDataParam carDataParam) {
        PageUtil.startPage(carDataParam, true, CarDataDTO.class);
        List<CarDataDTO> carData = carDataMapper.select(carDataParam);
        return PageUtil.getListDTO(carData, carDataParam);
    }

    @Override
    @Transactional
    public void deleteCarData(List<CarDataParam> carDataParam) {
        LambdaUpdateWrapper<Attachment> qw = new LambdaUpdateWrapper<>();
        for (int i = 0; i < carDataParam.size(); i++) {
            qw.eq(Attachment::getOwnerId, carDataParam.get(i).getId())
                    .eq(Attachment::getOwnerType,"car_info");
            if (!attachmentService.list(qw).isEmpty()) {
                throw new ToolsBusinessWarn("当前所选车辆均有附件没有删除，请将附件删除后继续");
            }
        }
        carDataMapper.deleteCarBasicParam(carDataParam);
        carDataMapper.deleteCarBodyParam(carDataParam);
        carDataMapper.deleteCarChassisStructure(carDataParam);
        LambdaUpdateWrapper<Attachment> uw = new LambdaUpdateWrapper<>();
        uw.in(Attachment::getOwnerId, carDataParam.stream().map(CarDataParam::getId).collect(Collectors.toList()))
                .eq(Attachment::getOwnerType, "car_info");
        attachmentService.remove(uw);
    }

    @Override
    public void setCarData(VehicleInfoDTO vehicleInfoDTO) {
        if (carDataMapper.selectCarTestId(vehicleInfoDTO) == null) {
            carDataMapper.addCarData(vehicleInfoDTO);
        } else {
            carDataMapper.setCarData(vehicleInfoDTO);
        }
    }

    @Override
    @Transactional
    public void saveCarData(CarFileDTO carDataDTO) {
        // 校验VIN是否重复
        // 去除所有空格
        String vin = StringUtils.deleteWhitespace(carDataDTO.getVIN());
        LambdaQueryWrapper<CarBasicParam> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(CarBasicParam::getVin,vin);
        if(carDataMapper.selectCount(queryWrapper) > 0)
            throw new ToolsBusinessWarn("VIN不能重复");
        List<CarFileDTO> carFileDTOS = new ArrayList<>();
        carFileDTOS.add(carDataDTO);
        carDataMapper.saveBasicParam(carFileDTOS);
        carDataMapper.saveBodyParam(carFileDTOS);
        carDataMapper.saveStructureParam(carFileDTOS);
        // 保存车辆能源信息
        CarEnergyParam carEnergyParam = new CarEnergyParam();
        carEnergyParam.setCarId(carDataDTO.getId());
        carEnergyParamMapper.insert(carEnergyParam);
    }
}
