package com.singularity.util;

import cn.dev33.satoken.exception.SaTokenException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.concurrent.ScheduledFuture;

@Component
@EnableScheduling
@Slf4j
public class TimedTaskUtil {

    /**
     * 当前的任务调度器
     */
    private ScheduledFuture<?> scheduledFuture;

    /**
     * 注入线程池任务调度类
     */
    @Resource
    private ThreadPoolTaskScheduler threadPoolTaskScheduler;

    /**
     * 指定时间执行任务
     * 注意：当传入时间为过去时间是会立刻执行
     *
     * @param task 任务
     * @param startTime 执行时间
     */
    public void startCron(Runnable task, Date startTime) {
        try {
            scheduledFuture = threadPoolTaskScheduler.schedule(task, startTime);
        }catch (Exception e){
            throw new SaTokenException("执行失败");
        }
    }

    /**
     * 没间隔执行时间执行定时任务
     *
     * @param task 任务
     * @param period 时间间隔毫秒
     */
    public void startCircleCron(Runnable task,long period) {
        try {
            scheduledFuture = threadPoolTaskScheduler.scheduleAtFixedRate(task,period);
        }catch (Exception e){
            throw new SaTokenException("执行失败");
        }
    }


    /**
     * 任务调度器不为null且有任务在执行
     */
    public void stopCron() {
        if (scheduledFuture != null) {
            if (scheduledFuture.isDone()) {
                scheduledFuture.cancel(true);
                log.info("old notify task end...");
            }
        }
    }

    /**
     * 先停止之前的任务，再执行新任务
     */
    public void startCronByStop(Runnable task, Date startTime) {
        //停止执行之前的任务
        stopCron();
        try {
            // 添加定时任务线程
            log.info("new notify task start...");
            scheduledFuture = threadPoolTaskScheduler.schedule(task, startTime);
        }catch (Exception e){
            log.error("notify push fail:",e);
        }
    }

    public void startCron(Object o) {
    }
}

